const BugTicket = require('../Model/BugTicket');
const path = require('path');
const fs = require('fs');
const moment = require('moment');
const utilitiesController = require('./Utilities.controller');
const base64toFile = require('../Utils/Base64ToFile');
const BugTicketServices = require('../Services/CreateServices')("BugTicket");

module.exports.getAll = async (req, res) => {
    try {
        const user = req.local.user;
        let query = {};

        if (["ADMIN", "ORG_ADMIN", "OWNER", "VALUER", "FRONTDESK"].indexOf(user?.role) < 0) {
            query = { userId: user._id }
        }

        const createTicketRes = await BugTicketServices.find({ query, populate: ["createdBy"], start: req.local.startIndex, limit: req.local.limit });
        if (!createTicketRes.status || createTicketRes.error) return res.status(500).send(createTicketRes);

        return res.status(200).send(createTicketRes);
    } catch (e) {
        console.log("🚀 ~ file: BugTicket.controller.js:11 ~ module.exports.createTicket= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}

module.exports.get = async (req, res) => {
    try {
        const id = req.params.id;

        const createTicketRes = await BugTicketServices.find({ query: { _id: id }, populate: ["createdBy"] });
        if (!createTicketRes.status || createTicketRes.error) return res.status(500).send(createTicketRes);
        if (createTicketRes.data.length == 0) return res.status(404).send({ status: false, error: "No Ticket Found!" })
        return res.status(200).send({ status: true, data: createTicketRes.data });
    } catch (e) {
        console.log("🚀 ~ file: BugTicket.controller.js:25 ~ module.exports.createTicket= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}

module.exports.createTicket = async (req, res) => {
    try {
        const user = req.local.user;
        let data = req.local.bugTicket;

        data.organizationId = user.organization._id;
        data.userId = user._id;
        data.createdBy = user._id;

        data.images = req.files['images'];
        let images = [];
        const audioNotes = data['audioNotes'];
        delete data['audioNotes'];
        console.log("🚀 ~ file: BugTicket.controller.js:42 ~ module.exports.createTicket= ~ data:", data)

        const maxNumber = await BugTicketServices.find({ limit: 1, sort: '-ticketNo' });
        data.ticketNo = (maxNumber.count > 0) ? (maxNumber.data[0].ticketNo + 1) : 1;

        if (data.images) {
            const images = data.images;
            if (images) {
                images.map(img => {
                    if (!img.id) {
                        const img_id = Math.floor(100000 + Math.random() * 900000);
                        img.id = img_id;
                        img.uploadedBy = req.local.user._id;
                        img.createdAt = new Date()
                    }
                })
            }
        }

        const createTicketRes = await BugTicketServices.create(data);
        if (!createTicketRes.status || createTicketRes.error) return res.status(500).send(createTicketRes);

        const id = createTicketRes.data._id.toString();

        if (createTicketRes.data.images.length > 0) {
            const targetFolderPath = path.join(process.env.CDN_PATH, 'bugTicket', id);
            fs.mkdirSync(targetFolderPath, { recursive: true });

            await Promise.all(createTicketRes.data.images.map(async file => {
                const currentPath = path.join(process.cwd(), file.path);
                const fileName = 'img_' + file.filename;
                const destination = path.join(targetFolderPath, fileName);

                console.log("🚀 ~ file: BugTicket.controller.js:80 ~ moveFilePromises ~ currentPath, destination:", currentPath, destination)
                await utilitiesController.moveFile(currentPath, destination);

                const filepath = path.join('bugTicket', id, fileName);
                images.push({
                    ...file,
                    fileName: fileName,
                    path: filepath
                })
            }));
            createTicketRes.data.images = images;
        }
        if (audioNotes.length > 0) {
            const targetDirectory = path.join('bugTicket', id, 'audioNotes');
            let audioFiles = [];
            await Promise.all(audioNotes.map(async (base64) => {

                const fileName = 'audio_' + moment(new Date()).format("dd_mm_yyyy_hh_mm_ss") + ".m4a";
                const { status, filepath, error } = await base64toFile(base64, fileName, targetDirectory)
                if (status) {
                    audioFiles.push({
                        id: Math.floor(100000 + Math.random() * 900000),
                        fileName: fileName,
                        path: filepath,
                        uploadedBy: req.local.user._id,
                        createdAt: new Date(),
                    })
                }
            }));
            createTicketRes.data.audioNotes = audioFiles;

        }
        const result = await createTicketRes.data.save();
        return res.status(200).send({ status: true, data: createTicketRes.data });

    } catch (e) {
        console.log("🚀 ~ file: BugTicket.controller.js:39 ~ module.exports.createTicket= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}

module.exports.updateTicket = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        let data = req.local.bugTicket;

        if (!!data.log) {
            data = {
                ...data,
                $push: {
                    'logs': {
                        log: data.log,
                        createdBy: user,
                        createdAt: new Date()
                    }
                }
            };
            delete data.log;
        }

        const updateTicketRes = await BugTicketServices.updateById(id, data);
        if (!updateTicketRes.status || updateTicketRes.error) return res.status(500).send(updateTicketRes);

        return res.status(200).send(updateTicketRes);
    } catch (e) {
        console.log("🚀 ~ file: BugTicket.controller.js:54 ~ module.exports.updateTicket= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}