const fs = require('fs');
const path = require('path');
const moment = require("moment");
const _ = require('loadsh');

const Case = require('../Model/Case');
const Utility = require('../Utils/Utility');

const CONFIG = require('../Constants/Config');
const PdfDoc = require('../Middleware/pdfkit-tables');

const DocumentServices = require('../Services/DocumentList.services');
const CaseServices = require('../Services/Case.services');
const UserServices = require('../Services/User.services');
const OrganizationServices = require('../Services/Organization.services');
const ClientServices = require('../Services/Client.services');
const BranchServices = require('../Services/Branch.services');
const ZoneServices = require('../Services/Zone.services');
const EmpanelmentServices = require('../Services/CreateServices')('Empanelment');
const CaseLogServices = require('../Services/CaseLog.services');
const utilitiesController = require('../Controllers/Utilities.controller');
const TaskServices = require('../Services/Task.services');
const nInvoiceServices = require('../Services/CreateServices')('nInvoice');
const MailLogServices = require('../Services/CreateServices')('MailLog');
const PaymentServices = require('../Services/Payment.services');
const CreditOfficersServices = require('../Services/CreateServices')('CreditOfficer');
const { ObjectId, ObjectID } = require('mongodb');

const Task = require('../Model/Task');
const { default: axios } = require('axios');
const { generatePDF } = require('../Middleware/pdfGenerate');
const { logger } = require('../logger');

Array.prototype.unique = function () {
    var a = this.concat();
    for (var i = 0; i < a.length; ++i) {
        for (var j = i + 1; j < a.length; ++j) {
            if (a[i] === a[j])
                a.splice(j--, 1);
        }
    }
    return a;
};

module.exports.dashboard = async (req, res) => {
    try {
        let query = {};
        const id = req.local.user._id;
        const role = req.local.user.role;

        // query['organization._id'] = req.local.user.organization._id;

        if (!['VALUER', 'ADMIN', 'FRONTDESK', 'PRINTING', 'ACCOUNTANT'].includes(role)) {
            query['$or'] = [
                { "createdBy": id },
                { "staff.pickupPerson._id": id },
                { "staff.maker._id": id },
                { "staff.checker._id": id },
                { "staff.valuers._id": id },
                { "staff.rateVerifier._id": id },
                { "staff.siteEngineer._id": id }
            ]
        }
        // 1. Lead
        const leadCount = await CaseServices.getCount({ caseStatus: "LEAD", type: "LEAD", ...query });
        if (!leadCount.status || leadCount.error) return res.status(500).send(leadCount);

        // 2. New
        const newCount = await CaseServices.getCount({ caseStatus: "NEW", type: "CASE", ...query });
        if (!newCount.status || newCount.error) return res.status(500).send(newCount);

        //  3 Document Verified
        const documentVerifiedCount = await CaseServices.getCount({ "verifiedDocument.value": true, type: "LEAD", caseStatus: "DOCUMENTVERIFIED", ...query });
        if (!documentVerifiedCount.status || documentVerifiedCount.error) return res.status(500).send(documentVerifiedCount);

        // 4. Remaining Site VISIT / Site Visit Assigned
        const remainingSiteVisitCount = await CaseServices.getCount({ type: "CASE", caseStatus: "REMAININGSITEVISIT", ...query });
        if (!remainingSiteVisitCount.status || remainingSiteVisitCount.error) return res.status(500).send(remainingSiteVisitCount);

        // 5. Rate Finalized
        const rateFinalizedCount = await CaseServices.getCount({ type: "CASE", caseStatus: "RATEFINALIZED", ...query });
        if (!rateFinalizedCount.status || rateFinalizedCount.error) return res.status(500).send(rateFinalizedCount);

        // 6. Site Visit Completed
        const siteVisitCompletedCount = await CaseServices.getCount({ type: "CASE", "siteVisit.siteVisitStatus": true, caseStatus: "SITEVISITCOMPLETED", ...query });
        if (!siteVisitCompletedCount.status || siteVisitCompletedCount.error) return res.status(500).send(siteVisitCompletedCount);

        // 7. ON Hold
        const onHoldCount = await CaseServices.getCount({ type: "CASE", caseStatus: "ONHOLD", ...query });
        if (!onHoldCount.status || onHoldCount.error) return res.status(500).send(onHoldCount);

        // 8. Ready For Maker
        const readyForMakerCount = await CaseServices.getCount({ type: "CASE", caseStatus: "READYFORMAKER", ...query });
        if (!readyForMakerCount.status || readyForMakerCount.error) return res.status(500).send(readyForMakerCount);

        // 9. Ready For Checker
        const readyForCheckerCount = await CaseServices.getCount({ type: "CASE", caseStatus: "READYFORCHECKER", ...query });
        if (!readyForCheckerCount.status || readyForCheckerCount.error) return res.status(500).send(readyForCheckerCount);

        // 10. Prepare Invoice / Report Generated
        const reportGeneratedCount = await CaseServices.getCount({ type: "CASE", caseStatus: "REPORTGENERATED", ...query });
        if (!reportGeneratedCount.status || reportGeneratedCount.error) return res.status(500).send(reportGeneratedCount);

        // 11. Attach Signed Report
        const attachSignCount = await CaseServices.getCount({ type: "CASE", caseStatus: "ATTACHSIGN", ...query });
        if (!attachSignCount.status || attachSignCount.error) return res.status(500).send(signedReportAttachCount);

        // 12. Signed Report Attached
        const signedReportAttachCount = await CaseServices.getCount({ type: "CASE", caseStatus: "SIGNEDREPORTATTACH", ...query });
        if (!signedReportAttachCount.status || signedReportAttachCount.error) return res.status(500).send(signedReportAttachCount);

        // READY TO PRINT - after rate finalized
        const readyForPrintCount = await CaseServices.getCount({ type: "CASE", caseStatus: "READYFORPRINT", ...query });
        if (!readyForPrintCount.status || readyForPrintCount.error) return res.status(500).send(readyForPrintCount);

        // 13. 
        const sentToDeliveryCount = await CaseServices.getCount({ type: "CASE", caseStatus: "SENTTODELIVERY", ...query });
        if (!sentToDeliveryCount.status || sentToDeliveryCount.error) return res.status(500).send(sentToDeliveryCount);

        //  14. 
        const deliveredCount = await CaseServices.getCount({ type: "CASE", caseStatus: "DELIVERED", ...query });
        if (!deliveredCount.status || deliveredCount.error) return res.status(500).send(deliveredCount);

        // 15. 
        const paymentPendingCount = await CaseServices.getCount({ type: "CASE", caseStatus: "PAYMENTPENDING", ...query });
        if (!paymentPendingCount.status || paymentPendingCount.error) return res.status(500).send(paymentPendingCount);

        // 16. 
        const completedCount = await CaseServices.getCount({ type: "CASE", caseStatus: "COMPLETED", ...query });
        if (!completedCount.status || completedCount.error) return res.status(500).send(completedCount);

        const cancelledCount = await CaseServices.getCount({ type: "CASE", caseStatus: "CANCELLED", ...query });
        if (!cancelledCount.status || cancelledCount.error) return res.status(500).send(cancelledCount);

        const draftCount = await CaseServices.getCount({ type: "CASE", caseStatus: "DRAFT", ...query });
        if (!draftCount.status || draftCount.error) return res.status(500).send(draftCount);

        const draftCompletedCount = await CaseServices.getCount({ type: "CASE", caseStatus: "DRAFTCOMPLETED", ...query });
        if (!draftCompletedCount.status || draftCompletedCount.error) return res.status(500).send(draftCompletedCount);

        const verifiedCount = await CaseServices.getCount({ type: "CASE", caseStatus: "VERIFIED", ...query });
        if (!verifiedCount.status || verifiedCount.error) return res.status(500).send(verifiedCount);

        const holdForPaymentCount = await CaseServices.getCount({ type: "CASE", caseStatus: "HOLDFORPAYMENT", ...query });
        if (!holdForPaymentCount.status || holdForPaymentCount.error) return res.status(500).send(holdForPaymentCount);

        return res.status(200).send({
            status: true,
            leadCount: leadCount.cases,
            newCount: newCount.cases,
            documentVerifiedCount: documentVerifiedCount.cases,
            // documentUnverifiedCount: documentUnverifiedCount.cases,
            readyForMakerCount: readyForMakerCount.cases,
            readyForCheckerCount: readyForCheckerCount.cases,
            draftCount: draftCount.cases,
            remainingSiteVisitCount: remainingSiteVisitCount.cases,
            rateFinalizedCount: rateFinalizedCount.cases,
            onHoldCount: onHoldCount.cases,
            siteVisitCompletedCount: siteVisitCompletedCount.cases,
            draftCompletedCount: draftCompletedCount.cases,
            verifiedCount: verifiedCount.cases,
            reportGeneratedCount: reportGeneratedCount.cases,
            holdForPaymentCount: holdForPaymentCount.cases,
            attachSignCount: attachSignCount.cases,
            signedReportAttachCount: signedReportAttachCount.cases,
            sentToDeliveryCount: sentToDeliveryCount.cases,
            readyForPrintCount: readyForPrintCount.cases,
            deliveredCount: deliveredCount.cases,
            paymentPendingCount: paymentPendingCount.cases,
            completedCount: completedCount.cases,
            cancelledCount: cancelledCount.cases
        });
    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.dashboard= ~ CaseResult: ${e.toString()}`)
        return res.status(500).send({
            status: false,
            error: "Internal Server Error!"
        });
    }
};

/**
 * @description This function is used for update caseStatus and create Case Log every time .
 * @param {*} req  caseId from req.params
 * @param {Object} res
 * @returns Object caseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */

module.exports.updateCaseStatus = async (req, res) => {
    try {
        const id = req.params.id;
        const user = req.local.user;
        const organization = req.local.user.organization;
        const { caseStatus, reason } = req.body;

        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        let query = {};
        query['updatedBy'] = user._id;
        query['caseStatus'] = caseStatus;
        query['reason'] = reason;

        if (caseStatus == "RATEFINALIZED") {
            query['rateVerify.status'] = true;
            query['rateVerify.dateTime'] = new Date();
            query['rateVerify.user'] = user;

            data['taskStatus.RATE_VERIFY'] = true;
        }
        else if (caseStatus == "SITEVISITCOMPLETED") {
            const siteVisitedDate = req.body.siteVisitedDate;
            query['$push'] = {
                "siteVisit.visitedDate": { remark: "MANUAL", status: "COMPLETED", dateTime: siteVisitedDate }
            };
            // query['siteVisit.siteVisitDate'] = moment(new Date(siteVisitedDate)).format("YYYY-MM-DD");;
            query['siteVisit.siteVisitStatus'] = true;
        }
        else if (caseStatus == "READYFORCHECKER") {
            query['taskStatus.MAKER'] = true;
        }
        else if (caseStatus == "REPORTGENERATED") {
            query['taskStatus.CHECKER'] = true;
        }
        const CaseResult = await CaseServices.updateCase(id, query);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        // create case log as per changed case status
        const createCaseLog = await CaseLogServices.createCaseLog({ user: user, organization: organization, caseStatus: caseStatus, reason: reason, case: CaseResult.cases, dateTime: new Date() })
        if (!createCaseLog.status || createCaseLog.error) return res.status(500).send(createCaseLog);

        return res.status(200).send(CaseResult);
    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.updateCaseStatus= ~ CaseResult: ${e.toString()}`)
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for Create CaseLog
 * @param {*} req.body.caseStatus from this get caseStatus
 * @param {*} res
 * @returns
 */
module.exports.createCaseLog = async (req, res) => {
    try {
        const id = req.params.id; // caseId
        const orgId = req.local.user.organization._id;
        let caseStatus = req.body.caseStatus;
        let query = {
            // 'organization._id': orgId,
            "case._id": id,
            caseStatus: caseStatus
        };

        if (caseStatus == "DATAENTRYSTART") {
            const getCaseLog = await CaseLogServices.getAllCaseLog(query)
            if (!getCaseLog.status || getCaseLog.error) return res.status(500).send(getCaseLog);

            if (getCaseLog.caseLogs.length !== 0) {
                caseStatus = "UPDATECASEVALUATION";
            }
            else {
                caseStatus == "DATAENTRYSTART"
            }
        }

        const CaseResult = await CaseServices.findCase(id);

        query['organization'] = req.local.user.organization;
        query['user'] = req.local.user;
        query['caseStatus'] = caseStatus;
        query['case'] = CaseResult.cases;
        query['dateTime'] = new Date();
        query['reason'] = req.body.remark;

        const createCaseLog = await CaseLogServices.createCaseLog(query);

    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.createCaseLog= ~ CaseResult: ${e.toString()}`)
        return { status: false, error: e };
    }
};

/**
 * @description This function is used for get caseLog for requested caseId
 * @param {*} req  case Id from req.params.id
 * @param {*} res
 * @returns Object getCaseLog{ "status": true, "caseLogs": Array }
 */
module.exports.getCaseLog = async (req, res) => {
    try {
        const id = req.params.id;
        const orgId = req.local.user.organization._id;
        let query = {
            // "organization._id": orgId,
            "case._id": id
        }
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const CaseResult = await CaseServices.findCase(id);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        const getCaseLog = await CaseLogServices.getAllCaseLog(query)
        if (!getCaseLog.status || getCaseLog.error) return res.status(500).send(getCaseLog);

        return res.status(200).send({ ...getCaseLog, caseNumber: CaseResult.cases.caseNumber, ownerName: CaseResult.cases.ownerName });
    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.getCaseLog= ~ CaseResult: ${e.toString()}`)
        res.status(500).send({
            status: false,
            error: "Internal Server Error!"
        });
    }
};

/**
 * @description This function is used for get all cases i.e. assigned case
 * @param {*} req
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: array }
 * -{ status: false, error: string }
 */
module.exports.getAllCases = async (req, res) => {
    try {
        const id = req.local.user._id;
        const role = req.local.user.role;
        const sortStr = req.query.sort || "";
        let sortBy = [];
        let query = {};
        var count = 0;

        query['organization._id'] = !!req.query.orgBranch && req.query.orgBranch || null;
        query['type'] = 'CASE';

        const creditOfficerId = req.query.creditOfficerId;
        const branchId = req.query.branchId;
        // const clientId = req.query.clientId;

        if (creditOfficerId) { query['creditOfficer._id'] = creditOfficerId; }
        if (branchId) { query['branch._id'] = branchId; }
        // if (clientId) { query['client._id'] = clientId; }

        // For sort
        if (!!sortStr) {
            const sort = sortStr.split(',').map((srt, id) => srt.split(':'))
            sortBy = sort
        } else {
            sortBy = { createdAt: -1 }
        }

        // For search
        const searchValue = req.query.searchValue;
        if (searchValue) {
            query['$and'] = [];
            query['$and'].push({
                '$or': [
                    { "staff.pickupPerson.firstName": { $regex: searchValue, $options: 'i' } },
                    { "staff.pickupPerson.lastName": { $regex: searchValue, $options: 'i' } },
                    { "staff.maker.firstName": { $regex: searchValue, $options: 'i' } },
                    { "staff.maker.lastName": { $regex: searchValue, $options: 'i' } },
                    { "staff.checker.firstName": { $regex: searchValue, $options: 'i' } },
                    { "staff.checker.lastName": { $regex: searchValue, $options: 'i' } },
                    { "staff.siteEngineer.firstName": { $regex: searchValue, $options: 'i' } },
                    { "staff.siteEngineer.lastName": { $regex: searchValue, $options: 'i' } },
                    { "staff.valuers.firstName": { $regex: searchValue, $options: 'i' } },
                    { "staff.valuers.lastName": { $regex: searchValue, $options: 'i' } },
                    { "staff.rateVerifier.firstName": { $regex: searchValue, $options: 'i' } },
                    { "staff.rateVerifier.lastName": { $regex: searchValue, $options: 'i' } },
                    { fullAddress: { $regex: searchValue, $options: 'i' } },
                    { caseNumber: { $regex: searchValue, $options: 'i' } },
                    { ownerName: { $regex: searchValue, $options: 'i' } },
                    { proposePurchaserName: { $regex: searchValue, $options: 'i' } },
                    { 'proposePurchasers.name': { $regex: searchValue, $options: 'i' } },
                    { caseStatus: { $regex: searchValue, $options: 'i' } },
                    { "typeOfReport.label": { $regex: searchValue, $options: 'i' } },
                    { "client.name": { $regex: searchValue, $options: 'i' } },
                    { "client.initial": { $regex: searchValue, $options: 'i' } },
                    { "branch.name": { $regex: searchValue, $options: 'i' } },
                    { "creditOfficer.firstName": { $regex: searchValue, $options: 'i' } },
                    { "creditOfficer.lastName": { $regex: searchValue, $options: 'i' } },
                    { "outward.number": { $regex: searchValue, $options: 'i' } }
                ]
            })
        }

        if (!['VALUER', 'ADMIN', 'FRONTDESK', "PRINTING", 'ACCOUNTANT'].includes(role)) {
            query['$or'] = [
                { "createdBy": id },
                { "staff.pickupPerson._id": id },
                { "staff.maker._id": id },
                { "staff.checker._id": id },
                { "staff.valuers._id": id },
                { "staff.rateVerifier._id": id },
                { "staff.siteEngineer._id": id },
                { "staff.deliveryPerson._id": id },
            ];
            if (!(!!req.query.orgBranch)) {
                delete query['organization._id'];
            }
        } else {
            if (!(!!req.query.orgBranch)) {
                delete query['organization._id'];
            }
        }

        // For Advance Search By ID
        const maker = req.query.maker;
        const checker = req.query.checker;
        const siteEngineer = req.query.siteEngineer;
        const valuer = req.query.valuer;
        const pickupPerson = req.query.pickupPerson;
        const deliveryPerson = req.query.deliveryPerson;
        const rateVerifier = req.query.rateVerifier;
        const creditOfficer = req.query.creditOfficer;
        const client = req.query.client;
        const { branch, paymentStatus } = req.query;

        if (pickupPerson) { query['staff.pickupPerson._id'] = pickupPerson; }
        if (maker) { query['staff.maker._id'] = maker; }
        if (checker) { query['staff.checker._id'] = checker; }
        if (deliveryPerson) { query['staff.deliveryPerson._id'] = deliveryPerson; }
        if (siteEngineer) { query['staff.siteEngineer._id'] = siteEngineer; }
        if (rateVerifier) { query['staff.rateVerifier._id'] = rateVerifier; }
        if (creditOfficer) { query['creditOfficer._id'] = creditOfficer; }
        if (valuer) { query['staff.valuers._id'] = valuer; }
        if (branch) { query['branch._id'] = branch; }
        if (client) { query['client._id'] = client; }
        if (!!paymentStatus) { query['paymentStatus'] = paymentStatus; }

        // For Advance Search By Name

        const ownerName = req.query.ownerName;
        const caseNumber = req.query.caseNumber;
        const fullAddress = req.query.fullAddress;
        const typeOfReport = req.query.typeOfReport;
        const caseStatus = req.query.caseStatus;
        const isUrgent = req.query.isUrgent; // true, false

        const createTo = req.query.createTo;
        const createFrom = req.query.createFrom;
        const dateKey = req.query.dateKey;

        if (fullAddress) { query['fullAddress'] = { $regex: fullAddress, $options: 'i' } }
        if (caseNumber) { query['caseNumber'] = { $regex: caseNumber, $options: 'i' } }
        if (ownerName) { query['ownerName'] = { $regex: ownerName, $options: 'i' } }
        if (typeOfReport) { query["typeOfReport.label"] = { $regex: typeOfReport, $options: 'i' } }
        if (caseStatus) {
            query['caseStatus'] = { $regex: caseStatus, $options: 'i' }
            // if (caseStatus === "SITEVISITCOMPLETED") {
            //     query['siteVisit.siteVisitStatus'] = true;
            // }
        }
        if (isUrgent) { query['isUrgent'] = { $regex: isUrgent, $options: 'i' } }

        if (createFrom && createTo && dateKey == "createdAtDate") { query['createdAt'] = { $gte: createFrom, $lte: createTo } }
        if (dateKey == "reportGeneratedDate") { query['reportGeneratedDate'] = { $gte: moment(new Date(createFrom)).format("YYYY-MM-DD"), $lte: moment(new Date(createTo)).format("YYYY-MM-DD") } }
        if (dateKey == "siteVisitDate") { query['siteVisitDate'] = { $gte: moment(new Date(createFrom)).format("YYYY-MM-DD"), $lte: moment(new Date(createTo)).format("YYYY-MM-DD") } }
        if (dateKey == "siteScheduleDate") { query['scheduleSiteVisitDate'] = { $gte: moment(new Date(createFrom)).format("YYYY-MM-DD"), $lte: moment(new Date(createTo)).format("YYYY-MM-DD") } }

        let displayFields = {
            owners: 1,
            ownerName: 1,
            proposePurchasersName: 1,
            reportInformation: 1,
            proposePurchasers: 1,
            ownerPhoneNo: 1,
            organization: 1,
            caseNumber: 1,
            fullAddress: 1,
            typeOfReport: 1,
            caseStatus: 1,
            paymentStatus: 1,
            isUrgent: 1,
            createdAtDate: 1,
            reportGeneratedDate: 1,
            siteVisitDate: 1,
            siteVisit: 1,
            creditOfficer: 1,
            branch: 1,
            client: 1,
            scheduleSiteVisitDate: 1,
            createCase: 1,
            staff: 1,
            externalDocuments: 1,
            clientType: 1,
            outward: 1,
            createdAt: 1,
            externalReportDetails: 1,
            location: 1,
            remarks: 1,
        }

        const CaseResult = await CaseServices.getAllCases(query, { display: displayFields, sort: sortBy }, req.local.startIndex, req.local.limit);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        const CaseResultCount = await CaseServices.getCount(query);
        if (!CaseResultCount.status || CaseResultCount.error) return res.status(500).send(CaseResultCount);
        count = CaseResultCount.cases;

        return res.status(200).send({ status: true, cases: CaseResult.cases, count: count });

    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.getAllCases= ~ CaseResult: ${e.toString()}`)
        return res.status(500).send({
            status: false,
            error: "Internal Server Error!"
        });
    }
};

module.exports.getLeadCaseList = async (req, res) => {
    try {
        const { id: _id, role } = req.local.user;
        const { orgBranch, clientId, branchId, creditOfficerId, searchText } = req.query;
        let query = {};

        if (orgBranch) { query['organization._id'] = orgBranch; }
        if (creditOfficerId) { query['creditOfficer._id'] = creditOfficerId; }
        if (branchId) { query['branch._id'] = branchId; }
        // if (clientId) { query['client._id'] = clientId; }

        // For search
        if (searchText) {
            query['$and'] = [];
            query['$and'].push({
                '$or': [
                    { "staff.pickupPerson.firstName": { $regex: searchText, $options: 'i' } },
                    { "staff.pickupPerson.lastName": { $regex: searchText, $options: 'i' } },
                    { "staff.maker.firstName": { $regex: searchText, $options: 'i' } },
                    { "staff.maker.lastName": { $regex: searchText, $options: 'i' } },
                    { "staff.checker.firstName": { $regex: searchText, $options: 'i' } },
                    { "staff.checker.lastName": { $regex: searchText, $options: 'i' } },
                    { "staff.siteEngineer.firstName": { $regex: searchText, $options: 'i' } },
                    { "staff.siteEngineer.lastName": { $regex: searchText, $options: 'i' } },
                    { "staff.valuers.firstName": { $regex: searchText, $options: 'i' } },
                    { "staff.valuers.lastName": { $regex: searchText, $options: 'i' } },
                    { "staff.rateVerifier.firstName": { $regex: searchText, $options: 'i' } },
                    { "staff.rateVerifier.lastName": { $regex: searchText, $options: 'i' } },
                    { fullAddress: { $regex: searchText, $options: 'i' } },
                    { caseNumber: { $regex: searchText, $options: 'i' } },
                    { ownerName: { $regex: searchText, $options: 'i' } },
                    { proposePurchaserName: { $regex: searchText, $options: 'i' } },
                    { 'proposePurchasers.name': { $regex: searchText, $options: 'i' } },
                    { caseStatus: { $regex: searchText, $options: 'i' } },
                    { "typeOfReport.label": { $regex: searchText, $options: 'i' } },
                    { "client.name": { $regex: searchText, $options: 'i' } },
                    { "client.initial": { $regex: searchText, $options: 'i' } },
                    { "branch.name": { $regex: searchText, $options: 'i' } },
                    { "creditOfficer.firstName": { $regex: searchText, $options: 'i' } },
                    { "creditOfficer.lastName": { $regex: searchText, $options: 'i' } },
                    { "outward.number": { $regex: searchText, $options: 'i' } }
                ]
            })
        }

        // For Advance Search By ID
        const { client, branch, maker, checker, siteEngineer, valuer, pickupPerson, deliveryPerson, rateVerifier, creditOfficer } = req.query;

        if (pickupPerson) { query['staff.pickupPerson._id'] = pickupPerson; }
        if (maker) { query['staff.maker._id'] = maker; }
        if (checker) { query['staff.checker._id'] = checker; }
        if (deliveryPerson) { query['staff.deliveryPerson._id'] = deliveryPerson; }
        if (siteEngineer) { query['staff.siteEngineer._id'] = siteEngineer; }
        if (rateVerifier) { query['staff.rateVerifier._id'] = rateVerifier; }
        if (creditOfficer) { query['creditOfficer._id'] = creditOfficer; }
        if (valuer) { query['staff.valuers._id'] = valuer; }
        if (branch) { query['branch._id'] = branch; }
        if (client) { query['client._id'] = client; }

        // For Advance Search By Name
        const { ownerName, caseNumber, fullAddress, typeOfReport, caseStatus, isUrgent, createTo, createFrom, dateKey } = req.query;
        if (fullAddress) { query['fullAddress'] = { $regex: fullAddress, $options: 'i' } }
        if (caseNumber) { query['caseNumber'] = { $regex: caseNumber, $options: 'i' } }
        if (ownerName) { query['ownerName'] = { $regex: ownerName, $options: 'i' } }
        if (typeOfReport) { query["typeOfReport.label"] = { $regex: typeOfReport, $options: 'i' } }
        if (caseStatus) {
            query['caseStatus'] = { $regex: caseStatus, $options: 'i' }
            if (caseStatus === "DOCUMENTVERIFIED") {
                query['verifiedDocument.value'] = true;
            }
            if (caseStatus === "SITEVISITCOMPLETED") {
                query['siteVisit.siteVisitStatus'] = true;
            }
        }
        if (isUrgent) { query['isUrgent'] = { $regex: isUrgent, $options: 'i' } }

        if (createFrom && createTo && dateKey == "createdAtDate") { query['createdAt'] = { $gte: createFrom, $lte: createTo } }
        if (dateKey == "reportGeneratedDate") { query['reportGeneratedDate'] = { $gte: moment(new Date(createFrom)).format("YYYY-MM-DD"), $lte: moment(new Date(createTo)).format("YYYY-MM-DD") } }
        if (dateKey == "siteVisitDate") { query['siteVisitDate'] = { $gte: moment(new Date(createFrom)).format("YYYY-MM-DD"), $lte: moment(new Date(createTo)).format("YYYY-MM-DD") } }
        if (dateKey == "siteScheduleDate") { query['scheduleSiteVisitDate'] = { $gte: moment(new Date(createFrom)).format("YYYY-MM-DD"), $lte: moment(new Date(createTo)).format("YYYY-MM-DD") } }

        let displayFields = {
            owners: 1, ownerName: 1, proposePurchasersName: 1, proposePurchasers: 1, ownerPhoneNo: 1,
            reportInformation: 1, organization: 1, caseNumber: 1, fullAddress: 1, typeOfReport: 1,
            caseStatus: 1, isUrgent: 1, createdAtDate: 1, reportGeneratedDate: 1,
            siteVisitDate: 1, siteVisit: 1, creditOfficer: 1, leadType: 1, type: 1,
            branch: 1, client: 1, scheduleSiteVisitDate: 1, createCase: 1, staff: 1, externalDocuments: 1,
            clientType: 1, outward: 1, createdAt: 1, externalReportDetails: 1, location: 1, remarks: 1,
        }

        const CaseResult = await CaseServices.find({ query, display: displayFields, start: req.local.startIndex, limit: req.local.limit });
        if (!CaseResult.status || CaseResult.error) {
            logger.error(`🚀 ~ module.exports.getLeadCaseList= ~ CaseResult: ${CaseResult.error?.toString()}`)
            return res.status(500).send(CaseResult);
        }

        return res.status(200).send({ status: true, cases: CaseResult.cases, count: CaseResult.count });

    } catch (e) {
        logger.error(`🚀 ~ module.exports.getLeadCaseList= ~ e:` + e.toString())
        res.status(500).send({ status: false, error: e.toString() });
    }
};

/**
 * @description This function is used for get all cases i.e. assigned Pending case
 * @param {*} req
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: array }
 * -{ status: false, error: string }
 */
module.exports.getAllPendingCases = async (req, res) => {
    try {
        const id = req.local.user._id;
        const role = req.local.user.role;

        let query = {};

        // query['organization._id'] = req.local.user.organization._id;
        query['type'] = 'CASE';
        query['caseStatus'] = {
            $nin: ["CLOSED", "CANCELLED"
                // , "PAYMENTPENDING", "DELIVERED", "SENTTODELIVERY", "HOLDFORPAYMENT", "REPORTGENERATED"
            ]
        }
        // For search
        const searchValue = req.query.searchValue;
        if (searchValue) {
            query['$and'] = [];
            query['$and'].push({
                '$or': [
                    { fullAddress: { $regex: searchValue, $options: 'i' } },
                    { caseNumber: { $regex: searchValue, $options: 'i' } },
                    { ownerName: { $regex: searchValue, $options: 'i' } },
                    { caseStatus: { $regex: searchValue, $options: 'i' } },
                    { "typeOfReport.label": { $regex: searchValue, $options: 'i' } },
                    { "client.name": { $regex: searchValue, $options: 'i' } },
                    { "client.initial": { $regex: searchValue, $options: 'i' } },
                    { "branch.name": { $regex: searchValue, $options: 'i' } },
                    { "creditOfficer.firstName": { $regex: searchValue, $options: 'i' } },
                    { "creditOfficer.lastName": { $regex: searchValue, $options: 'i' } },
                ]
            })
        }

        if (!['VALUER', 'ADMIN', 'FRONTDESK'].includes(role)) {
            query['$or'] = [
                { "staff.pickupPerson._id": id },
                { "staff.maker._id": id },
                { "staff.checker._id": id },
                { "staff.valuers._id": id },
                { "staff.rateVerifier._id": id },
                { "staff.siteEngineer._id": id },
            ]
        }
        const CaseResult = await CaseServices.getAllCases(query, {}, req.local.startIndex, req.local.limit);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        const CaseResultCount = await CaseServices.getCount(query);
        if (!CaseResultCount.status || CaseResultCount.error) return res.status(500).send(CaseResultCount);
        count = CaseResultCount.cases;

        return res.status(200).send({ status: true, cases: CaseResult.cases, count: count });

    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.getAllPendingCases=  ${e.toString()}`)
        return res.status(500).send({
            status: false,
            error: e.toString()
        });
    }
};

/**
 * @description This function is used for find case from caseId
 * @param {String} req.params get caseId
 * @param {*} res
 * @returns CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.getCase = async (req, res) => {
    try {
        const id = req.params.id;

        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const CaseResult = await CaseServices.findCase(id);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        const tasks = await TaskServices.findAll({ caseId: id }, {});
        const hasInvoiceRes = await nInvoiceServices.find({ query: { 'cases._id': id, invoiceStatus: "COMPLETED" } });
        if (!hasInvoiceRes.status || hasInvoiceRes.error) return res.status(500).send(hasInvoiceRes);

        const hasPaymentRes = await PaymentServices.getAllPayments({ caseIds: { $in: id }, paymentStatus: "COMPLETED" });
        if (!hasPaymentRes.status || hasPaymentRes.error) return res.status(500).send(hasPaymentRes);
        const payments = hasPaymentRes.payments || [];

        const mailLogRes = await MailLogServices.find({ query: { 'case._id': id } });
        return res.status(200).send({ ...CaseResult, tasks: tasks, payment: payments, invoice: hasInvoiceRes.data, mailLogs: mailLogRes.data || [] });
    } catch (e) {
        logger.error(`🚀 ~ Case.Controller.getCase= ~ ${e.toString()}`)
        res.status(500).send({ status: false, error: e.toString() });
    }
};

/**
 * @description This function is used for verified document., if not then add remark. then update case
 * @param {*} req
 * @param {*} res
 * @returns CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.verifiedDocuments = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        let data = req.body;
        data.updatedBy = user._id;

        if (data.verifiedDocument.value == false || data.verifiedDocument.value == null) {
            data.verifiedDocument.value = false
            data.verifiedDocument.dateTime = new Date();

            data.caseStatus = "LEAD";
            const caseResult = await CaseServices.updateCase(id, data);

            if (!caseResult) return res.status(500).send(caseResult);
            if (!caseResult.status) return res.status(500).send(caseResult);

            req.body.caseStatus = "LEAD";

            const createCaseLog = this.createCaseLog(req);

            return res.status(200).send(caseResult);

        }
        else {
            data.verifiedDocument.dateTime = new Date();
            data.caseStatus = "DOCUMENTVERIFIED";
            data['taskStatus.DOCUMENT_VERIFY'] = true;
            const caseResult = await CaseServices.updateCase(id, data);

            if (!caseResult) return res.status(500).send(caseResult);
            if (!caseResult.status) return res.status(500).send(caseResult);

            req.body.caseStatus = "DOCUMENTVERIFIED";

            const createCaseLog = this.createCaseLog(req);

            return res.status(200).send(caseResult);

        }
    }
    catch (e) {
        logger.error(`🚀 ~ Case.Controller.verifiedDocuments= ~ ${e.toString()}`)
        res.status(500).send({ status: false, error: e.toString() });
    }
};
/**
 * @description This function is used for create case
 * @param {Object} req from req.local
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.createCase = async (req, res) => {
    try {
        const datas = req.local.case;
        const orgId = req.local.case.organizationId || req.local.user.organization._id;
        datas.createCase = "true";
        let caseStatus = "NEW";

        const Organizations = await OrganizationServices.getById(orgId);
        if (!Organizations.status || Organizations.error) return res.status(500).send(Organizations);
        datas.organization = Organizations.Organizations;

        if (['INDIVIDUAL', 'PRIVATE_CLIENT'].includes(datas.clientType.value)) {
            const hasClient = await ClientServices.getAllClients({ name: datas.ownerName })
            if (!hasClient.status || hasClient.error) return res.status(500).send(hasClient);
            let data = { type: datas.clientType.value };

            if (hasClient.clients.length == 0) {
                data.name = datas.ownerName;
                data.phone = datas.ownerPhoneNo;
                data.displayName = datas.ownerName;
                data.initial = datas.ownerName.slice(0, 1);
                data.ledgerName = datas.ownerName;
                data.address = datas.fullAddress;
                data.createdBy = req.local.user._id;
                data.uniqueId = Math.floor(100000 + Math.random() * 900000);
                data.email = data.uniqueId + "_owner@gmail.com";

                data.organization = Organizations.Organizations;

                const client_res = await ClientServices.createClient(data);
                if (!client_res.status) return res.status(500).send(client_res);

                datas.client = client_res.clients;
            }
            else {
                datas.client = hasClient.clients[0];
            }
        }
        else {
            if (datas.creditOfficer) {
                const creditOfficer = await CreditOfficersServices.findById(datas.creditOfficer);
                if (!creditOfficer.status || creditOfficer.error) return res.status(500).send(creditOfficer);
                datas.creditOfficer = creditOfficer.data;
            }

            const Client = await ClientServices.getClientById(datas.client);
            if (!Client.status || Client.error) return res.status(500).send(Client);
            datas.client = Client.clients;

            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status || Branch.error) return res.status(500).send(Branch);
            datas.branch = Branch.branches;

            if (datas.zone) {
                const Zone = await ZoneServices.getZone(datas.zone);
                if (!Zone.status || Zone.error) return res.status(500).send(Zone);
                datas.zone = Zone.zones;
            }
        }

        // TODO - Check if staff assigned then dont add users 
        if (datas.staff) {
            if (datas.staff.pickupPerson) {
                const pickupPerson = await UserServices.findById(datas.staff.pickupPerson);
                if (!pickupPerson.status || pickupPerson.error) return res.status(500).send(pickupPerson);
                datas.staff.pickupPerson = pickupPerson.users;
            }
            if (datas.staff.deliveryPerson) {
                const deliveryPerson = await UserServices.findById(datas.staff.deliveryPerson);
                if (!deliveryPerson.status || deliveryPerson.error) return res.status(500).send(deliveryPerson);
                datas.staff.deliveryPerson = deliveryPerson.users;
            }
            if (datas.staff.siteEngineer) {
                const siteEngineer = await UserServices.findById(datas.staff.siteEngineer);
                if (!siteEngineer.status || siteEngineer.error) return res.status(500).send(siteEngineer);
                datas.staff.siteEngineer = siteEngineer.users;
            }
            if (datas.staff.maker) {
                const maker = await UserServices.findById(datas.staff.maker);
                if (!maker.status || maker.error) return res.status(500).send(maker);
                datas.staff.maker = maker.users;
            }
            if (datas.staff.checker) {
                const checker = await UserServices.findById(datas.staff.checker);
                if (!checker.status || checker.error) return res.status(500).send(checker);
                datas.staff.checker = checker.users;
            }
            if (datas.staff.rateVerifier) {
                const rateVerifier = await UserServices.findById(datas.staff.rateVerifier);
                if (!rateVerifier.status || rateVerifier.error) return res.status(500).send(rateVerifier);
                datas.staff.rateVerifier = rateVerifier.users;
            }
            let valuerData = [];
            if (datas.staff.valuers) {

                await Promise.all(datas.staff.valuers.map(async (valuer) => {

                    const valuerRes = await UserServices.findById(valuer._id);

                    var valuer_res = valuerRes.users;
                    valuer_res = valuer_res.toJSON();
                    valuer_res.valuersRegistrationNumber = valuer.valuersRegistrationNumber

                    valuerData.push(valuer_res);
                }))
                datas.staff.valuers = valuerData;
            }
        }
        if (datas.owners) {
            datas.owners.map((owner) => {
                const owner_id = Math.floor(100000 + Math.random() * 900000);
                owner.id = owner_id;
            })
        }
        if (datas.siteVisit) {
            if (datas.siteVisit.schedule && datas.siteVisit.schedule.dateTime != "") {
                datas.scheduleSiteVisitDate = moment(new Date(datas.siteVisit.schedule.dateTime)).format("YYYY-MM-DD");
                caseStatus = "REMAININGSITEVISIT"
            }
        }
        if (datas.documents) {

            await Promise.all(datas.documents.map(async (document) => {
                if (!document.id) {
                    const doc_id = Math.floor(100000 + Math.random() * 900000);
                    document.id = doc_id;
                    document.uploadedBy = req.local.user._id;
                    document.createdAt = new Date();
                }
            }))
        }

        // ===================== For give unique CaseNumber to lead and case ===================

        const case_result = await CaseServices.find({ limit: 1 });
        if (!case_result.status || case_result.error) return res.status(500).send(case_result);

        if (case_result.cases.length == 0) {
            const str = "" + 1, pad = "000000"
            const answer = pad.substring(0, pad.length - str.length) + str;
            datas.caseNumber = answer;
        }
        else {
            var caseNo = case_result.cases[0].caseNumber;
            var caseNumber = parseInt(caseNo) + 1;
            const str = "" + caseNumber, pad = "000000"
            var answer = pad.substring(0, pad.length - str.length) + str;
            datas.caseNumber = answer;
        }

        // datas.caseNumber = Math.floor(100000 + Math.random() * 900000);
        datas.createdBy = req.local.user._id;
        let caseHistory = [];
        caseHistory = [{
            "id": Math.floor(100000 + Math.random() * 900000),
            "title": "Create Case",
            "description": "Create Case",
            "datetime": new Date(),
        }];

        datas.caseHistory = caseHistory;
        datas.type = "CASE";
        datas.caseStatus = caseStatus;

        const CaseResult = await CaseServices.createCase(datas);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        // From here call createCaseLog function for create caseLog
        req.body.caseStatus = "CREATECASE";
        const createCaseLog = this.createCaseLog(req);

        // ========================== Document Image ==========================

        const id = CaseResult.cases._id.toString();
        if (CaseResult.cases.documents.length > 0) {

            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
            }

            await Promise.all(CaseResult.cases.documents.map(async file => {

                const fileName = "document_" + file.fileName;
                let currentPath = process.cwd() + "/" + file.path;

                const destination = path.join(process.env.CDN_PATH, "cases", id, fileName);

                // call function for move file
                await utilitiesController.moveFile(currentPath, destination);
                const filepath = path.join("cases", id, fileName);

                file.fileName = fileName;
                file.path = filepath;
            }))
            CaseResult.cases.save();
        }
        return res.status(200).send({ status: true, caseResult: CaseResult.cases });
    } catch (e) {
        logger.error(`🚀 ~ file: Case.controller.js:889 ~ module.exports.createCase= ~ e: e.toString()`);
        res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

// Function to generate a new caseNumber
async function generateNewCaseNumber() {
    // Get the last created case from the collection
    const lastCaseRes = await CaseServices.find({ sort: { createdAt: -1 } });
    const lastCase = lastCaseRes.cases[0] || { caseNumber: "000000" };

    var caseNo = lastCase.caseNumber;
    var caseNumber = parseInt(caseNo) + 1;
    const str = "" + caseNumber, pad = "000000"
    var answer = pad.substring(0, pad.length - str.length) + str;
    return answer;
}

function updateIdInJsonString(jsonString, oldId, newId) {
    try {
        const updatedJsonString = jsonString.replace(new RegExp(`\\b${oldId}\\b`, 'gi'), newId);
        const jsonObject = JSON.parse(updatedJsonString);
        return jsonObject;
    } catch (error) {
        console.error('Error parsing or updating JSON:', error);
        return null;
    }
}


function copyFolderRecursive(sourcePath, destinationPath) {
    if (!fs.existsSync(sourcePath)) {
        return;
    }
    // Ensure the destination directory exists
    if (!fs.existsSync(destinationPath)) {
        fs.mkdirSync(destinationPath);
    }
    // Read the contents of the source directory
    const files = fs.readdirSync(sourcePath);
    console.log("->OLD: ", sourcePath, "\n New: ", destinationPath);

    files.forEach((file) => {
        const sourceFilePath = path.join(sourcePath, file);
        const destinationFilePath = path.join(destinationPath, file);
        // Check if the current item is a directory
        const isDirectory = fs.statSync(sourceFilePath).isDirectory();

        if (isDirectory) {
            // Recursively copy subdirectories
            copyFolderRecursive(sourceFilePath, destinationFilePath);
        } else {
            // Copy individual files
            fs.copyFileSync(sourceFilePath, destinationFilePath);
        }
    });
}

/**
 * @description This function is used for create case
 * @param {Object} req from req.local
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.copyCase = async (req, res) => {
    try {
        const { id: caseId } = req.params;
        const { copyDocuments = false, copyComposition = false, copyExternalReport = false, copyImages = false, copyIndicator = false } = req.query;

        const existingCaseRes = await CaseServices.find({ query: { _id: caseId } });
        if (!existingCaseRes.status || existingCaseRes.count == 0) {
            console.log(`No case found with caseNumber: ${caseNumber}`);
            return res.status(500).send({ status: false, error: `No case found with caseNumber: ${caseNumber}` });
        }
        console.log("exsisting case: ", JSON.stringify(existingCaseRes));
        const existingCase = existingCaseRes.cases[0].toJSON();
        const oldId = existingCase._id;
        const newId = new ObjectId();

        // Duplicate the existing document with a new _id
        const duplicatedCase = { ...existingCase, _id: newId };
        const updatedJsonObject = updateIdInJsonString(JSON.stringify(duplicatedCase), oldId, newId);
        // Update caseNumber and createdAt in the duplicated document
        updatedJsonObject._id = newId;
        updatedJsonObject.caseNumber = await generateNewCaseNumber();
        updatedJsonObject.createdAt = new Date();
        // updatedJsonObject.createdBy = ObjectID(updatedJsonObject.createdBy);

        updatedJsonObject.outward = { status: false };
        updatedJsonObject.caseStatus = "CASE";
        updatedJsonObject.paymentStatus = "NA";
        updatedJsonObject.invoiceId = null;
        updatedJsonObject.externalDocuments = [];
        updatedJsonObject.refCaseId = caseId;

        // Insert the duplicated document into the collection
        const newCaseRes = await CaseServices.createCase(updatedJsonObject);
        if (!newCaseRes.status) {
            return res.status(500).send({ status: false, error: "Error in Create duplicate case. ", message: newCaseRes.error });
        }

        logger.error(`Case ${duplicatedCase.caseNumber} duplicated successfully with new caseNumber: ${JSON.stringify(updatedJsonObject.caseNumber )}`);
        const CDN = process.env.CDN_PATH;
        // Copy the entire folder
        await copyFolderRecursive(CDN + 'cases/' + oldId, CDN + 'cases/' + newId);
        return res.status(200).send({ status: true, caseResult: newCaseRes.cases });

    } catch (e) {
        console.log(e);
        logger.error(`🚀 ~ file: Case.controller.js:819 ~ module.exports.createCase= ~ e:", ${e}`)
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for update case
 * @param {*} req.params.id from that get caseId and from req.local.case get case Object
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.updateCase = async (req, res) => {
    try {
        const id = req.params.id;
        const user = req.local.user;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const CaseFound = await CaseServices.findCase(id);
        if (!CaseFound.status || CaseFound.error) return res.status(500).send(CaseFound);

        let caseObj = CaseFound.cases;
        let datas = req.local.case;

        datas.updatedBy = user._id;

        if (!!datas.siteVisit && datas.siteVisit.visitedDate) {
            datas.siteVisit.visitedDate.map(async (value) => {
                if (value.status == "COMPLETED") {
                    datas.siteVisit.siteVisitStatus = 'true';
                    datas.siteVisitDate = moment(new Date(value.dateTime)).format("YYYY-MM-DD");
                }
            })
        }

        if (datas.siteVisit.siteVisitStatus != true && caseObj.caseStatus == "NEW") {
            datas.caseStatus = "REMAININGSITEVISIT";
        }

        if (datas.creditOfficer) {
            const creditOfficer = await CreditOfficersServices.findById(datas.creditOfficer);
            if (!creditOfficer.status || creditOfficer.error) return res.status(500).send(creditOfficer);
            datas.creditOfficer = creditOfficer.data;
        }
        if (datas.organizationId) {
            const Organizations = await OrganizationServices.getById(datas.organizationId);
            if (!Organizations.status || Organizations.error) return res.status(500).send(Organizations);
            datas.organization = Organizations.Organizations;
        }
        if (datas.client) {
            const Client = await ClientServices.getClientById(datas.client);
            if (!Client.status || Client.error) return res.status(500).send(Client);
            datas.client = Client.clients;
        }
        if (datas.branch) {
            const Branch = await BranchServices.getBranch(datas.branch);
            if (!Branch.status || Branch.error) return res.status(500).send(Branch);
            datas.branch = Branch.branches;
        }
        if (datas.zone) {
            const Zone = await ZoneServices.getZone(datas.zone);
            if (!Zone.status || Zone.error) return res.status(500).send(Zone);
            datas.zone = Zone.zones;
        }
        if (datas.staff) {
            if (datas.staff.pickupPerson) {
                const pickupPerson = await UserServices.findById(datas.staff.pickupPerson);
                if (!pickupPerson.status || pickupPerson.error) return res.status(500).send(pickupPerson);
                datas.staff.pickupPerson = pickupPerson.users;
            }
            if (datas.staff.deliveryPerson) {
                const deliveryPerson = await UserServices.findById(datas.staff.deliveryPerson);
                if (!deliveryPerson.status || deliveryPerson.error) return res.status(500).send(deliveryPerson);
                datas.staff.deliveryPerson = deliveryPerson.users;
            }
            if (datas.staff.siteEngineer) {
                const siteEngineer = await UserServices.findById(datas.staff.siteEngineer);
                if (!siteEngineer.status || siteEngineer.error) return res.status(500).send(siteEngineer);
                datas.staff.siteEngineer = siteEngineer.users;
            }
            if (datas.staff.maker) {
                const maker = await UserServices.findById(datas.staff.maker);
                if (!maker.status || maker.error) return res.status(500).send(maker);
                datas.staff.maker = maker.users;
            }
            if (datas.staff.checker) {
                const checker = await UserServices.findById(datas.staff.checker);
                if (!checker.status || checker.error) return res.status(500).send(checker);
                datas.staff.checker = checker.users;
            }
            if (datas.staff.rateVerifier) {
                const rateVerifier = await UserServices.findById(datas.staff.rateVerifier);
                if (!rateVerifier.status || rateVerifier.error) return res.status(500).send(rateVerifier);
                datas.staff.rateVerifier = rateVerifier.users;
            }
            let valuerData = [];
            if (datas.staff.valuers) {

                await Promise.all(datas.staff.valuers.map(async (valuer) => {

                    const valuerRes = await UserServices.findById(valuer._id);

                    var valuer_res = valuerRes.users;
                    valuer_res = valuer_res.toJSON();
                    valuer_res.valuersRegistrationNumber = valuer.valuersRegistrationNumber

                    valuerData.push(valuer_res);
                }))
                datas.staff.valuers = valuerData;
            }
        }

        if (datas.owners) {
            datas.owners.map((owner) => {
                if (!owner.id) {
                    const owner_id = Math.floor(100000 + Math.random() * 900000);
                    owner.id = owner_id;
                }
            })
        }

        if (datas.documents) {
            await Promise.all(datas.documents.map(async (document) => {
                if (!document.id) {
                    const doc_id = Math.floor(100000 + Math.random() * 900000);
                    document.id = doc_id;
                    document.uploadedBy = req.local.user._id;
                    document.createdAt = new Date();
                }
            }))
        }

        if (datas.images) {
            datas.images.map((image) => {
                if (!image.id) {
                    const img_id = Math.floor(100000 + Math.random() * 900000);
                    image.id = img_id;
                    image.uploadedBy = req.local.user._id;
                    image.updatedAt = new Date()
                }
            })
        }

        datas.updatedBy = req.local.user._id;
        if (datas.composition) {
            datas.composition.map(value => {

                if (!value.id) {
                    value.id = Math.floor(100000 + Math.random() * 900000);
                }
                value.images.map((image) => {
                    if (!image.id) {
                        const img_id = Math.floor(100000 + Math.random() * 900000);
                        image.id = img_id;
                        image.uploadedBy = req.local.user._id;
                        image.createdAt = new Date()
                    }
                })
            })
        }
        if (datas.otherComposition) {

            datas.otherComposition.map(value => {
                // if (!value.id) {
                //     value.id = Math.floor(100000 + Math.random() * 900000);
                // }
                value.images.map((image) => {
                    if (!image.id) {
                        const img_id = Math.floor(100000 + Math.random() * 900000);
                        image.id = img_id;
                        image.uploadedBy = req.local.user._id;
                        image.createdAt = new Date()
                    }
                })
            })
        }
        if (datas.documentPerArea) {
            datas.documentPerArea.map((document) => {
                const doc_id = Math.floor(100000 + Math.random() * 900000);
                document.id = doc_id;
            })
        }

        // console.log(datas.siteVisit);
        // if (datas.scheduleSiteVisit.dateTime == "" || datas.scheduleSiteVisit.dateTime == null) {
        //     datas.caseStatus = "REMAININGSITEVISIT";
        // } else {
        //     datas.caseStatus = "SITEVISITCOMPLETED";
        // }

        if (datas.indicatorArr) {
            datas.indicatorArr.map(value => {
                if (!value.id) {
                    value.id = Math.floor(100000 + Math.random() * 900000);
                }
            })
        }
        if (datas.routeMap) {
            if (datas.routeMap.locationMapImg) {
                const locationMapImg = datas.routeMap.locationMapImg;
                if (locationMapImg) {
                    locationMapImg.map(images => {
                        if (!images.id) {
                            const img_id = Math.floor(100000 + Math.random() * 900000);
                            images.id = img_id;
                            images.uploadedBy = req.local.user._id;
                            images.createdAt = new Date()
                        }
                    })
                }
            }
        }

        const CaseResult = await CaseServices.updateCase(id, datas);
        if (!CaseResult.status) return res.status(500).send(CaseResult);

        // From here call createCaseLog function for create caseLog

        req.body.caseStatus = "UPDATECASE";
        const createCaseLog = this.createCaseLog(req);

        return res.status(200).send(CaseResult);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1044 ~ module.exports.updateCase= ~ err:", err)
        return res.status(500).send({ status: false, error: JSON.stringify(err) });
    }
};

/**
 * @description This function is used for update caseValuation
 * @param {*} req.params.id from that get caseId and from req.local get caseValuation Object
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.updateCaseValuation = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const findCase = await CaseServices.findCase(id);
        if (!findCase.status) return res.status(500).send(findCase);

        let datas = req.local.caseValuation;
        const user = req.local.user;
        // datas.caseStatus = "READYFORMAKER";// ready for maker
        // datas.outwardStatus = "SYSTEMGENERATED";
        datas.updatedBy = req.local.user._id;

        if (datas.owners) {
            datas.owners.map((owner) => {
                if (!owner.id) {
                    const owner_id = Math.floor(100000 + Math.random() * 900000);
                    owner.id = owner_id;
                }
            })
        }
        if (datas.images) {
            datas.images.map((image) => {
                if (!image.id) {
                    const img_id = Math.floor(100000 + Math.random() * 900000);
                    image.id = img_id;
                    image.path = image.path;
                    image.name = image.name;
                    image.uploadedBy = req.local.user._id;
                    image.updatedAt = new Date()
                }
            })
        }
        if (datas.marketAnalysisAndRemark && datas.marketAnalysisAndRemark.audioNotes) {
            datas.marketAnalysisAndRemark.audioNotes.map((image) => {
                if (!image.id) {
                    const img_id = Math.floor(100000 + Math.random() * 900000);
                    image.id = img_id;
                    image.path = image.path;
                    image.name = image.name;
                    image.uploadedBy = req.local.user._id;
                    image.updatedAt = new Date()
                }
            })
        }

        if (datas.documents) {
            await Promise.all(datas.documents.map(async (document) => {
                if (!document.id) {
                    const doc_id = Math.floor(100000 + Math.random() * 900000);
                    document.id = doc_id;
                    document.uploadedBy = req.local.user._id;
                    document.createdAt = new Date();
                }
            }))
        }

        var obj = {};
        var tmp_obj = [];

        if (datas.composition) {

            datas.composition.map(value => {
                var key = value.name;
                if (value.name in obj) {
                    obj[value.name] += 1;
                    tmp_obj[key] = obj[value.name] + " " + key;
                }
                else {
                    obj[value.name] = 1;
                    if (key == "Bedroom") {
                        tmp_obj[key] = obj[value.name] + " " + key;
                    } else {
                        tmp_obj[key] = key;
                    }
                }
                if (!value.id) {
                    value.id = Math.floor(100000 + Math.random() * 900000);
                }
                if (value.image) {
                    value.images.map((image) => {
                        if (!image.id) {
                            const img_id = Math.floor(100000 + Math.random() * 900000);
                            image.id = img_id;
                            image.uploadedBy = req.local.user._id;
                            image.createdAt = new Date()
                        }
                    })
                }
            })

            // Sorting array sequence declare below in Array format, You can also add new value if you want.
            let sortingArr = ['Bedroom', 'Living Room', 'Kitchen']
            sortingArr = sortingArr.concat(Object.keys(tmp_obj)).unique();
            tmp_obj = Object.keys(tmp_obj).sort((a, b) => (sortingArr.indexOf(b) != -1) && sortingArr.indexOf(a) - sortingArr.indexOf(b)).reduce((r, k) => (r[k] = tmp_obj[k], r), {})
            datas.briefDescOfProperty = Object.values(tmp_obj).join(" + ");
        }

        if (datas.otherComposition) {
            datas.otherComposition.map(value => {
                if (value.image) {
                    value.images.map((image) => {
                        if (!image.id) {
                            const img_id = Math.floor(100000 + Math.random() * 900000);
                            image.id = img_id;
                            image.uploadedBy = req.local.user._id;
                            image.createdAt = new Date()
                        }
                    })
                }
            })
        }

        if (datas.documentPerArea) {
            datas.documentPerArea.map((document) => {
                const doc_id = Math.floor(100000 + Math.random() * 900000);
                document.id = doc_id;
            })
        }
        if (datas.igrRate) {
            if (findCase.cases.rateVerify) {
                if (findCase.cases.rateVerify.status == true) {
                    req.body.caseStatus = "RATEUNVERIFIED";
                    const createCaseLog = await this.createCaseLog(req);
                    await findCase.cases.rateVerifier(user);
                }
            }
        }
        if (datas.compositeRate) {
            if (findCase.cases.rateVerify) {
                if (findCase.cases.rateVerify.status == true) {
                    req.body.caseStatus = "RATEUNVERIFIED";
                    const createCaseLog = await this.createCaseLog(req);
                    await findCase.cases.rateVerifier(user);
                }
            }
        }
        if (datas.indicatorArr) {

            datas.indicatorArr.map(value => {
                if (!value.id) {
                    value.id = Math.floor(100000 + Math.random() * 900000);
                }
            })
            if (findCase.cases.rateVerify) {
                if (findCase.cases.rateVerify.status == true) {
                    req.body.caseStatus = "RATEUNVERIFIED";
                    const createCaseLog = await this.createCaseLog(req);
                    await findCase.cases.rateVerifier(user);
                }
            }
        }
        if (datas.routeMap) {
            if (datas.routeMap.locationMapImg) {
                const locationMapImg = datas.routeMap.locationMapImg;
                if (locationMapImg) {
                    locationMapImg.map(images => {
                        if (!images.id) {
                            const img_id = Math.floor(100000 + Math.random() * 900000);
                            images.id = img_id;
                            images.uploadedBy = req.local.user._id;
                            images.createdAt = new Date()
                        }
                    })
                }
            }
        }

        // addded to restrict to udpate the staff member on case update
        if (!!datas.staff) {
            delete datas.staff;
        }

        const CaseResult = await CaseServices.updateCase(id, datas);
        if (!CaseResult.status) return res.status(500).send(CaseResult);

        req.body.caseStatus = "DATAENTRYSTART";
        const createCaseLog = await this.createCaseLog(req);

        if (CaseResult.cases.composition || CaseResult.cases.otherComposition) {
            const compositionPDF = await this.makePDF(id, CaseResult.cases);
            CaseResult.compositionPDF = compositionPDF;

            return res.status(200).send(CaseResult);
        }

        return res.status(200).send(CaseResult);

    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1219 ~ module.exports.updateCaseValuation= ~ err:", err)
        return res.status(500).send({ status: false, error: JSON.stringify(err) });
    }
};

/**
 * @description This function is used for update caseValuation
 * @param {*} req.params.id from that get caseId and from req.local get caseValuation Object
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.sitePartialInspection = async (req, res) => {
    try {
        const user = req.local.user;
        let datas = req.local.caseValuation;
        datas.updatedBy = user._id;

        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });
        const taskId = req.local.caseValuation.taskId ? req.local.caseValuation.taskId : null;

        const findCase = await CaseServices.findCase(id);
        if (!findCase.status) return res.status(500).send(findCase);
        let caseData = findCase.cases;

        var visitedDate = caseData.siteVisit.visitedDate || [];
        let siteVisit = {};

        if (datas.extrenalImages) {
            datas.extrenalImages = datas.extrenalImages.map((ei, index) => {
                const doc_id = Math.floor(100000 + Math.random() * 900000);
                return !ei.id ? { ...ei, id: doc_id, uploadedBy: req.local.user._id, createdAt: new Date() } : ei
            });
        }

        if (datas.documents) {
            datas.documents = datas.documents.map((ei, index) => {
                const doc_id = Math.floor(100000 + Math.random() * 900000);
                return !ei.id ? { ...ei, id: doc_id, uploadedBy: req.local.user._id, createdAt: new Date() } : ei
            });
        }

        // add to site visit Date
        siteVisit.status = 'COMPLETED';
        siteVisit.dateTime = moment(new Date()).format("YYYY-MM-DD");
        siteVisit.remark = datas.remark;
        visitedDate.push(siteVisit);

        datas.taskStatus = { ...caseData.taskStatus, SITE_INSPECTION: true };
        datas.caseStatus = "SITEVISITCOMPLETED";
        datas.siteVisitType = "PARTIAL";
        datas.siteVisit = { siteVisitStatus: true, visitedDate };

        // addded to restrict to udpate the staff member on case update
        if (!!datas.staff) {
            delete datas.staff;
        }

        const CaseResult = await CaseServices.updateCase(id, datas);
        if (!CaseResult.status) return res.status(500).send(CaseResult);

        req.body.caseStatus = "SITE_VISIT_PARTIAL_COMPLETED";
        const createCaseLog = await this.createCaseLog(req);

        const taskRes = await Task.updateOne({ _id: taskId }, { status: "COMPLETED", completedAt: new Date() });
        console.log("🚀 ~ file: Case.controller.js:1289 ~ module.exports.sitePartialInspection= ~ taskRes:", taskRes)

        return res.status(200).send(CaseResult);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1305 ~ module.exports.sitePartialInspection= ~ err:", err)
        return res.status(500).send({ status: false, error: JSON.stringify(err) });
    }
};

/**
 * @description This function is used for reschedule site visit
 * @param {Object} req.params.id from that get CaseId
 * @param {*} res
 * @returns Object case_res { status: true, cases: object }
 * -{ status: false, error: string }
 */

module.exports.rescheduleSiteVisit = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });
        const taskId = req.body.taskId || null;

        const CaseResult = await CaseServices.findCase(id);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        var sitevisit = {};
        var visitedDate = CaseResult.cases.siteVisit.visitedDate || [];
        var schedule = CaseResult.cases.siteVisit.schedule || [];

        let query = {};
        query.updatedBy = user._id;
        const datas = req.body;
        if (datas.status == "COMPLETED") {

            query['siteVisit.siteVisitStatus'] = true;
            query['siteVisitDate'] = moment(new Date(datas.dateTime)).format("YYYY-MM-DD");
            query['caseStatus'] = "SITEVISITCOMPLETED";
            query['taskStatus.SITE_INSPECTION'] = true;
            query['siteVisitType'] = "FULL";

            // From here call createCaseLog function for create caseLog
            req.body.caseStatus = "SITEVISITCOMPLETED";
            const createCaseLog = this.createCaseLog(req);

            // if site visit complete then add it to visitedDate
            sitevisit.status = datas.status;
            sitevisit.dateTime = datas.dateTime;
            sitevisit.remark = datas.remark;
            visitedDate.push(sitevisit);

            const taskRes = await Task.updateOne({ _id: taskId }, { status: 'COMPLETED', completedAt: new Date() });
        }
        else {
            query['siteVisit.siteVisitStatus'] = false;
            query['caseStatus'] = "REMAININGSITEVISIT";

            // From here call createCaseLog function for create caseLog
            req.body.caseStatus = "REMAININGSITEVISIT";
            const createCaseLog = this.createCaseLog(req);

            query['scheduleSiteVisitDate'] = moment(new Date(datas.dateTime)).format("YYYY-MM-DD"); // for schedule last Date

            sitevisit.dateTime = datas.dateTime;
            sitevisit.remark = datas.remark;
            schedule.push(sitevisit);
        }

        query['siteVisit.visitedDate'] = visitedDate;
        query['siteVisit.schedule'] = schedule;

        const case_res = await CaseServices.updateCase(id, query);
        if (!case_res.status) return res.status(500).send(case_res);

        return res.status(200).send(case_res);

    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1390 ~ module.exports.rescheduleSitevsit= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.updateDates = async (req, res) => {
    try {
        const id = req.params.id;
        const user = req.local.user;

        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });
        const dates = req.local.dates;

        const CaseResult = await CaseServices.findCase(id);
        if (!CaseResult.status || CaseResult.error) return res.status(500).send(CaseResult);

        let resp = {};

        if (dates.reportGeneratedDate) {
            const createCaseLog = await CaseLogServices.createCaseLog({ user: user, organization: CaseResult.cases.organization, caseStatus: "REPORTGENERATED", reason: "MANUAL_REPORT_DATE_UPDATE", case: CaseResult.cases, dateTime: new Date(dates.reportGeneratedDate) })
            resp.reportGeneratedDate = createCaseLog;
        }
        if (dates.siteVisitedDate) {
            let query = {};
            query.updatedBy = user._id;
            var sitevisit = {};
            var visitedDate = CaseResult.cases.siteVisit.visitedDate || [];

            query['siteVisit.siteVisitStatus'] = true;
            query['siteVisitDate'] = moment(new Date(dates.siteVisitedDate)).format("YYYY-MM-DD");

            sitevisit.status = "COMPLETED";
            sitevisit.dateTime = query['siteVisitDate'];
            sitevisit.remark = "MANUAL_IN_TASK";
            visitedDate.push(sitevisit);

            query['siteVisit.visitedDate'] = visitedDate;
            if (resp.reportGeneratedDate.status) {
                query['reportGeneratedDate'] = new Date(dates.reportGeneratedDate);
            }

            const case_res = await CaseServices.updateCase(id, query);
            resp.siteVisitedDate = case_res;
        }

        return res.status(200).send(resp);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1481 ~ module.exports.updateDates= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.addCaseRemark = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        const body = req.body;

        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const data = { $push: { 'remarks': { ...body, createdBy: user, createdAt: new Date(), } } };
        data.updatedBy = user._id;
        if (body.caseStatus) {
            data.caseStatus = body.caseStatus;
        }

        const case_res = await CaseServices.updateCase(id, data);
        if (!case_res.status) return res.status(500).send(case_res);

        return res.status(200).send(case_res);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1672 ~ module.exports.addCaseRemark= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.enableCaseStatus = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const caseStatus = { status: true };
        caseStatus.updatedBy = user._id;

        const case_res = await CaseServices.updateCase(id, caseStatus);
        if (!case_res.status) return res.status(500).send(case_res);

        return res.status(200).send(case_res);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1306 ~ module.exports.enableCaseStatus= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.disableCaseStatus = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const caseStatus = { status: false };
        caseStatus.updatedBy = user._id;

        const case_res = await CaseServices.updateCase(id, caseStatus);
        if (!case_res.status) return res.status(500).send(case_res);

        return res.status(200).send(case_res);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1323 ~ module.exports.disableCaseStatus= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.saveOldOutward = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const case_id = req.params.id;
        const { dayCount, monthCount, number, reportDate } = req.body;

        if (!(case_id && case_id.length != 0)) return res.status(404).send({ status: false, error: "Case Id Not Found" });

        const _case = await CaseServices.findCase(case_id);
        if (!_case.status) return res.status(500).send({ status: false, error: "Case not found" });

        if (!_case.cases.outward.number) {
            const pad = "0000000"
            var new_outward_no = pad.substring(0, pad.length - String(number).length) + number;

            const CaseResult = await CaseServices.updateCase(case_id, { updatedBy: userId, outward: { date: reportDate, number: new_outward_no, dayCount, monthCount, generatedBy: userId, status: true, remark: "MANUAL_GENERATED" }, outwardStatus: "MANUAL_GENERATED" });
            if (!CaseResult.status) return res.status(500).send(CaseResult);
            return res.status(200).send({ status: true, case: CaseResult.cases });
        }
        else {
            return res.status(200).send({ status: true, case: _case.cases, message: "Already have outward number" });
        }
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1351 ~ module.exports.generateOutward= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
}

module.exports.generateOutward = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const case_id = req.params.id;

        if (!(case_id && case_id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const _case = await CaseServices.findCase(case_id);
        if (!_case.status) return res.status(500).send({ status: false, error: "Case not found" });
        
        const todayDateTime = new Date();
        
        if (!_case.cases.outward.number) {

            const case_result = await CaseServices.getAllCases({ 'outward.number': { $exists: true }, type: "CASE", }, { display: { 'outward': 1 }, sort: { 'outward.number': -1 } }, 0, 1);
            if (!case_result.status || case_result.error) return res.status(500).send(case_result);

            const todayDate = new Date();
            if (case_result.cases.length > 0) {
                // check and  plus 1
                const ow = case_result.cases[0].outward;
                var outward_no = parseInt(ow.number) + 1;

                let dayCount = moment(ow.date).isSame(todayDate, 'day') ? parseInt(ow.dayCount) + 1 : 1;
                let monthCount = moment(ow.date).isSame(todayDate, 'month') ? parseInt(ow.monthCount) + 1 : 1;

                const pad = "000000"
                var new_outward_no = pad.substring(0, pad.length - String(outward_no).length) + outward_no;

                const CaseResult = await CaseServices.updateCase(case_id, { updatedBy: userId, outward: { datetime: todayDateTime, date: todayDate, number: new_outward_no, dayCount: dayCount, monthCount: monthCount, generatedBy: userId, status: true, remark: "SYSTEMGENERATED" }, outwardStatus: "SYSTEMGENERATED" });
                if (!CaseResult.status) return res.status(500).send(CaseResult);

                return res.status(200).send({ status: true, case: CaseResult.cases });
            }
            else {
                // new one
                const CaseResult = await CaseServices.updateCase(case_id, { updatedBy: userId, outward: { datetime: todayDateTime, date: todayDate, number: "000001", dayCount: 1, monthCount: 1, generatedBy: userId, status: true }, outwardStatus: "SYSTEMGENERATED" });
                if (!CaseResult.status) return res.status(500).send(CaseResult);
                return res.status(200).send({ status: true, case: CaseResult.cases });
            }
        }
        else {
            return res.status(200).send({ status: true, case: _case.cases, message: "Already have outward number" });
        }
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:1351 ~ module.exports.generateOutward= ~ err:", err)
        return res.status(500).send({ status: false, error: err });
    }
}

/**
 * @description This function is used for convert number into rupees format. example => 230570 - 2,30,570
 * @param {*} number
 * @returns res  i.e. 2,30,570
 */
module.exports.NumberToRupees = async (number) => {

    if (number) {
        let x = number;
        x = x.toString();
        let lastThree = x.substring(x.length - 3);
        let otherNumbers = x.substring(0, x.length - 3);
        if (otherNumbers != '')
            lastThree = ',' + lastThree;
        let res = otherNumbers.replace(/\B(?=(\d{2})+(?!\d))/g, ",") + lastThree;
        return res;
    } else {
        return "0";
    }
}
/**
 * @description This is used for convert to Degrees, Minutes and seconds
 * -This function is call from convertToDMS function
 * @param {String} coordinate
 * @returns String i.e. converted degree, minutes and seconds
 */

module.exports.toDegreesMinutesAndSeconds = async (coordinate) => {
    var absolute = Math.abs(coordinate);
    var degrees = Math.floor(absolute);
    var minutesNotTruncated = (absolute - degrees) * 60;
    var minutes = Math.floor(minutesNotTruncated);
    var seconds = ((minutesNotTruncated - minutes) * 60).toFixed(1);

    return degrees + '\xB0' + minutes + "'" + seconds + '"';
}
/**
 * @description This is used For convert point latLng (DD) to degree latLng (DMS)
 * @param {String} lat
 * @param {String} lng
 * @returns String i.e. converted DMS string
 */
module.exports.convertToDMS = async (lat, lng) => {

    var latitude = await this.toDegreesMinutesAndSeconds(lat);
    var latitudeCardinal = lat >= 0 ? "N" : "S";

    var longitude = await this.toDegreesMinutesAndSeconds(lng);
    var longitudeCardinal = lng >= 0 ? "E" : "W";

    return latitude + latitudeCardinal + " " + longitude + longitudeCardinal;
};

/**
 * @description This function is used for make PDF into table format of composition measurement and other composition measurement
 * @param {String} id // caseId
 * @param {Object} data // case Object
 * @returns Object { filePath: string }
 */

module.exports.makePDF = async (id, data) => {

    try {
        const Id = id;
        const datas = data;

        var compositionRowArr = [];
        var compositionArr = ['Composition :'];
        var otherCompositionArr = ['Other Composition :'];
        compositionRowArr.push(compositionArr);

        if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + id)) {
            fs.mkdirSync(process.env.CDN_PATH + '/cases/' + id, { recursive: true });
        }
        const filePath = process.env.CDN_PATH + "/cases/" + Id + "/composition_calculation_" + Id + ".pdf";

        const doc = new PdfDoc();
        doc.pipe(fs.createWriteStream(filePath));
        if (datas.composition) {

            await Promise.all(datas.composition.map(async (value, i) => {
                compositionArr = [];
                if (value.measurementType == 'Square_Meter') {
                    compositionArr.push(value.name, value.length1, value.width1, value.totalArea)
                }
                else {
                    compositionArr.push(value.name, value.length1 + "'" + value.length2 + '"', value.width1 + "'" + value.width2 + '"', value.totalArea)
                }
                compositionRowArr.push(compositionArr);
            }));
            delete datas.composition;
        }
        if (datas.otherComposition) {
            compositionRowArr.push(otherCompositionArr);
            await Promise.all(datas.otherComposition.map(async value => {
                var compositionArr = [];
                if (value.measurementType == 'Square_Meter') {
                    compositionArr.push(value.name.label, value.length1, value.width1, value.totalArea)
                }
                else {
                    compositionArr.push(value.name.label, value.length1 + "'" + value.length2 + '"', value.width1 + "'" + value.width2 + '"', value.totalArea)
                }
                compositionRowArr.push(compositionArr);
            }))
            delete datas.otherComposition;
        }

        const table0 = { headers: ['Name', 'Length', 'Width', 'Total'], rows: compositionRowArr };

        doc.text(JSON.stringify(datas, null, 2));

        doc.table(table0, { prepareHeader: () => doc.font('Helvetica-Bold'), prepareRow: (row, i) => doc.font('Helvetica').fontSize(12) });
        doc.end();
        return { filePath: filePath };
    }

    catch (e) {
        console.log("Error In PDF Generation: ", e);
        // res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used numbers to roman
 * @param {number} num integer number
 * @returns roman
*/
module.exports.romanize = (num) => {
    if (!+num) return false;
    var digits = String(+num).split('');
    var key = ['', 'C', 'CC', 'CCC', 'CD', 'D', 'DC', 'DCC', 'DCCC', 'CM',
        '', 'X', 'XX', 'XXX', 'XL', 'L', 'LX', 'LXX', 'LXXX', 'XC',
        '', 'I', 'II', 'III', 'IV', 'V', 'VI', 'VII', 'VIII', 'IX'];
    var roman = '', i = 3;
    while (i--) roman = (key[+digits.pop() + (i * 10)] || '') + roman;
    return Array(+digits.join('') + 1).join('M') + roman;
}

/**
 * @description This function is used numbers to roman
 * @param {string} str roman string
 * @returns roman
*/
module.exports.deromanize = (str) => {
    var str = str.toUpperCase();
    var validator = /^M*(?:D?C{0,3}|C[MD])(?:L?X{0,3}|X[CL])(?:V?I{0,3}|I[XV])$/;
    var token = /[MDLV]|C[MD]?|X[CL]?|I[XV]?/g;
    var key = { M: 1000, CM: 900, D: 500, CD: 400, C: 100, XC: 90, L: 50, XL: 40, X: 10, IX: 9, V: 5, IV: 4, I: 1 };
    var num = 0, m;
    if (!(str && validator.test(str))) return false;
    while (m = token.exec(str)) num += key[m[0]];
    return num;
}

/**
 * @description This function is used for replace html string through key
 * @param {string} req.params.id from that get CaseId
 * @param {*} res
 * @returns Object { status: true, htmlString: string , caseObj: object }
 * - {status : false, error : string }
 */

module.exports.replaceData = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const case_id = req.params.id;
        const orgId = req.local.user.organization._id;
        const draftReport = req.query.draft === "true" || false;

        if (!(case_id && case_id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        let CaseData = await CaseServices.findCase(case_id);
        if (!CaseData.status) return res.status(500).send({ status: false, error: "Case not found" });

        /**
         * set outward number for case and update case
         */
        // if (!_case.cases.outward.number) {

        //     const case_result = await CaseServices.getAllCases({ 'outward.number': { $exists: true }, type: "CASE", }, { display: { 'outward': 1 }, sort: { 'outward.number': -1 } }, 0, 1);
        //     if (!case_result.status || case_result.error) return res.status(500).send(case_result);

        //     const todayDate = new Date();
        //     if (case_result.cases.length > 0) {
        //         // check and  plus 1
        //         const ow = case_result.cases[0].outward;
        //         var outward_no = parseInt(ow.number) + 1;

        //         let dayCount = moment(ow.date).isSame(todayDate, 'day') ? parseInt(ow.dayCount) + 1 : 1;
        //         let monthCount = moment(ow.date).isSame(todayDate, 'month') ? parseInt(ow.monthCount) + 1 : 1;

        //         const pad = "000000"
        //         var new_outward_no = pad.substring(0, pad.length - String(outward_no).length) + outward_no;

        //         const CaseResult = await CaseServices.updateCase(case_id, { outward: { date: todayDate, number: new_outward_no, dayCount: dayCount, monthCount: monthCount, generatedBy: userId, status: true }, outwardStatus: "SYSTEMGENERATED" });
        //         if (!CaseResult.status) return res.status(500).send(CaseResult);
        //     }
        //     else { // new one
        //         const CaseResult = await CaseServices.updateCase(case_id, { outward: { date: todayDate, number: "000001", dayCount: 1, monthCount: 1, generatedBy: userId, status: true }, outwardStatus: "SYSTEMGENERATED" });
        //         if (!CaseResult.status) return res.status(500).send(CaseResult);
        //     }
        // }

        const Organization = await OrganizationServices.findOne({ _id: CaseData.cases.organization._id });
        const MainOrganization = await OrganizationServices.findOne({ _id: Organization.Organizations.organization._id });
        let case_data = CaseData.cases.toJSON();

        if (case_data.branch) {
            const Branch = await BranchServices.getBranch(case_data.branch._id);
            if (!Branch.status || Branch.error) return res.status(500).send(Branch);
            case_data['branch'] = Branch.branches;
        }

        if (!case_data.outward.number) {
            case_data['outward'] = {
                dayCount: "dd",
                monthCount: "mmm",
                number: "230xxxx"
            };
        }

        let floorNo = '';
        let selectNo = '';
        let plotNo = '';
        let country = '';
        let buildingNo = '';
        let village = '';
        let taluka = '';
        let district = '';
        let city = '';
        let state = '';
        let tahasilNo = '';
        let buildingName = '';
        let selectArea = '';
        let street1 = '';
        let street2 = '';
        let landmark = '';
        let wing = '';
        let projectName = '';
        let oldSurveyNumber = '';
        let newSurveyNumber = '';
        let hissaNo = '';
        let CTSNo = '';
        let blockNo = '';
        let municipalityWardNo = '';
        let khataNo = '';
        let mouza = '';
        let SRO = '';
        let policeStation = '';
        let GUTNo = '';
        let postalCode = '';
        let suburb = '';

        let full_address = case_data['fullAddress'] || "";

        let address = case_data.address;
        let floorWithoutSupOrdinal = "";
        if (address['floorNo']['key'] == "floorNo") {
            floorNo = address['floorNo']['value'];

            if (isNaN(floorNo) == false) {
                var s = ["th", "st", "nd", "rd"],
                    v = floorNo % 100;
                floorWithoutSupOrdinal = floorNo + (s[(v - 20) % 10] || s[v] || s[0])
                floorNo = floorNo + `<sup>${(s[(v - 20) % 10] || s[v] || s[0])}</sup>`
            };

        }
        if (address['selectNo']['key'] == "selectNo") { selectNo = address['selectNo']['value']; }
        if (address['plotNo']['key'] == "plotNo" && address['plotNo']['value'] !== "") { plotNo = " Plot No - " + address['plotNo']['value']; }
        if (address['country']['key'] == "country" && address['country']['value'] !== "") {
            country = "Country - " + address['country']['value'].charAt(0).toUpperCase() + address['country']['value'].slice(1);
        }
        if (address['buildingNo']['key'] == "buildingNo") { buildingNo = address['buildingNo']['value']; }
        if (address['village']['key'] == "village" && address['village']['value'] !== "") { village = "Village - " + address['village']['value'].charAt(0).toUpperCase() + address['village']['value'].slice(1); }
        if (address['taluka']['key'] == "taluka" && address['taluka']['value'] !== "") {
            taluka = "Taluka - " + address['taluka']['value'].charAt(0).toUpperCase() + address['taluka']['value'].slice(1);
        }
        if (address['district']['key'] == "district" && address['district']['value'] !== "") {
            district = "District - " + address['district']['value'].charAt(0).toUpperCase() + address['district']['value'].slice(1);
        }
        if (address['city']['key'] == "city" && address['city']['value'] !== "") {
            city = "City - " + address['city']['value'].charAt(0).toUpperCase() + address['city']['value'].slice(1);
        }
        if (address['state']['key'] == "state" && address['state']['value'] !== "") {
            state = "State - " + address['state']['value'].charAt(0).toUpperCase() + address['state']['value'].slice(1);
        }
        if (address['tahasilNo']['key'] == "tahasilNo" && address['tahasilNo']['value'] !== "") {
            tahasilNo = "Tahasil No - " + address['tahasilNo']['value'];
        }
        if (address['buildingName']['key'] == "buildingName") {
            buildingName = address['buildingName']['value'];
            if (!!buildingName) {
                full_address = full_address.replace(buildingName, `<strong>"${buildingName}"</strong>`);
            }
            if (!!floorWithoutSupOrdinal) {
                full_address = full_address.replace(floorWithoutSupOrdinal, floorNo);
            }
        }
        if (address['selectArea']['key'] == "selectArea") {
            selectArea = address['selectArea']['value'];
        }
        if (address['street1']['key'] == "street1") {
            street1 = address['street1']['value'];
        }
        if (address['street2']['key'] == "street2") {
            street2 = address['street2']['value'];
        }
        if (address['landmark']['key'] == "landmark") {
            landmark = address['landmark']['value'];
        }
        if (address['wing']['key'] == "wing" && address['wing']['value'] !== "") {
            wing = "Wing - " + address['wing']['value'];
        }
        if (address['projectName']['key'] == "projectName") {
            projectName = address['projectName']['value'];
        }
        if (address['oldSurveyNumber']['key'] == "oldSurveyNumber" && address['oldSurveyNumber']['value'] !== "") {
            oldSurveyNumber = "Old Survey No - " + address['oldSurveyNumber']['value'];
        }
        if (address['newSurveyNumber']['key'] == "newSurveyNumber" && address['newSurveyNumber']['value'] !== "") {
            newSurveyNumber = "New Survey No - " + address['newSurveyNumber']['value'];
        }
        if (address['hissaNo']['key'] == "hissaNo" && address['hissaNo']['value'] !== "") {
            hissaNo = "Hissa No - " + address['hissaNo']['value'];
        }
        if (address['CTSNo']['key'] == "CTSNo" && address['CTSNo']['value'] !== "") {
            CTSNo = "CTS No - " + address['CTSNo']['value'];
        }
        if (address['blockNo']['key'] == "blockNo") {
            blockNo = address['blockNo']['value'];
        }
        if (address['municipalityWardNo']['key'] == "municipalityWardNo" && address['municipalityWardNo']['value'] !== "") {
            municipalityWardNo = "Municipality Ward No -  " + address['municipalityWardNo']['value'];
        }
        if (address['khataNo']['key'] == "khataNo") {
            khataNo = address['khataNo']['value'];
        }
        if (address['mouza']['key'] == "mouza" && address['mouza']['value'] !== "") {
            mouza = "Mouza - " + address['mouza']['value'];
        }
        if (address['SRO']['key'] == "SRO") {
            SRO = address['SRO']['value'];
        }
        if (address['policeStation']['key'] == "policeStation") {
            policeStation = address['policeStation']['value'];
        }
        if (address['GUTNo']['key'] == "GUTNo" && address['GUTNo']['value'] !== "") {
            GUTNo = "GUT No - " + address['GUTNo']['value'];
        }
        if (address['postalCode']['key'] == "postalCode") {
            postalCode = address['postalCode']['value'];
        }
        if (address['suburb']['key'] == "suburb") {
            suburb = address['suburb']['value'];
        }

        //======================== ownersName and ownerShipShare =============================
        var ownerName = "";
        var i = 1;
        var len = case_data['owners'].length;

        case_data['owners'].map((value, index) => {
            if (len === 1) { ownerName += value.name; }
            else if (len > 1) {
                ownerName += value.name;
                if (i < len) {
                    if (len - 1 == i) { ownerName += " & "; }
                    else { ownerName += ", "; }
                }
            }
            i += 1;
        })

        let userLength = case_data['owners'].length;
        let ownerString = "";
        let owner_list = [];
        let count = 1;
        case_data['owners'].map((owner, index) => {
            // owner_list.push(owner.name+", ")
            if (userLength === 1) {
                ownerString += owner.name + (owner.ownershipShare !== "" ? "(" + (owner.ownershipShare !== "" ? owner.ownershipShare + "%" : "") + ")" : "");
                if (owner.ownershipShare === "") {
                    owner_list.push("Not Available")
                }
            } else if (userLength > 1) {
                ownerString += owner.name + (owner.ownershipShare !== "" ? "(" + (owner.ownershipShare !== "" ? owner.ownershipShare + "%" : "") + ")" : "");
                if (owner.ownershipShare === "") {
                    owner_list.push("Not Available")
                }
                if (count < userLength) {
                    if (userLength - 1 == count) {
                        ownerString += " & ";
                    }
                    else {
                        ownerString += ", ";
                    }
                }
            }
            count += 1;
        });

        ownerString = owner_list.length === userLength ? "Details of ownership share is not available" : ownerString;

        let document = [];
        let sr_no = 1;
        // ==================================== document String ========================================

        /**
         * for document String
         */
        var from_name = "";
        var to_name = "";
        var document_type = "";
        var document_date = "";
        var purchase_value = "";

        let authStr = "";//"<table style='width: 100%;border: none !important; padding:0px !important; border-collapse:collapse;'>";
        case_data['documents'].map((value) => {

            authStr = "<tr style='width: 100%;'><td style='width: 7%;text-align:center;border: none !important; padding:2px !important;'>" + this.romanize(sr_no) + ")</td> <td style='width: 93%;text-align:justify;border: none !important; padding:2px !important;'> Copy of " + value.documentType.label;
            if (!!value.inputs && !!value.inputs.number) {
                authStr += " " + value.inputs.number.label + "." + value.inputs[value.inputs.number.value];
            }
            if (!!value.inputs && !!value.inputs.documentDate) {
                authStr += " Dated " + moment(value.inputs.documentDate, 'DD/MM/YYYY').format('DD.MM.YYYY');
            }
            if (!!value.inputs && value.inputs.to && !!value.inputs.from && value.displayStatus == true) {
                from_name = value.inputs[value.inputs.from.value];
                to_name = value.inputs[value.inputs.to.value];
                document_type = value.documentType.label;
                document_date = moment(value.inputs.documentDate, 'DD/MM/YYYY').format('DD.MM.YYYY');
                purchase_value = value.inputs.amount;
                // authStr += " " + value.inputs.from.label + " - " + from_name + " " + value.inputs.to.label + " - " + to_name;
            }
            if (!!value.inputs && value.inputs.from) {
                authStr += " between " + value.inputs[value.inputs.from.value] + "(The " + value.inputs.from.label + ")";
            }
            if (!!value.inputs && value.inputs.to) {
                authStr += " And " + value.inputs[value.inputs.to.value] + "(The " + value.inputs.to.label + ")";
            }

            if (!!value.inputs && value.inputs.nameOfApprovingAuthority && Object.keys(value.inputs.nameOfApprovingAuthority).length > 0 && value.inputs.nameOfApprovingAuthority.label) {
                authStr += " issued by " + value.inputs.nameOfApprovingAuthority.label;
            }
            if (value.inputs && value.inputs.remark) {
                authStr += value.inputs.remark;
            }

            authStr += ".</td></tr>";
            document.push(authStr);
            sr_no += 1;
        });
        // authStr += "</table>";
        document.push();

        //  ======================================= approved map / plan authority ==================================
        var date_of_issue = "Approved Building Plan were not provided and not verified";
        var map_approving_authority = "No"
        var comments_approved_plan = "No"

        case_data['documents'].map((value) => {
            console.log("date => ", value.documentType.label, " = ", value.fileName, value.documentType.label == "Approved Building Plan")
            if (value.documentType.label == "Approved Building Plan") {
                date_of_issue = "Copy of Approved building plan "
                if (!!value.inputs && !!value.inputs.number) {
                    date_of_issue += value.inputs.number.label + "." + value.inputs[value.inputs.number.value];
                }
                if (value.inputs.documentDate) {
                    date_of_issue += " dated " + value.inputs.documentDate;
                }

                if (value.inputs.nameOfApprovingAuthority && Object.keys(value.inputs.nameOfApprovingAuthority).length > 0 && value.inputs.nameOfApprovingAuthority.label) {
                    date_of_issue += " issued by " + value.inputs.nameOfApprovingAuthority.label;
                    map_approving_authority = "Yes";
                }

                if (value.inputs.remark) {
                    comments_approved_plan = " Remark - " + value.inputs.remark;
                }

            }
            if (["Occupancy Certificate", "Building Completion Certificate", "Part Occupancy Certificate"].includes(value.documentType.label)) {

                date_of_issue = "As " + value.documentType.label + " is received it may be assumed that the construction is as per sanctioned plan."

                if (value.inputs.nameOfApprovingAuthority && Object.keys(value.inputs.nameOfApprovingAuthority).length > 0 && value.inputs.nameOfApprovingAuthority.label) {
                    map_approving_authority = "Yes";
                }
                if (value.inputs.remark) {
                    comments_approved_plan = " Remark - " + value.inputs.remark;
                }
            }
        });

        // ============================================== Residential/ Commercial/ Industrial =======================
        let residential_area_status = "No";
        let commercial_area_status = "No";
        let industrial_area_status = "No";
        let walls = 'No';

        if (case_data['siteMeasurement']['walls'] && case_data['siteMeasurement']['walls']['label']) {
            walls = case_data['siteMeasurement']['walls']['label'];
        }
        var future_monthly_rent = "";

        if (case_data['typeOfReport']['value'] == "Land_and_Building") {
            // replacing case_data['valuation']['finaltotal'] with  case_data['landAndBuildingDetails'].summaryDetails.fairMarketValue 
            case_data['valuation'] = { ...case_data['valuation'], finaltotal: case_data['landAndBuildingDetails'].summaryDetails.fairMarketValue };
        }
        if (case_data['propertyType'].value == "Residential") {

            residential_area_status = "Yes";
            // residential_area_ = "Residential";

            var future_rent = parseInt(case_data['valuation']['finaltotal'] * 0.025);

            future_rent = Math.round(parseFloat(future_rent / 12).toFixed(2));
            future_rent = 100 * Math.round(future_rent / 100);

            future_monthly_rent = await this.NumberToRupees(future_rent);
            future_monthly_rent += ".00";
        }
        if (case_data['propertyType'].value == "Commercial") {

            commercial_area_status = "Yes";
            // commercial_area = "Commercial";

            var future_rent = parseInt(case_data['valuation']['finaltotal'] * 0.040);
            future_rent = parseFloat(future_rent / 12).toFixed(2);
            future_rent = 100 * Math.round(future_rent / 100);

            future_monthly_rent = await this.NumberToRupees(future_rent);
            future_monthly_rent += ".00";
        }
        if (case_data['propertyType'].value == "Industrial") {
            industrial_area_status = "Yes";
            // industrial_area = "Industrial";

            var future_rent = parseInt(case_data['valuation']['finaltotal'] * 0.040);
            future_rent = parseFloat(future_rent / 12).toFixed(2);
            future_rent = 100 * Math.round(future_rent / 100);

            future_monthly_rent = await this.NumberToRupees(future_rent);
        }

        // ============================================= compositionFlatType ==================================================
        var composition_flat_type = "";

        if (!!case_data['compositionFlatType'] && !!case_data['compositionFlatType']['label']) {
            composition_flat_type = case_data['compositionFlatType']['label'];
        }

        // ============================================= yearOfConstructionAsPer ==================================================
        var year_of_construction_as_per = "";

        if (!!case_data['siteMeasurement'] && !!case_data['siteMeasurement']['yearOfConstructionAsPer'] && !!case_data['siteMeasurement']['yearOfConstructionAsPer']['label']) {
            year_of_construction_as_per = case_data['siteMeasurement']['yearOfConstructionAsPer']['label'];
        }

        // ============================================= reportType ==================================================
        var reportType = "";
        if (case_data['typeOfReport']['label'] == "Single Flat") {
            reportType = "Flat";
        } else {
            reportType = case_data['typeOfReport']['label']
        }

        var type_of_structure = case_data['siteMeasurement']['typeOfStructure']['label'] || "";
        let findStringStructure = 'Structure';

        if (!!type_of_structure && !type_of_structure?.includes(findStringStructure)) {
            type_of_structure += (" " + findStringStructure);
        }

        // ========================================= lift installation =====================================
        var lift = "Not Provided";
        if (case_data['siteMeasurement']['noOfLifts'] !== "" && case_data['siteMeasurement']['noOfLifts'] !== null && !!(parseInt(case_data['siteMeasurement']['noOfLifts']))) {
            lift = case_data['siteMeasurement']['noOfLifts'];
            lift += (lift > 1 ? " Lifts" : "Lift");
        }

        // ========================================= pavement laid =========================================
        // var pavement_laid = "";
        // if (case_data['siteMeasurement']['pavementLaid']['label'] != "" && case_data['siteMeasurement']['pavementLaid']['label'] !== null) {
        //     pavement_laid = case_data['siteMeasurement']['pavementLaid']['label'])
        // }

        // ============================= car parking ===============================
        const car_parking = (case_data['siteMeasurement']['carParking'] ?
            (
                typeof case_data['siteMeasurement']['carParking'] == "object"
                    ?
                    (!!case_data['siteMeasurement']['carParking']['label'])
                        ?
                        case_data['siteMeasurement']['carParking']['label']
                        :
                        ""
                    :
                    (case_data['siteMeasurement']['carParking'] || "")
            ) : "")
        // ========================== Measurement type ===========================
        var measurementType = case_data['composition'].length > 0 && !!("measurementType" in case_data['composition'][0]) && case_data['composition'][0].measurementType.value === "Square_Feet" ? "Sq. Ft." : "Sq. M.";
        // console.log("measurementType ==========>", measurementType)

        // ============================== Rate per unit ================================
        var rate_per_unit = case_data['siteMeasurement'] ? parseFloat(case_data['siteMeasurement']['ageOfBuilding']) >= 5 ? (case_data['compositeRate'] && Object.keys(case_data['compositeRate']).length > 0) ? case_data['compositeRate']['depreciatedCostOfConstruction'] : "0" : case_data['flatRate'] : "0";
        rate_per_unit = Math.round(rate_per_unit)
        // console.log("rate_per_unit =>", rate_per_unit, typeof case_data['compositeRate'])

        //============================ total_composite_rate ========================================

        var total_composite_rate = "";
        if (case_data['compositeRate'] && case_data['compositeRate']['depreciatedCostOfConstruction'] != "") {
            // total_composite_rate = case_data['siteMeasurement'] ? parseFloat(case_data['siteMeasurement']['ageOfBuilding']) >= 5 ? (case_data['compositeRate'] && Object.keys(case_data['compositeRate']).length > 0) ? await this.NumberToRupees(Math.ceil(case_data['compositeRate']['depreciatedCostOfConstruction'])) + ".00" : "0" : case_data['flatRate'] : "0"
            total_composite_rate = await this.NumberToRupees(Math.ceil(case_data['compositeRate']['depreciatedCostOfConstruction'])) + ".00";
        } else {
            total_composite_rate = case_data['siteMeasurement'] ? parseFloat(case_data['siteMeasurement']['ageOfBuilding']) >= 5 ? (case_data['compositeRate'] && Object.keys(case_data['compositeRate']).length > 0) ? await this.NumberToRupees(Math.ceil(case_data['compositeRate']['depreciatedCostOfConstruction'])) + ".00" : "0" : await this.NumberToRupees(Math.ceil(case_data['flatRate'])) + ".00" : "0.00";
        }

        // =========================== building service charge ======================================
        var building_service_rate_rs = await this.NumberToRupees((case_data['buildingServicesRate'] || "").toString().split(".")[0]);
        var building_service_rate = ((case_data['buildingServicesRate'] || "").indexOf(".") >= 0) ? building_service_rate_rs + "." + case_data['buildingServicesRate'].split(".")[1] : building_service_rate_rs + ".00";

        //============================ depreciated_building_rate ======================================

        var depreciated_building_rate = "";
        if (case_data['compositeRate'] && case_data['compositeRate']['depreciatedBuildingRate'] != "") {
            depreciated_building_rate = await this.NumberToRupees(Math.ceil(case_data['compositeRate']['depreciatedBuildingRate'])) + ".00";
        } else {
            depreciated_building_rate = building_service_rate
        }

        // ================================= In case of variation of 20% or more ==========================

        var in_case_of_variation = "It is a foregone conclusion that market value is always more than the RR price. As the RR Rates area Fixed by respective State Government for computing Stamp Duty / Rgstn. Fees. Thus the differs from place to place and Location, Amenities per se as evident from the fact than even RR Rates Decided by Government Differs.";
        var igr_rate = "";

        if (case_data['finalMeasurementType'] == "Sq. Ft.") {

            if (case_data['igrRate']['squareMeterGuideLineRate'] != "") {
                igr_rate = parseFloat(parseFloat(case_data['igrRate']['squareFeetGuideLineRate'] || 0).toFixed(2));
            }
            else {
                igr_rate = parseFloat(case_data['igrRate']['rate'] / 10.7639).toFixed(2);
            }
        }
        else {
            // measurement type = meter;
            if (case_data['igrRate']['squareMeterGuideLineRate'] != "") {
                igr_rate = parseFloat(case_data['igrRate']['squareMeterGuideLineRate']);
            }
            else {
                igr_rate = parseFloat(case_data['igrRate']['rate']);
            }
        }

        // console.log(igr_rate, "============ igr_rate ================");

        var igr_per = ((parseFloat(rate_per_unit) * 100) / igr_rate).toFixed(2);

        var final_per = (igr_per - 100).toFixed(2);

        // console.log(final_per, "============ final_per ================");

        if (final_per >= 20) {
            in_case_of_variation_of_20 = in_case_of_variation;
        }
        else {
            in_case_of_variation_of_20 = "No justification require as variation is less than 20%";
        }

        // ================================== Number to Rupees ======================================


        var final_valuation_area = parseFloat(Math.round(case_data['finalValuationArea'])).toFixed(2);
        if (case_data['typeOfReport']['value'] == "Land_and_Building") {
            let findLandTotalOBJ = case_data['landAndBuildingDetails']['landDetails']['data'].find(({ Survey_No }) => (Survey_No || "").toLowerCase() == "total")
            if (!!findLandTotalOBJ && findLandTotalOBJ['Area']) {
                final_valuation_area = Math.round(findLandTotalOBJ['Area'] || 0).toFixed(2);
            }
        }
        var finalTotal = await this.NumberToRupees(Math.round(case_data['valuation']['finaltotal'])) + ".00";

        var finalValueCount = rate_per_unit * final_valuation_area;

        var finalValue = await this.NumberToRupees(finalValueCount) + ".00";

        var bookValue = (case_data['valuation']['bookValue']) ? await this.NumberToRupees(Math.round(case_data['valuation']['bookValue'])) + ".00" : "";

        var flat_rate_rs = await this.NumberToRupees((case_data['flatRate'] || "0").toString().split(".")[0]);
        var flat_rate = ((case_data['flatRate'] || "").indexOf(".") >= 0) ? flat_rate_rs + "." + case_data['flatRate'].split(".")[1] : flat_rate_rs + ".00";

        var land_others_rate_rs = await this.NumberToRupees((case_data['landOtherRate'] || "0").toString().split(".")[0]);
        var land_others_rate = ((case_data['landOtherRate'] || "").indexOf(".") >= 0) ? land_others_rate_rs + "." + case_data['landOtherRate'].split(".")[1] : land_others_rate_rs + ".00";

        // ================================ Guideline rate  ========================================
        var guidline_rate_sq_mt = "N.A";
        var guidline_rate_sq_ft = "";

        var guideline_value_of_the_property_depreciation = "";
        var guideline_value_of_the_property = "";
        var finalArea = parseFloat(case_data['finalValuationArea']);
        var finalAreaInBuiltup = parseFloat(case_data['finalBuiltupArea']);

        if (case_data['igrFinalRate']['squareMeterGuideLineRate'] != "") {

            guidline_rate_sq_mt_rs = await this.NumberToRupees(parseFloat(case_data['igrFinalRate']['squareMeterGuideLineRate'] || 0).toFixed(2).split(".")[0]);
            guidline_rate_sq_mt = guidline_rate_sq_mt_rs + "." + parseFloat(case_data['igrFinalRate']['squareMeterGuideLineRate'] || 0).toFixed(2).split(".")[1] + " per Sq. M.";
            // guidline_rate_sq_mt = guidline_rate_sq_mt.replace(',.', '.'),

            guidline_rate_sq_ft_rs = await this.NumberToRupees(parseFloat(case_data['igrFinalRate']['squareFeetGuideLineRate'] || 0).toFixed(2).split(".")[0]);
            guidline_rate_sq_ft = guidline_rate_sq_ft_rs + "." + parseFloat(case_data['igrFinalRate']['squareFeetGuideLineRate'] || 0).toFixed(2).split(".")[1] + " per Sq. Ft.";
            // guidline_rate_sq_ft = guidline_rate_sq_ft.replace(',.', '.');

            // ========================= Guideline Rate with depreciation =======================

            // if (case_data['finalMeasurementType'] == "Sq. Ft.") {

            //     var RateInFt = Number(parseFloat(case_data['igrFinalRate']['squareFeetGuideLineRate'] || 0).toFixed(2));
            //     guideline_value_of_the_property_depreciation = finalAreaInBuiltup * RateInFt;
            // }
            // else {

            //     var RateInFt = Number(case_data['igrFinalRate']['squareMeterGuideLineRate']);
            //     guideline_value_of_the_property_depreciation = finalAreaInBuiltup * RateInFt;

            // }
        }

        // var number_mt = parseFloat(case_data['igrRate']['rate']).toFixed(2);
        // var number_ft = parseFloat(case_data['igrRate']['rate'] / 10.7639).toFixed(2);

        // var guidline_rate_sq_mt_without_depreciation_rs = await this.NumberToRupees(number_mt.split(".")[0]);
        // var guidline_rate_sq_mt_without_depreciation = guidline_rate_sq_mt_without_depreciation_rs + "." + number_mt.split(".")[1];

        // var guidline_rate_sq_ft_without_depreciation_rs = await this.NumberToRupees(number_ft.split(".")[0]);
        // var guidline_rate_sq_ft_without_depreciation = guidline_rate_sq_ft_without_depreciation_rs + "." + number_ft.split(".")[1];

        var number_ft = !!case_data['igrRate']['squareFeetGuideLineRate'] ? case_data['igrRate']['squareFeetGuideLineRate'] : 0.00;
        var number_mt = !!case_data['igrRate']['squareMeterGuideLineRate'] ? case_data['igrRate']['squareMeterGuideLineRate'] : 0.00;

        var guidline_rate_sq_mt_without_depreciation = !!case_data['igrRate']['squareMeterGuideLineRate'] ? Number(Math.round(case_data['igrRate']['squareMeterGuideLineRate'])) : 0.00;
        var guidline_rate_sq_mt_without_depreciation_rs = await this.NumberToRupees(guidline_rate_sq_mt_without_depreciation);
        guidline_rate_sq_mt_without_depreciation = guidline_rate_sq_mt_without_depreciation_rs + ".00";

        var guidline_rate_sq_ft_without_depreciation = !!case_data['igrRate']['squareFeetGuideLineRate'] ? Number(Math.round(case_data['igrRate']['squareFeetGuideLineRate'])) : 0.00;
        var guidline_rate_sq_ft_without_depreciation_rs = await this.NumberToRupees(guidline_rate_sq_ft_without_depreciation);
        guidline_rate_sq_ft_without_depreciation = guidline_rate_sq_ft_without_depreciation_rs + ".00";

        // ======================== Guideline value of the property without depreciation  =========================
        // if (!guideline_value_of_the_property_depreciation) {

        //     if (case_data['finalMeasurementType'] == "Sq. Ft.") {
        //         var guideline_value_of_the_property = finalAreaInBuiltup * number_ft
        //     }
        //     else {
        //         var guideline_value_of_the_property = finalAreaInBuiltup * number_mt
        //     }
        // }
        // else {
        //     guideline_value_of_the_property = guideline_value_of_the_property_depreciation;
        // }
        guideline_value_of_the_property = case_data['guidelineDetails']['finalGuidelineValue'] || 0.00;
        if (parseFloat(guideline_value_of_the_property) > 0) {
            guideline_value_of_the_property = await this.NumberToRupees(Math.round(guideline_value_of_the_property)) + ".00";
        }

        if (!case_data['valuation']['finaltotal'] || parseInt(case_data['valuation']['finaltotal']) == 0) {
            return res.status(500).send({ status: false, error: "Case Valuation Not Complated." });
        }

        // ================================ Rupees to Word ========================================
        var rupees_word = Utility.ConvertIntoRupees(Math.round(case_data['valuation']['finaltotal']));
        rupees_word = "Rupees " + rupees_word.replace("Rupees", "");

        // ======================= After analyzing the comparable sale instances ===============
        // const rateArray = [];
        // const carpetRateArray = [];
        // case_data['indicatorArr'].map((indicator, index) => {
        //     let rate = indicator.areaType === "Built_Up" ? indicator.expectedRate : indicator.builtUpExpectedRate;
        //     rate = 100 * Math.round(rate / 100);
        //     rateArray.push(rate);

        //     if (indicator.areaType == "Carpet") {
        //         let CarpetExpectedRate = indicator.expectedPrice / indicator.area;
        //         carpetRateArray.push(Math.round(CarpetExpectedRate).toFixed(2))
        //     } else if (indicator.areaType == "Built_Up") {
        //         let carpetArea = Utility.CarpetToBuiltUpConvert(indicator.area, parseInt(case_data['finalBuiltUpPercentage'] || "0") || 10, true);
        //         let CarpetExpectedRate = indicator.expectedPrice / carpetArea;
        //         carpetRateArray.push(Math.round(CarpetExpectedRate).toFixed(2))
        //     } else if (indicator.areaType == "Saleable") {

        //         let totalBuiltUpArea = (parseFloat(indicator.area) / ((parseInt(indicator.saleablePercentage) / 100) + 1.00)).toFixed(2);
        //         let carpetArea = Utility.CarpetToBuiltUpConvert(totalBuiltUpArea, parseInt(case_data['finalBuiltUpPercentage'] || "0") || 10, true);
        //         let CarpetExpectedRate = indicator.expectedPrice / carpetArea;
        //         carpetRateArray.push(Math.round(CarpetExpectedRate).toFixed(2))
        //     }
        // });

        // let builtUpPriceIndicatorRange = {
        //     min: await this.NumberToRupees(Math.ceil(Math.min(...rateArray))) + ".00",
        //     max: await this.NumberToRupees(Math.ceil(Math.max(...rateArray))) + ".00"
        // };
        // const carpetRangeMin = 100 * Math.round((Math.min(...carpetRateArray) / 100));
        // const carpetRangeMax = 100 * Math.round((Math.max(...carpetRateArray) / 100));

        // let carpetPriceIndicatorRange = {
        //     min: await this.NumberToRupees(carpetRangeMin) + ".00",
        //     max: await this.NumberToRupees(carpetRangeMax) + ".00"
        // };

        // change here for indicator range
        // let builtUpPriceIndicatorRange = "";
        // let carpetPriceIndicatorRange = "";

        // =============================== site visit date =======================================
        // site_v_date = case_data['siteVisit']['visitedDate'];
        var site_visit_date = !!case_data['siteVisitDate'] ? moment(case_data['siteVisitDate'], "YYYY-MM-DD").format("DD.MM.YYYY") : "";
        // await Promise.all(site_v_date.map(async (val, index) => {

        //     if (val.status == "COMPLETED" || val.status == "REVISIT") {
        //         site_visit_date += moment(val['dateTime']).format("DD.MM.YYYY") + " \n";
        //     }
        // }))

        // ================================= carpet Area  as per Doc ====================================

        var list = case_data['documentPerArea'];
        let plint_area_check = "";

        let carpetAsPerDoc = [];
        if (list.length > 0) {
            await list.map((body, index) => {
                let docMeasurementType = "docMeasurementType" in body && "label" in body.docMeasurementType && body.docMeasurementType.label !== "";
                // console.log("true", docMeasurement Type);
                let area = ("docMeasurementType" in body && body.docMeasurementType.label === "Square Feet" || "docMeasurementType" in body && body.docMeasurementType.label === "Square Meter") && body.selectAreaTypeDocument === "Carpet" ? parseFloat(body.totalAreaDocument).toFixed(2) : parseFloat(body.docFinalBuiltUp).toFixed(2);

                if (("docMeasurementType" in body && body.docMeasurementType.label === "Square Feet" && body.selectAreaTypeDocument == "Built_Up")) {
                    plint_area_check = parseFloat(body.totalAreaDocument).toFixed(2);
                    console.log("=> ", plint_area_check, "in built up")
                }
                else {
                    plint_area_check = parseFloat(body.docFinalBuiltUp).toFixed(2);
                    console.log("=> ", plint_area_check, "in else built up")
                }
                console.log("==> ", body.docMeasurementType.label, body.selectedAreaTypeDocument)

                let areaTypeString = body.selectAreaTypeDocument.replace("_", " ") + " Area in ";
                let measurementTypeString = docMeasurementType ? body.docMeasurementType.label === "Square Feet" ? "Sq. Ft." : "Sq. M." : "";
                let docType = "typeOfDocument" in body && "label" in body.typeOfDocument && body.typeOfDocument.label
                if (body.selectAreaTypeDocument === "Carpet") {
                    var carpetAsPerDoc_str = areaTypeString + measurementTypeString + " = " + area;
                    var carpetAsPerDoc_str_asPer = "(Area As Per " + docType + ")";
                    carpetAsPerDoc.push(carpetAsPerDoc_str);
                    carpetAsPerDoc.push(carpetAsPerDoc_str_asPer);
                }
            })
        }

        // =============================== Plinth Area ===========================================
        let plinth_area = "";

        if (case_data.finalPlinthArea && case_data.finalPlinthArea.area) {
            const as_per_plinth = "<br />(" + case_data.finalPlinthArea.areaAsPer + ")";
            plinth_area = "Built Up Area in " + case_data.finalPlinthArea.measurementType + " = " + Math.round(case_data.finalPlinthArea.area).toFixed(2) + as_per_plinth
        }

        // ======================== carpet Area  as per Actual ( site measurement) ========================


        // (("toBeAdded" in this.state && this.state.toBeAdded === "Carpet") ? this.state.final_carpet_area : "")
        //     ((("toBeAdded" in this.state) && this.state.toBeAdded === "Built_Up") ? this.state.final_builtup_area : "")
        //     ("toBeAdded" in this.state && this.state.toBeAdded === "Saleable") ? this.state.final_salable_area : "")
        // console.log("==============================", case_data, "======================== In Console===============");

        let areaType = " Area in ";

        let carpetAsPerActual = "";
        let extent_of_the_site_as_per_actual = "";

        if (case_data['isSiteMeasurementAllowed'] && case_data['composition'].length != 0) {
            carpetAsPerActual = !!parseFloat(case_data['finalCarpetArea'] || "0") ? "Carpet Area in " + measurementType + " = " + case_data.finalCarpetArea + "<br />(Area As per actual site measurement)" : "";

            // const as_per = case_data.finalCarpetArea !== "" ? "(As per Area actual site measurement)" : "Actual site measurement Not Available";

            // carpetAsPerActual = carpetAsPerActual + "<br />" + as_per;
            // console.log("measurementType =--------------------->", measurementType)
            const carpet = !!parseFloat(case_data['finalCarpetArea'] || "0") ? ("Carpet " + areaType + measurementType + " = " + case_data.finalCarpetArea) + "<br /> (Area as per Site measurement) <br />" : "";

            // const builtUp =  !!parseFloat(case_data['finalBuiltupArea'] || "0")? ("BuiltUp " + areaType + measurementType + " = " + case_data.finalBuiltupArea) : "";
            // const salebale =  !!parseFloat(case_data['finalSalableArea'] || "0")? ("Salebale " + areaType + measurementType + " = " + case_data.finalSalableArea) : "";

            // change by karan for report page no 4 and point no.14 remove build up area by umangbhai as per new report requirement 
            extent_of_the_site_as_per_actual = carpet;
            // backup line no 2585 by karan
            // extent_of_the_site_as_per_actual = carpet + builtUp + (case_data?.finalBuiltupArea != case_data?.finalSalableArea ? "<br />" + salebale : "")
        }

        var built_up_area_as_per_document = 0;

        let extent_of_the_site_as_per_doc = [];
        var list = case_data['documentPerArea'];

        if (list.length > 0) {
            await list.map((body, index) => {
                let docMeasurementType = "docMeasurementType" in body && "label" in body.docMeasurementType && body.docMeasurementType.label !== "";

                let area = ("docMeasurementType" in body && body.docMeasurementType.label === "Square Feet") ? parseFloat(body.totalAreaDocument).toFixed(2) : parseFloat(body.squareMeterArea).toFixed(2);

                let builtUpArea = !!body?.docFinalBuiltUp && body.docFinalBuiltUp !== "" ? (body.selectAreaTypeDocument === "Carpet" ? ("docMeasurementType" in body && body.docMeasurementType.label === "Square Feet") ? parseFloat(body.docFinalBuiltUp).toFixed(2) : parseFloat(body.docFinalBuiltUpSquareMeter).toFixed(2) : "") : "";

                // built_up_area_as_per_document = Math.round(builtUpArea);

                built_up_area_as_per_document = body.selectAreaTypeDocument === "Built_Up" ? area : (body.docMeasurementType.label === "Square Feet") ? parseFloat(body.docFinalBuiltUp).toFixed(2) : parseFloat(body.docFinalBuiltUpSquareMeter).toFixed(2);

                let areaTypeString = body.selectAreaTypeDocument.replace("_", " ") + " Area in ";

                let builtUpAreaTypeString = "Built Up" + " Area in ";

                let measurementTypeString = docMeasurementType ? body.docMeasurementType.label === "Square Feet" ? "Sq. Ft." : "Sq. M." : "";

                let docType = "typeOfDocument" in body && "label" in body.typeOfDocument && body.typeOfDocument.label;
                var as_per_doc = "(Area As Per " + docType + ")";

                if (body.selectAreaTypeDocument === "Carpet") {
                    let carpetDoc = areaTypeString + measurementTypeString + " = " + area + "<br />" + as_per_doc;
                    let builtUpCal = "(Carpet Area + " + body.docBuiltupPercentage + "%)";
                    let builtUpDoc = builtUpAreaTypeString + measurementTypeString + " = " + builtUpArea + "<br />" + builtUpCal;

                    // Removed as Tanmay changed
                    // extent_of_the_site_as_per_doc_str = carpetDoc + "<br />" + builtUpDoc;
                    extent_of_the_site_as_per_doc_str = carpetDoc + "<br /><br />" + builtUpDoc;
                    // .join("\n")+

                } else {
                    extent_of_the_site_as_per_doc_str = areaTypeString + measurementTypeString + " = " + area + "<br />" + as_per_doc;
                }

                extent_of_the_site_as_per_doc.push(extent_of_the_site_as_per_doc_str);
            })
        }
        else {
            extent_of_the_site_as_per_doc.push("Document Per Area Not Available ");
        }


        const distressValuePercentage = ((+case_data['valuation']['distressValuePercentage'] || 100) / 100);
        const realizableValuePercentage = ((+case_data['valuation']['realizableValuePercentage'] || 100) / 100);

        const finalTotalRate = (case_data['client'] && case_data['client']['initial'] || "").toLowerCase() == "cb" ? parseFloat(case_data['valuation']['fairMarketValue']['totalFairMarketValue']) : parseFloat(case_data['valuation']['finaltotal'])

        // ========================== Distress value ===========================
        let distressValue = case_data['valuation'] ? parseFloat(finalTotalRate * distressValuePercentage).toFixed(2) : "0";
        let realizableValue = case_data['valuation'] ? parseFloat(finalTotalRate * realizableValuePercentage).toFixed(2) : "0";

        if (case_data['typeOfReport']['value'] == "Land_and_Building") {
            distressValue = (case_data['landAndBuildingDetails'] && case_data['landAndBuildingDetails']['summaryDetails'] && case_data['landAndBuildingDetails']['summaryDetails']['realizableValue']) ? parseFloat(case_data['landAndBuildingDetails']['summaryDetails']['realizableValue']).toFixed(2) : "0";
            realizableValue = (case_data['landAndBuildingDetails'] && case_data['landAndBuildingDetails']['summaryDetails'] && case_data['landAndBuildingDetails']['summaryDetails']['distressValue']) ? parseFloat(case_data['landAndBuildingDetails']['summaryDetails']['distressValue']).toFixed(2) : "0";
        }

        // ========================== Insurable value =============================
        var insurable_count = case_data['valuation'] ? parseFloat(case_data['buildingServicesRate']) * parseFloat(built_up_area_as_per_document || 0) : "";
        var insurable_val_rs = "0.00";
        var insurable_area = parseFloat(built_up_area_as_per_document || 0);

        if (!!insurable_count) {
            insurable_val_rs = await this.NumberToRupees(Math.round(insurable_count)) + ".00";
        } else {
            insurable_area = parseFloat(case_data['guidelineDetails']['selectedBuildUpValue']);
            insurable_val_rs = await this.NumberToRupees(Math.round(parseFloat(case_data['buildingServicesRate']) * insurable_area)) + ".00";
        }
        var insurable_val = {
            rate: building_service_rate,
            totalValue: insurable_val_rs,
            area: parseFloat(insurable_area).toFixed(2) || "0.00"
        }
        console.log("=>\n\n", insurable_area, insurable_val_rs, insurable_val, "\n\n")

        var realizable_in_word = Utility.ConvertIntoRupees(Math.round(realizableValue));
        realizable_in_word = "Rupees " + realizable_in_word.replace("Rupees", "");

        var distress_in_word = Utility.ConvertIntoRupees(Math.round(distressValue));
        distress_in_word = "Rupees " + distress_in_word.replace("Rupees", "");


        distressValue = await this.NumberToRupees(Math.round(distressValue)) + ".00";

        realizableValue = await this.NumberToRupees(Math.round(realizableValue)) + ".00"

        rate_per_unit = await this.NumberToRupees(rate_per_unit);
        rate_per_unit = rate_per_unit + ".00";
        // ============================= for get valure's details =======================================

        let valuer = case_data['staff']['valuers'];
        var valuer_name = "";
        var valuer_father_name = "";
        var valuer_PAN_No = "";
        var designation = "";
        var registration_no = [];
        var declaration_string = "";

        var WealthTaxRegister = "";
        var IBBIRegister = "";

        var ValuerLength = (valuer.length || 0);
        await Promise.all(valuer.map(async (value) => {
            valuer_name += value.firstName + " " + value.lastName + "\n";
            designation = value.designation;
            valuer_father_name = value.fatherName + " " + value.lastName;
            valuer_PAN_No = value.PANNo;
            if (value.valuersRegistrationNumber) {
                value.valuersRegistrationNumber.map((num) => {
                    registration_no.push(num.value);
                    if (num.label == "Wealth Tax") {
                        if (ValuerLength > 1) {
                            WealthTaxRegister += (!!WealthTaxRegister ? " and " + valuer_name : valuer_name);
                        } else {
                            WealthTaxRegister = "I am";
                        }
                    }
                    if (num.label == "IBBI") {
                        if (ValuerLength > 1) {
                            IBBIRegister += (!!IBBIRegister ? " and " + valuer_name : valuer_name);
                        } else {
                            IBBIRegister = "I am";
                        }
                    }
                })
            }
        }))

        if (!!WealthTaxRegister) {
            declaration_string += "<li class='mb-10_5pt'>" + WealthTaxRegister + " registered under Section 34 AB of the Wealth Tax Act, 1957.</li>"
        }
        if (!!IBBIRegister) {
            declaration_string += "<li li class='mb-10_5pt'>" + IBBIRegister + " valuer registered with Insolvency & Bankruptcy Board of India(IBBI).</li>"
        }


        // ============================= for get empanelment details =======================================

        // const zone = await ZoneServices.getZone(case_data.zone._id)
        // if (!zone.status || zone.error) return res.status(500).send(zone);

        // var zoneRes = zone.zones;
        var zoneRes = case_data.zone;
        var empanelment_no = [];

        await Promise.all(valuer.map(async (value) => {

            const EmpanelmentResult = await EmpanelmentServices.find({ query: { zoneId: zoneRes._id, valuerId: value._id }, limit: 100, populate: ['zoneId', 'valuerId'] })

            await EmpanelmentResult.data.map(async (empanelment, eid) => {
                // empanelment is not expired
                // const endDate = moment(empanelment.endDate, "DD/M/YYYY")
                // if (moment(endDate.format("YYYY/MM/DD")).isAfter(moment().format("YYYY/MM/DD"))) {
                if (empanelment.masterTypeOfPurpose == case_data.masterTypeOfPurpose.label && empanelment.assetsClass == case_data.assetsClass.label) {
                    if (empanelment_no.length == 0) { empanelment_no.push(empanelment.empanelmentNo); }
                    // }
                }
            })
        }))

        const caseEmpanelmentNo = await CaseServices.updateCase(case_id, { empanelmentNo: empanelment_no });
        if (!caseEmpanelmentNo.status || caseEmpanelmentNo.error) return res.status(500).send(caseEmpanelmentNo);

        // await case_data.zone.valuers.map(async (data) => {
        //     if (value._id == data._id) {
        //         await data.zoneEmpanelment.map(async (empanelment) => {
        //             await empanelment.propertyType.map(async (propertyValue) => {
        //                 if (propertyValue == propertyType.label) {
        //                     empanelment_no = empanelment.empanelmentNo;
        //                 }
        //             })
        //         })
        //     }
        // })

        //  ================================ age of building =============================

        var age_of_building = "Less than 1 year";

        if (!!case_data['siteMeasurement']['ageOfBuilding'] && case_data['siteMeasurement']['ageOfBuilding'] != 0) {
            age_of_building = case_data['siteMeasurement']['ageOfBuilding'] + " year" + (parseInt(case_data['siteMeasurement']['ageOfBuilding']) > 1 ? "s" : "");
        }

        //  ================================== Number of Floor ================================

        var no_of_floor = "";
        const basement = case_data['siteMeasurement']['noOfBasement'] || "";
        const ground = (case_data['siteMeasurement'] && case_data['siteMeasurement']['groundStilt'] && case_data['siteMeasurement']['groundStilt']['label']) ? case_data['siteMeasurement']['groundStilt']['label'] : "";
        const podium = (case_data['siteMeasurement'] && case_data['siteMeasurement']['noOfPodium']) ? case_data['siteMeasurement']['noOfPodium'] : "";
        const amenityFloors = (case_data['siteMeasurement'] && case_data['siteMeasurement']['noOfAmenityFloors']) ? case_data['siteMeasurement']['noOfAmenityFloors'] : "";
        const noOfFloors = (case_data['siteMeasurement'] && case_data['siteMeasurement']['noOfFloors']) ? case_data['siteMeasurement']['noOfFloors'] : "";

        if (basement !== "" && basement !== null && basement !== "0") {
            no_of_floor += basement + " basement" + (parseInt(basement) > 1 ? "s" : "") + " + ";
        }
        if (ground !== "" && ground !== null) {
            no_of_floor += ground + " + ";
        }
        if (podium !== "" && podium !== null && podium !== "0") {
            no_of_floor += podium + " Podium" + (parseInt(podium) > 2 ? "s" : "") + " + ";
        }
        if (amenityFloors !== "" && amenityFloors !== null && amenityFloors !== "0") {
            no_of_floor += amenityFloors + " Amenity Floor" + (parseInt(amenityFloors) > 1 ? "s" : "") + " + ";
        }
        if (noOfFloors !== "" && noOfFloors !== null && noOfFloors !== "0") {
            no_of_floor += noOfFloors + " Upper Floor" + (parseInt(noOfFloors) > 1 ? "s" : "");
        }
        // "The composition of " + case_data['outgoingDetail']['propertyTypeOfUseList']['label'] + (case_data['typeOfReport']['label'] == "Single Flat") ? "Flat" : case_data['typeOfReport']['label'] + "is" + (case_data['briefDescOfProperty']) ? case_data['briefDescOfProperty'] : ""

        /**
         * var => composition_name
         * description => this is display string of all composition name example : 2 bedroom + 1 kitchen etc.
         */

        var composition_name = ""
        if (reportType == "Flat" && case_data['briefDescOfProperty']) {
            composition_name = "The composition of " + case_data['propertyType'].value + " " + reportType + " is " + case_data['briefDescOfProperty'] + ".";
        }

        // ============================ short name of maker, checker and rate verifier =====================

        const makerShortName = case_data['staff']['maker'] ? case_data['staff']['maker'].shortName : "NM";
        const checkerShortName = case_data['staff']['checker'] ? case_data['staff']['checker'].shortName : "NC";
        const rateVerifierShortName = case_data['staff']['rateVerifier'] ? case_data['staff']['rateVerifier'].shortName : "NR";

        let MCR = "";
        if (makerShortName) { MCR += makerShortName.toUpperCase(); }
        if (makerShortName != checkerShortName) { MCR += checkerShortName.toUpperCase(); }
        if (makerShortName != rateVerifierShortName && checkerShortName != rateVerifierShortName) {
            MCR += rateVerifierShortName.toUpperCase();
        }

        // ====================================== lat lng  ======================================

        var latLng = await this.convertToDMS(case_data['routeMap']['routeMapDetail']['routeLat'], case_data['routeMap']['routeMapDetail']['routeLng'])

        const org_short_name = case_data['organization'].shortName;

        // ====================================== monthly rent details =====================================
        let completion_after = "";
        if (case_data.siteMeasurement && !!case_data.siteMeasurement.constructionWorkStatus && !!case_data.siteMeasurement.constructionWorkStatus.label && case_data.siteMeasurement.constructionWorkStatus.label != "Completed") {
            completion_after = " after completion of construction works";
        }
        let monthly_rent_details = (["details not available", "n.a.", "n.a"].indexOf((case_data['outgoingDetail']['monthlyRent'] || "").toLowerCase()) >= 0) ?
            (future_monthly_rent + ` (Expected rental income per month${completion_after})`)
            :
            (!isNaN(case_data['outgoingDetail']['monthlyRent']))
                ? parseFloat(case_data['outgoingDetail']['monthlyRent']).toFixed(2)
                : case_data['outgoingDetail']['monthlyRent']
        // var monthly_rent_details = (!!case_data['outgoingDetail']['monthlyRent'] && 
        // (["details not available", "n.a.", "n.a"].indexOf(case_data['outgoingDetail']['monthlyRent'] || "".toLowerCase())) >= 0)
        //     ? future_monthly_rent
        //     : (!!case_data['outgoingDetail']['monthlyRent'] && case_data['outgoingDetail']['monthlyRent'] !== "N.A")
        //         ?
        //         (case_data['outgoingDetail']['monthlyRent'] ? !case_data['outgoingDetail']['monthlyRent'].includes('.00')
        //             ? case_data['outgoingDetail']['monthlyRent'] + '.00' : case_data['outgoingDetail']['monthlyRent']
        //             : "").replace('`', "") + ` (${case_data['outgoingDetail']['propertyOccupiedByList']['label']
        //                 && case_data['outgoingDetail']['propertyOccupiedByList']['label'].toLowerCase().includes('tenant') ? 'Present' : 'Expected'} rented income per month${completion_after})`
        //         :
        //         (future_monthly_rent + ` (Expected rental income per month${completion_after})`);

        // ====================================== report generated date =====================================
        let report_gen_date = moment().format('Do MMMM YYYY');
        let report_generate_date = moment().format('DD.MM.YYYY');
        let report_generate_month = moment().format('M');
        let report_generate_year = moment().format('YYYY');
        let report_generate_date_only = moment().format('D');
        // "organization._id": orgId,
        const { caseLogs } = await CaseLogServices.getOneCaseLog({ "case._id": case_id, "caseStatus": "REPORTGENERATED" })

        if (!!caseLogs) {
            // const reportGeneratedDate = moment(caseLogs.dateTime).format();
            const reportGeneratedDate = moment(new Date(caseLogs.dateTime)).format();
            report_gen_date = moment(reportGeneratedDate).format('Do MMMM YYYY');
            report_generate_date = moment(reportGeneratedDate).format('DD.MM.YYYY');
            report_generate_month = moment(reportGeneratedDate).format('M');
            report_generate_year = moment(reportGeneratedDate).format('YYYY');
            report_generate_date_only = moment(reportGeneratedDate).format('D');
        }
        const owner_name = (case_data['ownerName'] || "").trim();
        const contact_person_name = (case_data["contactPersonName"] || "").trim();
        let contect_person_relation_with_owner = case_data['relationWithOwner'] || "";
        // console.log(contact_person_name, owner_name);
        if (owner_name == contact_person_name || ((contect_person_relation_with_owner.toLowerCase()) == "self")) {
            contect_person_relation_with_owner = "Owner";
        }

        // let unauthorized_construction = "";
        // if (case_data.siteMeasurement && !!case_data.siteMeasurement.constructionWorkStatus && !!case_data.siteMeasurement.constructionWorkStatus.label && case_data.siteMeasurement.constructionWorkStatus.label != "Completed") {
        //     if (!!case_data.client && case_data.client.initial && case_data.client.initial == "PNB") {
        //         unauthorized_construction = "N.A., The property under consideration is Rasidential Flat."
        //     }
        // }

        let depreciation_per = "-";
        if ((case_data['compositeRate'] && !!case_data['compositeRate']['depreciationPercentage'] &&
            parseFloat(case_data['compositeRate']['depreciationPercentage']).toFixed(2) > 5)) {
            depreciation_per = parseFloat(case_data['compositeRate']['depreciationPercentage']).toFixed(2) + "%";
        }
        else if ((!!case_data['siteMeasurement']['ageOfBuilding'] && case_data['siteMeasurement']['ageOfBuilding'] < 5)) {
            depreciation_per = "N.A. Age of Property below 5 year";
        }

        var data = {
            organization_name: case_data["organization"].name,
            organization_address: case_data["organization"].address,
            organization_short_name: case_data['organization'].shortName,
            place: case_data['organization'].city,
            org_short_name: org_short_name,//VCIPL
            client_name: case_data['client'].name,
            client_initial_name: case_data['client'].initial,
            address: full_address,
            branch: case_data['branch'].name,
            owner: case_data['ownerName'], // single owner
            owner_name: owner_name,
            owners_name: ownerName,
            case_number: case_data['caseNumber'],
            type_of_report: reportType,
            M_C_R: MCR,

            outward_number: case_data['outward']['number'],
            outward: case_data['outward'],

            final_value: finalValue,
            future_monthly_rent: monthly_rent_details,
            final_total: finalTotal,
            rupees_word: rupees_word,

            documents: document.join(""),
            property_type: case_data['propertyType'].value,
            contact_person_name: contact_person_name,
            contact_person_mobileNo: case_data['siteVisitContactNo'] != "0000000000" ? case_data['siteVisitContactNo'] : "-",
            relation: contect_person_relation_with_owner,
            ownership: case_data['general']['ownership']['label'],
            ownership_share: ownerString,
            townplan_approved_by_list: case_data['general']['townplanApprovedByList']['label'],
            whether_covered_under_any_state: case_data['general']['whetherCoveredUnderAnyState']['label'],
            branch_address: case_data['branch'].address,
            owner_contact_no: case_data['ownerPhoneNo'],
            latitude_longitude: latLng,

            boundaries_of_property_north: case_data['general']['asPerActualAgreement'].north,
            boundaries_of_property_south: case_data['general']['asPerActualAgreement']['south'],
            boundaries_of_property_east: case_data['general']['asPerActualAgreement']['east'],
            boundaries_of_property_west: case_data['general']['asPerActualAgreement']['west'],

            purpose_of_valuation: case_data["purposeOfValuation"]['label'],
            to_assess: case_data['toAssess']['label'],
            client_type: case_data['clientType']['label'].toLowerCase(), //purpose
            owner_type: case_data['general']['ownership']['label'],

            composition_name: composition_name,
            population_group_list: case_data['general']['populationGroupList']['label'],
            population_class_list: case_data['general']['populationClassList']['label'],

            floorNo: floorNo,

            plotNo: plotNo,
            selectNo: selectNo,
            buildingNo: buildingNo,
            village: village,
            country: country,
            tahasilNo: tahasilNo,
            taluka: taluka,
            district: district,
            city: city,
            state: state,
            buildingName: buildingName,
            selectArea: selectArea,
            street1: street1,
            street2: street2,
            landmark: landmark,
            wing: wing,
            projectName: projectName,
            oldSurveyNumber: oldSurveyNumber,
            newSurveyNumber: newSurveyNumber,
            hissaNo: hissaNo,
            CTSNo: CTSNo,
            blockNo: blockNo,
            wardNo: municipalityWardNo,
            khataNo: khataNo,
            mouza: mouza,
            SRO: SRO,
            policeStation: policeStation,
            GUTNo: GUTNo,
            postalCode: postalCode,
            suburb: suburb,

            residential_area: residential_area_status,
            commercial_area: commercial_area_status,
            industrial_area: industrial_area_status,

            date_of_issue: date_of_issue,
            map_approving_authority: map_approving_authority,
            comments_approved_plan: comments_approved_plan,

            as_per_actuals_north: (case_data['general']['asPerActualAgreement']) ? case_data['general']['asPerActualAgreement']['north'] : "",
            as_per_actuals_south: case_data['general']['asPerActualAgreement']['south'],
            as_per_actuals_east: case_data['general']['asPerActualAgreement']['east'],
            as_per_actuals_west: case_data['general']['asPerActualAgreement']['west'],

            as_per_document_north: case_data['general']['asPerDocumentAgreement']['north'] || "-",
            as_per_document_south: case_data['general']['asPerDocumentAgreement']['south'] || "-",
            as_per_document_east: case_data['general']['asPerDocumentAgreement']['east'] || "-",
            as_per_document_west: case_data['general']['asPerDocumentAgreement']['west'] || "-",

            composition_flat_type: composition_flat_type,
            year_of_construction_as_per: year_of_construction_as_per,

            extent_of_the_site_as_per_doc: extent_of_the_site_as_per_doc.join("<br />"),
            extent_of_the_site_as_per_actual: extent_of_the_site_as_per_actual,

            measurement_type: measurementType,
            tobeadd: (case_data['toBeAdded']) ? case_data['toBeAdded'] : " ",
            carpet_total: (case_data['carpetTotal']) ? case_data['carpetTotal'] : " ",
            final_area_type: case_data['finalAreaType'],
            final_valuation_area_string: case_data['finalValuationAreaString'] + "<br />" + "( Area As Per " + case_data['finalAsPerArea'] + ")",

            locality: case_data['siteMeasurement']['planningArea']['label'],
            year_of_construction: (case_data['siteMeasurement'] && case_data['siteMeasurement']['dateOfConstruction'] && case_data['siteMeasurement']['dateOfConstruction']['year']) ? case_data['siteMeasurement']['dateOfConstruction']['year'] : "",

            no_of_floor: no_of_floor,
            dwelling_units: case_data['siteMeasurement']['noOfDwellingUnitsInBuilding'],
            type_of_structure: type_of_structure,
            quality_of_construction: (case_data['siteMeasurement']['qualityOfConstruction'] && case_data['siteMeasurement']['qualityOfConstruction']['label']) ? case_data['siteMeasurement']['qualityOfConstruction']['label'] : "",
            appearance_of_the_building: (case_data['siteMeasurement']['appearanceOfTheBuilding'] && case_data['siteMeasurement']['appearanceOfTheBuilding']['label']) ? case_data['siteMeasurement']['appearanceOfTheBuilding']['label'] : "",
            maintenance_of_the_building: (case_data['siteMeasurement']['maintenanceOfTheBuilding'] && case_data['siteMeasurement']['maintenanceOfTheBuilding']['label']) ? case_data['siteMeasurement']['maintenanceOfTheBuilding']['label'] : "",
            no_of_lift: lift,
            water_source_list: (case_data['siteMeasurement']['waterTank'] && case_data['siteMeasurement']['waterTank']['label']) ? case_data['siteMeasurement']['waterTank']['label'] : "",
            sewerage_system: (case_data['siteMeasurement']['sewerageSystem'] && case_data['siteMeasurement']['sewerageSystem']['label']) ? case_data['siteMeasurement']['sewerageSystem']['label'] : "",
            car_parking: car_parking,
            walls: walls,
            pavement_laid: (case_data['siteMeasurement']['pavementLaid'] ? (case_data['siteMeasurement']['pavementLaid']['label'] != '' && case_data['siteMeasurement']['pavementLaid']['label'] != null ? case_data['siteMeasurement']['pavementLaid']['label'] : "") : ""),
            roof_type: (case_data['flatSpecifications'] && case_data['flatSpecifications']['roof'] && case_data['flatSpecifications']['roof']['label']) ? case_data['flatSpecifications']['roof']['label'] : "",
            flooring_type: (case_data['flatSpecifications'] && case_data['flatSpecifications']['flooring'] && case_data['flatSpecifications']['flooring']['label']) ? case_data['flatSpecifications']['flooring']['label'] : "",
            door_type: (case_data['flatSpecifications'] && case_data['flatSpecifications']['doors'] && case_data['flatSpecifications']['doors']['label']) ? case_data['flatSpecifications']['doors']['label'] : "",
            window_type: (case_data['flatSpecifications'] && case_data['flatSpecifications']['windows'] && case_data['flatSpecifications']['windows']['label']) ? case_data['flatSpecifications']['windows']['label'] : "",
            fitting: (case_data['flatSpecifications'] && case_data['flatSpecifications']['fittings'] && case_data['flatSpecifications']['fittings']['label']) ? case_data['flatSpecifications']['fittings']['label'] : "",
            finishing: (case_data['flatSpecifications'] && case_data['flatSpecifications']['finishing'] && case_data['flatSpecifications']['finishing']['label']) ? case_data['flatSpecifications']['finishing']['label'] : "",

            unit_of_floor: case_data['siteMeasurement']['flatFloorNo'],

            assessment_no: (case_data['outgoingDetail'] && case_data['outgoingDetail']['assessmentNo']) ? case_data['outgoingDetail']['assessmentNo'] : "",
            tax_paid_holder_name: (case_data['outgoingDetail'] && case_data['outgoingDetail']['taxPaidHolderName']) ? case_data['outgoingDetail']['taxPaidHolderName'] : "",
            tax_amount: (case_data['outgoingDetail'] && case_data['outgoingDetail']['taxAmount']) ? case_data['outgoingDetail']['taxAmount'] : "",
            electricity_service_connection_no: (case_data['outgoingDetail'] && case_data['outgoingDetail']['electricityServiceConnectionNo']) ? case_data['outgoingDetail']['electricityServiceConnectionNo'] : "",
            meter_card_holder_name: (case_data['outgoingDetail'] && case_data['outgoingDetail']['meterCardHolderName']) ? case_data['outgoingDetail']['meterCardHolderName'] : "",
            maintenance_of_the_flat: (case_data['outgoingDetail'] && case_data['outgoingDetail']['maintenanceOfFlat']) ? case_data['outgoingDetail']['maintenanceOfFlat'] : "",
            undivided_area_of_land_as_per_sale_deed: (case_data['outgoingDetail'] && case_data['outgoingDetail']['undividedAreaOfLandAsPerSaleDeed']) ? case_data['outgoingDetail']['undividedAreaOfLandAsPerSaleDeed'] : "",
            // carpet_area: (carpetAsPerDoc != "" ? carpetAsPerDoc : (carpetAsPerActual != "" ? carpetAsPerActual : carpetAsPerDoc)),
            // as_per: as_per,
            carpet_as_per_doc: carpetAsPerDoc.join("<br />"),
            carpet_as_per_actual: carpetAsPerActual,
            floor_space_index: (case_data['outgoingDetail'] && case_data['outgoingDetail']['floorSpaceIndex']) ? case_data['outgoingDetail']['floorSpaceIndex'] : "",
            area_status_list: (case_data['outgoingDetail'] && case_data['outgoingDetail']['areaStatusList']) ? case_data['outgoingDetail']['areaStatusList']['label'] : "",
            property_type_of_use_list: (case_data['outgoingDetail'] && case_data['outgoingDetail']['propertyTypeOfUseList']) ? case_data['outgoingDetail']['propertyTypeOfUseList']['label'] : "",
            occupied_by: case_data['outgoingDetail']['propertyOccupiedByList']['label'] ? case_data['outgoingDetail']['propertyOccupiedByList']['label'] : "",
            // monthly_rent: (case_data['outgoingDetail']['monthlyRent'] !== '' && case_data['outgoingDetail']['monthlyRent'] !== "N.A") ? case_data['outgoingDetail']['monthlyRent'].replace('`', "₹") : "",
            monthly_rent: monthly_rent_details,
            tenant_name: (case_data['outgoingDetail']['tenantName'] != '') ? "Tenant Name : " + case_data['outgoingDetail']['tenantName'] : "",

            marketability_list: (case_data['marketability'] && case_data['marketability']['marketabilityList']) ? case_data['marketability']['marketabilityList']['label'] : "",
            any_favouring_for_extra_potential_value: (case_data['marketability'] && case_data['marketability']['anyFavouringForExtraPotentialValue']) ? case_data['marketability']['anyFavouringForExtraPotentialValue'] : "",
            negative_factors_of_affect_market_value: (case_data['marketability'] && case_data['marketability']['negativeFactorsOfAffectMarketValue']) ? case_data['marketability']['negativeFactorsOfAffectMarketValue'] : "",

            plinth_area: plinth_area,
            age_of_building: age_of_building,
            estimated_age_building: case_data['siteMeasurement']['estimatedAge'],
            depreciated_building: (case_data['compositeRate'] && !!case_data['compositeRate']['depreciationRatioOfBuilding']) ? case_data['compositeRate']['depreciationRatioOfBuilding'] : "-",
            distress_value: distressValue,
            distress_in_word: distress_in_word,
            realizable_value: realizableValue,
            realizable_in_word: realizable_in_word,

            // =================== employee details ===================================

            from_name: from_name,
            to_name: to_name,
            document_type: document_type,
            document_date: document_date,
            purchase_value: purchase_value,
            book_value: bookValue,

            valuer_name: valuer_name,
            designation: designation,
            empanelment_no: empanelment_no.join("<br />"),
            registration_no: registration_no.join("<br />"),
            declaration_string: declaration_string,
            valuer_father_name: valuer_father_name,
            valuer_PAN_No: valuer_PAN_No,

            site_engineer: case_data['staff']['siteEngineer'] ? (case_data['staff']['siteEngineer'].firstName + " " + case_data['staff']['siteEngineer'].lastName + " - " + case_data['staff']['siteEngineer'].designation) : "NO SITEENGINEER",
            maker: case_data['staff']['maker'] ? (case_data['staff']['maker'].firstName + " " + case_data['staff']['maker'].lastName + " - " + case_data['staff']['maker'].designation) : "NO MAKER",
            checker: case_data['staff']['checker'] ? (case_data['staff']['checker'].firstName + " " + case_data['staff']['checker'].lastName + " - " + case_data['staff']['checker'].designation) : "NO CHECKER",

            // ===================== Rate ======================
            // builtUpPriceIndicatorRange,
            // carpetPriceIndicatorRange,
            builtUpPriceIndicatorRange: case_data['indicatorRange']['builtUpRateRange'],
            carpetPriceIndicatorRange: case_data['indicatorRange']['carpetRateRange'],
            salablePriceIndicatorRange: case_data['indicatorRange']['saleableRateRange'],
            flat_rate: flat_rate,
            building_service_rate: building_service_rate,
            land_others_rate: land_others_rate,
            remarks: (case_data['compositeRate'] && !!case_data['compositeRate']['remark']) ? case_data['compositeRate']['remark'] : "",

            guidline_rate_sq_mt: guidline_rate_sq_mt,
            guidline_rate_sq_ft: guidline_rate_sq_ft,

            guidline_rate_sq_mt_without_depreciation: guidline_rate_sq_mt_without_depreciation,
            guidline_rate_sq_ft_without_depreciation: guidline_rate_sq_ft_without_depreciation,
            guideline_value: guideline_value_of_the_property,
            in_case_of_variation_of_20: in_case_of_variation_of_20,

            //======================= CopmositeRate Adopted After... ========================
            estimated_age: case_data['siteMeasurement']['estimatedAge'],
            depreciation_per: depreciation_per,

            // depreciated_building_rate: (depreciated_building_rate != "" || depreciated_building_rate != "0") ? depreciated_building_rate + " per " + (case_data['finalMeasurementType'] || 'Sq. Ft.') : "",
            depreciated_building_rate: (depreciated_building_rate != "" || depreciated_building_rate != "0") ? depreciated_building_rate + " per " + (case_data['finalMeasurementType'] || 'Sq. Ft.') : "",

            total_composite_rate: (total_composite_rate != "") ? total_composite_rate + " per " + (case_data['finalMeasurementType'] || 'Sq. Ft.') : "",

            //=========== Details of Valuation ======================

            // present_value_of_flat: (case_data['valuation']['totalothersRate'] == "0" || case_data['valuation']['totalothersRate'] == "") ? "Present value  of the Flat (incl. car parking, if provided)" : "Present value  of the Flat",
            present_value_of_flat: (case_data['valuation']['totalothersRate'] == "0" || case_data['valuation']['totalothersRate'] == "") ? "Present value  of the " + reportType : "Present value  of the " + reportType, // removed by darshan as per discuss with shyam on 15 April 2023

            final_valuation_area: "" + final_valuation_area,
            final_measurement_type: case_data['finalMeasurementType'],
            rate_per_unit: rate_per_unit,
            wardrobes_qty: (case_data['valuation']['wardrobesQty'] == "0") ? "" : case_data['valuation']['wardrobesQty'],
            wardrobes_rate: (case_data['valuation']['wardrobesRate'] == "0") ? "" : case_data['valuation']['wardrobesRate'],
            wardrobes_val: (case_data['valuation']['totalwardrobesRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalwardrobesRate'])) + ".00",
            showcase_qty: (case_data['valuation']['showcasesQty'] == "0") ? "" : case_data['valuation']['showcasesQty'],
            showcase_rate: (case_data['valuation']['showcasesRate'] == "0") ? "" : case_data['valuation']['showcasesRate'],
            showcase_val: (case_data['valuation']['totalshowcasesRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalshowcasesRate'])) + ".00",
            kitchen_qty: (case_data['valuation']['kitchenArrangementsQty'] == "0") ? "" : case_data['valuation']['kitchenArrangementsQty'],
            kitchen_rate: (case_data['valuation']['kitchenArrangementsRate'] == "0") ? "" : case_data['valuation']['kitchenArrangementsRate'],
            kitchen_val: (case_data['valuation']['totalkitchenArrangementsRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalkitchenArrangementsRate'])) + ".00",
            superfine_qty: (case_data['valuation']['superfineFinishQty'] == "0") ? "" : case_data['valuation']['superfineFinishQty'],
            superfine_rate: (case_data['valuation']['superfineFinishRate'] == "0") ? "" : case_data['valuation']['superfineFinishRate'],
            superfine_val: (case_data['valuation']['totalsuperfineFinishRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalsuperfineFinishRate'])) + ".00",
            interior_qty: (case_data['valuation']['interiorDecorationsQty'] == "0") ? "" : case_data['valuation']['interiorDecorationsQty'],
            interior_rate: (case_data['valuation']['interiorDecorationsRate'] == "0") ? "" : case_data['valuation']['interiorDecorationsRate'],
            interior_val: (case_data['valuation']['totalinteriorDecorationsRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalinteriorDecorationsRate'])) + ".00",
            elec_qty: (case_data['valuation']['electricityDepositsQty'] == "0") ? "" : case_data['valuation']['electricityDepositsQty'],
            elec_rate: (case_data['valuation']['electricityDepositsRate'] == "0") ? "" : case_data['valuation']['electricityDepositsRate'],
            elec_val: (case_data['valuation']['totalelectricityDepositsRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalelectricityDepositsRate'])) + ".00",
            extra_collapse_qty: (case_data['valuation']['extraCollapsibleGatesQty'] == "0") ? "" : case_data['valuation']['extraCollapsibleGatesQty'],
            extra_collapse_rate: (case_data['valuation']['extraCollapsibleGatesRate'] == "0") ? "" : case_data['valuation']['extraCollapsibleGatesRate'],
            extra_collapse_val: (case_data['valuation']['totalextraCollapsibleGatesRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalextraCollapsibleGatesRate'])) + ".00",
            potential_qty: (case_data['valuation']['potentialValueQty'] == "0") ? "" : case_data['valuation']['potentialValueQty'],
            potential_rate: (case_data['valuation']['potentialValueRate'] == "0") ? "" : case_data['valuation']['potentialValueRate'],
            potential_val: (case_data['valuation']['totalpotentialValueRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalpotentialValueRate'])) + ".00",
            others_qty: (case_data['valuation']['othersQty'] == "0") ? "" : case_data['valuation']['othersQty'],
            others_rate: (case_data['valuation']['othersRate'] == "0") ? "" : case_data['valuation']['othersRate'],
            others_val: (case_data['valuation']['totalothersRate'] == "0") ? "" : await this.NumberToRupees(Math.round(case_data['valuation']['totalothersRate'])) + ".00",

            insurable_value: insurable_val,
            site_visit_date: site_visit_date,
            report_gen_date: report_gen_date,
            report_generate_date: report_generate_date,
            report_generate_month: report_generate_month,
            report_generate_year: report_generate_year,
            report_generate_date_only: report_generate_date_only,
            case_created_date: moment(case_data['createdAt']).format('DD.MM.YYYY'),
            organization: Organization.Organizations,
            main_organization: MainOrganization.Organizations
        }

        case_data['report_info'] = data;
        // const caseObj = case_data + { report_info: data };
        // console.log("type: ", typeof case_data, typeof data, typeof caseObj);

        await case_data.composition.map(async (data) => {
            await data.images.map(async (img) => {
                img.path = img.path.toString().replace(/\\/gi, "/");
            })
        })

        //   send response based on the api need
        //   if generate-report then call the report generate and then send url received from it
        //   else send replace data normally
        if (req.path.includes("generate-report")) {
            const pdfGenerationResult = await axios.post(`${CONFIG.PDF_GENERATE_URL}${draftReport ? "/draft.php" : ""}/?case_id=${case_data._id}`, JSON.stringify({
                "cases": { ...case_data }
            }));

            if (pdfGenerationResult.status == 200) {
                return res.send({ status: true, ...(typeof pdfGenerationResult.data == "object" ? pdfGenerationResult.data : { error: Object.values(pdfGenerationResult.data).join(""), message: Object.values(pdfGenerationResult.data).join("") }) });
            }
            else {
                return res.send({ status: false, ...(typeof pdfGenerationResult.data == "object" ? pdfGenerationResult.data : { error: Object.values(pdfGenerationResult.data).join(""), message: Object.values(pdfGenerationResult.data).join("") }), ...pdfGenerationResult.error });
            }
        } else {
            return res.send({ status: true, caseObj: case_data, reportInfo: data });
        }
    }
    catch (err) {
        console.log("🚀 ~ module.exports.replaceData= ~ err", err)
        logger.error(`🚀 ~ module.exports.replaceData= ~ err: ${JSON.stringify(err, ["message", "arguments", "type", "name"])}`);
        return res.status(500).send({ status: false, error: err.toString(), message: err.toString() });
    }
};

/**
 * @description This function is used for replace key into reportTemplate
 * @param {String} str that have template string
 * @param {Object} map that have keys and value
 * @returns String str i.e. replaced
 */

module.exports.replaceAll = async (str, map) => {

    try {
        for (key in map) {
            var regex = new RegExp("{" + key + "}", "gi");
            // console.log("+============ Regex ============", regex);
            str = str.replace(regex, (Boolean(map[key]) ? map[key] : ""));
        }
        return str;
    }
    catch (e) {
        console.log("Error In replace code: ", e);
        // res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This function is used for find nearest closed cases in the range of 1 km.
 * @param {String} req.params.id from that get caseId.
 * @param {*} res
 * @returns Object result { "status": true,"cases": array }
 * -{ status: false, error: e }
 */
module.exports.nearByLocation = async (req, res) => {
    try {
        const { id } = req.params;
        const { lat, long, start, end, distance: dis } = req.query;
        const distance = (+dis * 1000) || 2500;
        let query = {
            query: {
                caseStatus: { $in: ["COMPLETED", "REPORTGENERATED", "READYFORDELIVERY", "SENTTODELIVERY", "SIGNEDREPORTATTACH", "DELIVERED"] },
                createdAt: Utility.dateRange(start, end)
            },
            distance,
        };

        if (!!id) {
            const case_res = await CaseServices.findCase(id);
            if (!case_res.status || case_res.error) return res.status(500).send(case_res);
            const _case = case_res.cases;

            if (!(_case && _case.location && _case.location.longitude && _case.location.longitude)) {
                return res.status(501).send({ status: false, error: "No Location Found! " });
            }

            query.coordinates = [parseFloat(_case.location.longitude), parseFloat(_case.location.latitude)];
        }
        else if (!!lat && !!long && parseFloat(long) && parseFloat(lat)) {
            query.coordinates = [parseFloat(long), parseFloat(lat)];
        }
        else {
            return res.status(501).send({ status: false, error: "No Location Provided!" });
        }

        const result = await CaseServices.nearByLocation(query);
        if (!result.status || result.error) return res.status(500).send(result);

        const cases = await result.cases.map((value, index) => { value.rate = value.flatRate; return value; });
        return res.status(200).send({ status: true, cases });
    }
    catch (e) {
        logger.error("case controller 3207 nearbyLocation error : " + e.toString());
        return res.status(500).send({ status: false, error: e.toString() });
    }
}

/**
 * @description This function is used for find nearest closed cases in the range of 1 km.
 * @param {String} req.params.id from that get caseId.
 * @param {*} res
 * @returns Object result { "status": true,"cases": array }
 * -{ status: false, error: e }
 */
module.exports.searchByAddress = async (req, res) => {
    try {
        const { search } = req.query;
        let query = {};

        if (search) {
            query['$and'] = [];
            query['$and'].push({
                '$or': [
                    { fullAddress: { $regex: search, $options: 'i' } }
                ]
            })
        };

        const result = await CaseServices.find({ query, display: { 'fullAddress': 1, caseNumber: 1, location: 1 } });
        if (!result.status || result.error) return res.status(500).send(result);

        return res.status(200).send({ status: true, cases: result.cases });
    }
    catch (e) {
        console.log("🚀 ~ file: Case.controller.js:3353 ~ module.exports.searchByAddress= ~ e:", e)
        return res.status(500).send({ status: false, error: e });
    }
}

// for calculate fees from propertyValue
module.exports.feeCalculate = async (req, res) => {
    try {
        const client = req.body.client;
        const branch = req.body.branch;
        const zone = req.body.zone;
        const valuer = req.body.valuer;
        const propertyType = req.body.propertyType;

        const zone_res = await ZoneServices.getOneZone({
            _id: zone, 'client._id': client, 'valuers._id': valuer, //'valuers.zoneEmpanelment.propertyType' : propertyType
        });
        if (!zone_res.status || zone_res.error) return res.status(500).send(zone_res);

        var status = false;
        let value;
        if (zone_res.zones != null) {

            await zone_res.zones.valuers.map(async empanelments => {

                await empanelments.zoneEmpanelment.map(async fees => {

                    var obj = fees.feeStructure;

                    for (const key in obj) {

                        if (key == propertyType) {
                            // console.log(propertyType);
                            status = true;
                            value = obj[key];
                            // return res.status(200).send({ status: true, feeStructure: key });
                        }
                    }
                })
            })

            if (status == true) {
                return res.status(200).send({ status: true, feeStructure: value });
            }
            return res.status(500).send({ status: false, error: "Fee Structure Not Found" });
        }
        else {
            return res.status(500).send({ status: false, error: "Data Not Found" });
        }
    }
    catch (e) {
        console.log(e);
        return res.status(500).send(e);
    }
}

module.exports.externalDocumentImageUpdate = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const caseId = req.params.id;
        const documentType = JSON.parse(req.body.documentType || "") || {};
        const files = req.files ? req.files : [];

        if (files && files.length > 0) {

            const docRes = await DocumentServices.getOneDocumentList({ _id: documentType.value });
            if (!docRes.status || !docRes.documentLists) { return res.status(400).send({ status: false, error: "Document not found" }) }
            const document = docRes.documentLists && docRes.documentLists.document.caseStatus ? { caseStatus: docRes.documentLists.document.caseStatus.value } : {};

            const caseRes = await CaseServices.findCase({ _id: caseId });
            if (!caseRes.status || !caseRes.cases) { return res.status(400).send({ status: false, error: "Case not found" }) }

            const destinationPath = path.join(process.env.CDN_PATH, "cases", caseId, 'externalDocuments');
            const pdfRes = await generatePDF(files, destinationPath);

            if (pdfRes.status) {
                pdfRes.documentType = documentType || {};
                const docs = { ...pdfRes, createdAt: new Date(), uploadedBy: userId };

                const updateCaseRes = await CaseServices.updateCase(caseId, { updatedBy: userId, "$push": { externalDocuments: docs }, ...document })
                return res.status(200).send({ status: updateCaseRes.status, externalDocuments: updateCaseRes.cases.externalDocuments, ...document })
            } else {
                fs.writeFile(process.cwd() + "/errors/" + caseId + '.txt', pdfRes.path, function (err) {
                    if (err) throw err;
                    console.log("🚀 ~ file: Case.controller.js:3245 ~ err:", err)
                });
                return res.status(500).send(pdfRes);
            }
        }
        return res.status(200).send({ status: true, message: "No files found." })
    }
    catch (e) {
        console.log(e);
        return res.status(500).send({ status: false, error: JSON.stringify(e) });
    }
}

module.exports.externalDocumentUpdate = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const caseId = req.params.id;
        const documentType = JSON.parse(req.body.documentType || "") || {};
        const files = req.files ? req.files : [];

        if (req.files && req.files.length > 0) {

            const docRes = await DocumentServices.getOneDocumentList({ _id: documentType.value });
            if (!docRes.status || !docRes.documentLists) { return res.status(400).send({ status: false, error: "Document not found" }) }
            const document = docRes.documentLists && docRes.documentLists.document.caseStatus ? { caseStatus: docRes.documentLists.document.caseStatus.value } : {};

            const caseRes = await CaseServices.findCase({ _id: caseId });
            if (!caseRes.status || !caseRes.cases) { return res.status(400).send({ status: false, error: "Case not found" }) }

            if (!fs.existsSync(process.env.CDN_PATH + '/cases/' + caseId + '/externalDocuments')) {
                fs.mkdirSync(process.env.CDN_PATH + '/cases/' + caseId + '/externalDocuments', { recursive: true });
            }
            // console.log("=> ", docRes, document)
            const docs = await Promise.all(files.map(async (f, id) => {

                const destination = path.join(process.env.CDN_PATH, "cases", caseId, 'externalDocuments', f.filename);

                if (fs.existsSync(f.path)) {
                    // call function for move file
                    await utilitiesController.moveFile(f.path, destination);
                    const filepath = path.join("cases", caseId, 'externalDocuments', f.filename);

                    f.path = filepath;
                    f.documentType = { value: documentType.value, label: documentType.label };
                }
                else {
                    fs.writeFile(process.cwd() + "/errors/" + caseId + '.txt', f.path, function (err) {
                        if (err) throw err;
                    });
                }

                return { ...f, createdAt: new Date(), uploadedBy: userId };
            }));

            const updateCaseRes = await CaseServices.updateCase(caseId, { updatedBy: userId, "$push": { externalDocuments: docs }, ...document })

            return res.status(200).send({ status: updateCaseRes.status, externalDocuments: updateCaseRes.cases.externalDocuments, ...document })
        }

        return res.status(200).send({ status: true, message: "No files found." })
    }
    catch (e) {
        console.log(e);
        return res.status(500).send({ status: false, error: JSON.stringify(e) });
    }
}

module.exports.processingDocumentUpdate = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const caseId = req.params.id;
        const files = req.files ? req.files : [];

        if (req.files && req.files.length > 0) {

            const caseRes = await CaseServices.findCase({ _id: caseId });
            if (!caseRes.status || !caseRes.cases) { return res.status(400).send({ status: false, error: "Case not found" }) }

            fs.mkdirSync(process.env.CDN_PATH + '/cases/' + caseId + '/processingDocuments', { recursive: true });
            const docs = await Promise.all(files.map(async (f, id) => {

                const destination = path.join(process.env.CDN_PATH, "cases", caseId, 'processingDocuments', f.filename);

                if (fs.existsSync(f.path)) {
                    await utilitiesController.moveFile(f.path, destination);
                    const filepath = path.join("cases", caseId, 'processingDocuments', f.filename);

                    f.path = filepath;
                }
                else {
                    fs.writeFile(process.cwd() + "/errors/" + caseId + '.txt', f.path, function (err) { if (err) throw err; });
                }

                return { ...f, createdAt: new Date(), uploadedBy: userId };
            }));

            const updateCaseRes = await CaseServices.updateCase(caseId, { updatedBy: userId, "$push": { processingDocuments: docs } })
            return res.status(200).send({ status: updateCaseRes.status, processingDocuments: updateCaseRes.cases.processingDocuments })
        }

        return res.status(500).send({ status: true, message: "No files found." })
    }
    catch (e) {
        console.log(e);
        return res.status(500).send({ status: false, error: JSON.stringify(e) });
    }
}


module.exports.processingDocumentDelete = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const caseId = req.params.id;
        const fileName = req.query.fileName;

        if (fileName) {
            const caseRes = await CaseServices.findCase({ _id: caseId });

            const docs = await Promise.all(caseRes.cases.processingDocuments.map(async (f, id) => {
                if (fileName != f.filename) { return f; }
                const destination = path.join(process.env.CDN_PATH, "cases", caseId, 'processingDocuments', f.filename);
                if (fs.existsSync(destination)) {
                    await utilitiesController.deleteFile(destination);
                }
                return null;
            }));

            const updateCaseRes = await CaseServices.updateCase(caseId, { updatedBy: userId, processingDocuments: docs.filter((d, id) => !!d) })
            return res.status(200).send({ status: updateCaseRes.status, message: "File deleted successfully.", processingDocuments: updateCaseRes.cases.processingDocuments })
        }

        return res.status(200).send({ status: true, message: "No FileFound!" })
    }
    catch (e) {
        console.log("🚀 ~ file: Case.controller.js:3547 ~ module.exports.externalDocumentDelete ~ e:", e)
        return res.status(500).send({ status: false, error: JSON.stringify(e) });
    }
}

module.exports.externalReportDetails = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const caseId = req.params.id;
        const externalDetails = req.local.externalDetails || {};

        const caseRes = await CaseServices.findCase({ _id: caseId });
        if (!caseRes.status || !caseRes.cases) { return res.status(400).send({ status: false, error: "Case not found" }) }

        const updateCaseRes = await CaseServices.updateCase(caseId, { updatedBy: userId, ...externalDetails })
        return res.status(200).send(updateCaseRes);
    }
    catch (e) {
        console.log(e);
        return res.status(500).send({ status: false, error: JSON.stringify(e) });
    }
}

module.exports.externalDocumentDelete = async (req, res) => {
    try {
        const userId = req.local.user._id;
        const caseId = req.params.id;
        const fileName = req.query.fileName;
        // console.log(typeof files);

        if (fileName) {

            const caseRes = await CaseServices.findCase({ _id: caseId });

            const docs = await Promise.all(caseRes.cases.externalDocuments.map(async (f, id) => {
                if (fileName != f.filename) { return f; }
                const destination = path.join(process.env.CDN_PATH, "cases", caseId, 'externalDocuments', f.filename);
                if (fs.existsSync(destination)) {
                    await utilitiesController.deleteFile(destination);
                }
                return null;
            }));

            const updateCaseRes = await CaseServices.updateCase(caseId, { updatedBy: userId, externalDocuments: docs.filter((d, id) => !!d) })
            return res.status(200).send({ status: updateCaseRes.status, message: "File deleted successfully.", externalDocuments: updateCaseRes.cases.externalDocuments })
        }

        return res.status(200).send({ status: true, message: "No Case Found!" })
    }
    catch (e) {
        console.log(e);
        return res.status(500).send({ status: false, error: JSON.stringify(e) });
    }
}

/**
 * @description This function is used for update caseValuation
 * @param {*} req.params.id from that get caseId and from req.local get caseValuation Object
 * @param {*} res
 * @returns Object CaseResult { status: true, cases: object }
 * -{ status: false, error: string }
 */
module.exports.approxCaseValuation = async (req, res) => {
    try {
        const user = req.local.user;
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const findCase = await CaseServices.findCase(id);
        if (!findCase.status) return res.status(500).send(findCase);

        const datas = req.local.caseValuation;
        datas.updatedBy = user._id;

        if (datas.documentPerArea) {
            datas.documentPerArea.map((document) => {
                if (!document.id) {
                    document.id = Math.floor(100000 + Math.random() * 900000);
                }
            })
        }

        if (datas.indicatorArr) {
            datas.indicatorArr.map(value => {
                if (!value.id) {
                    value.id = Math.floor(100000 + Math.random() * 900000);
                }
            })
        }

        //START below code for convert new schema to old for temperory use.
        let oldFields = {};
        if (!!datas.detailsOfValuation && typeof datas.detailsOfValuation == "object") {
            Object.entries(datas.detailsOfValuation).map(([key, value], id) => {
                if (key == "presentValueOfFlat") {
                    // oldFields[`finalValuationArea`] = value.quantity;
                    // oldFields[``] = value.ratePerUnit;
                    oldFields[`finalValue`] = value.estimatedRate;
                }
                else if (key == "totalValues") {
                    oldFields[`finalTotal`] = value.estimatedRate;
                } else {
                    // oldFields[`title`] = key;
                    oldFields[`${key}Qty`] = value.quantity;
                    oldFields[`${key}Rate`] = value.ratePerUnit;
                    oldFields[`total${key}Rate`] = value.estimatedRate;
                    // oldFields[`status`] = true;
                    // oldFields[`isEditable`] = true;
                }
            })
        }
        //END  below code for convert new schema to old for temperory use.

        // addded to restrict to udpate the staff member on case update
        if (!!datas.staff) {
            delete datas.staff;
        }

        const CaseResult = await CaseServices.updateCase(id, { updatedBy: user._id, ...datas, valuation: oldFields });
        if (!CaseResult.status) return res.status(500).send(CaseResult);

        return res.status(200).send(CaseResult);
    } catch (err) {
        console.log("🚀 ~ file: Case.controller.js:3408 ~ module.exports.approxCaseValuation= ~ err: " + err.toString())
        return res.status(500).send({ status: false, error: err.toString() });
    }
};