const DocumentListServices = require('../Services/DocumentList.services');
const OrganizationServices = require('../Services/Organization.services');

/**
 * @description This is used for get list of documents
 * @param {*} req 
 * @param {*} res 
 * @returns object DocumentListResult { status: true, documentLists: Array }
 * - { status: false, error: string }
 */
module.exports.getAllDocumentList = async (req, res) => {
    try {

        let query = {};
        var count = 0;
        // query['organization._id'] = req.local.user.organization._id;
        const { searchValue, caseStatus } = req.query;

        if (searchValue) {
            query['$or'] = [
                { "document.documentType": { $regex: searchValue, $options: 'i' } }
            ]
        }
        if (!!caseStatus) {
            query['document.caseStatus'] = { $ne: null };
        }

        const DocumentListResult = await DocumentListServices.getAllDocumentList(query, req.local.startIndex, req.local.limit);
        if (!DocumentListResult.status || DocumentListResult.error) return res.status(500).send(DocumentListResult);

        const DocumentListResultCount = await DocumentListServices.getCount(query);
        if (!DocumentListResultCount.status || DocumentListResultCount.error) return res.status(500).send(DocumentListResultCount);

        count = DocumentListResultCount.documentLists;
        return res.status(200).send({ status: true, documentLists: DocumentListResult.documentLists, count: count });
    }
    catch (e) {
        console.log("🚀 ~ file: DocumentList.controller.js:38 ~ module.exports.getAllDocumentList= ~ e:", e)
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};

/**
 * @description This is used for get one Document 
 * @param {*} req.params.id from that get documentId 
 * @param {*} res 
 * @returns object DocumentListResult { status: true, documentLists: object }
 * - { status: false, error: string }
 */
module.exports.getDocumentList = async (req, res) => {
    try {
        const id = req.params.id;
        // console.log(id)
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        const DocumentListResult = await DocumentListServices.getDocumentList(id);
        if (!DocumentListResult.status || DocumentListResult.error) return res.status(500).send(DocumentListResult);

        return res.status(200).send(DocumentListResult);
    }
    catch (e) {
        console.log("🚀 ~ file: DocumentList.controller.js:61 ~ module.exports.getDocumentList= ~ e:", e)
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used  for create document list object for an particular Organization.
 * @param {Object} req.body from that get document Object 
 * @param {*} res 
 * @returns object DocumentListResult  { status: true, documentLists: object }
 * - { status: false, error: string }  
 */
module.exports.createDocumentList = async (req, res) => {
    try {
        let datas = req.body;
        const orgID = req.local.user.organization._id;

        // "organization._id": orgID,
        const DocumentListResult = await DocumentListServices.getAllDocumentList({ "document.documentType": datas.document.documentType });
        if (!DocumentListResult.status || DocumentListResult.error) return res.status(500).send(DocumentListResult);

        if (DocumentListResult.documentLists.length == 0) {

            const Organization = await OrganizationServices.getById(orgID);
            if (!Organization.status) return res.status(500).send(Organization);

            datas.organization = Organization.Organizations;

            const DocumentListResult = await DocumentListServices.createDocumentList(datas);
            if (!DocumentListResult.status || DocumentListResult.error) return res.status(500).send(DocumentListResult);

            return res.status(200).send(DocumentListResult);

        } else {
            return res.status(500).send({ status: false, error: "Already Created!" });
        }
    }
    catch (err) {
        console.log(err)
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update documentList object 
 * @param {*} req.body from that get document Object from req.params.id get documentId
 * @param {*} res 
 * @returns object DocumentListResult { status: true, documentLists: object }
 * - { status: false, error: string } 
 */
module.exports.updateDocumentList = async (req, res) => {
    try {
        // const orgID = req.local.user.organization._id;.
        const id = req.params.id;
        const datas = req.body;

        const DocumentListResult = await DocumentListServices.updateDocumentList(id, datas);
        if (!DocumentListResult.status || DocumentListResult.error) return res.status(500).send(DocumentListResult);

        return res.status(200).send(DocumentListResult);

    } catch (err) {
        console.log(err);
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.deleteDocumentList = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};