const DynamicFormServices = require('../Services/DynamicForm.services');
/**
 * @description This is used for get list of all dynamicform 
 * @param {*} req 
 * @param {*} res 
 * @returns Object DynamicFormResult {status : true , dynamicform : Array }
 * -  {status : false , error : string }
 */
module.exports.getAllDynamicForm = async (req, res) => {
    try {

        let query = {};
        query['orgId'] = req.local.user.organization._id;

        const DynamicFormResult = await DynamicFormServices.getAllDynamicForm(query, req.local.startIndex, req.local.limit);
        if (!DynamicFormResult.status || DynamicFormResult.error) return res.status(500).send(DynamicFormResult);

        return res.status(200).send(DynamicFormResult);
    }
    catch (e) {
        console.log("Error: ", e);
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for get one DynamicForm
 * @param {*} req 
 * @param {*} res 
 * @returns Object DynamicFormResult {status : true , dynamicform : Object }
 * -  {status : false , error : string }
 */
module.exports.getDynamicForm = async (req, res) => {
    try {
        const clientId = req.query.clientId;
        const constructionWorkStatus = req.query.constructionWorkStatus;

        let query = {};
        query['orgId'] = req.local.user.organization._id;
        query['clientId'] = clientId;
        query['constructionWorkStatus'] = constructionWorkStatus;

        const DynamicFormResult = await DynamicFormServices.getDynamicForm(query);
        if (!DynamicFormResult.status || DynamicFormResult.error) return res.status(500).send(DynamicFormResult);

        return res.status(200).send(DynamicFormResult);
    }
    catch (e) {
        console.log("Error: ", e);
        return res.status(500).send({ status: false, error: "Internal Server Error!" });
    }
};
/**
 * @description This is used for create new dynamicform  
 * @param {*} req.local.dynamicForm  from that get dynamicform Object
 * @param {*} res 
 * @returns Object DynamicFormResult {status : true , dynamicform : Object }
 * -  {status : false , error : string }
 * -  { status: false, error: "Already Created!" }
 */
module.exports.createDyanmicForm = async (req, res) => {
    try {
        const datas = req.local.dynamicForm;
        datas.orgId = req.local.user.organization._id;
        // const alredyCreated = await DynamicFormServices.getAllDynamicForm()
        // if (!alredyCreated.status || alredyCreated.error) return res.status(500).send(alredyCreated);
        // // console.log(alredyCreated)
        // if (alredyCreated.dynamicform.length == 0) {

        const DynamicFormResult = await DynamicFormServices.createDynamicForm(datas);
        if (!DynamicFormResult.status || DynamicFormResult.error) return res.status(500).send(DynamicFormResult);

        return res.status(200).send(DynamicFormResult);

        // }
        // else {
        //     return res.status(500).send({ status: false, error: "Already Created!" });
        // }
    }
    catch (err) {
        console.log(err)
        return res.status(500).send({ status: false, error: err });
    }
};
/**
 * @description This is used for update DynamicForm
 * @param {*} req.params.id from that get packageId and from req.local.dynamicForm get dynamicForm Object
 * @param {*} res 
 * @returns Object DynamicFormResult {status : true , dynamicform : Object }
 * -  {status : false , error : string }
 */
module.exports.updateDynamicForm = async (req, res) => {
    try {
        const id = req.params.id;
        if (!(id && id.length != 0)) return res.status(404).send({ status: false, error: "Not Found" });

        // console.log(id);
        const datas = req.local.dynamicForm;

        const DynamicFormResult = await DynamicFormServices.updateDynamicForm(id, datas);
        // console.log("DynamicForm: " + DynamicFormResult);
        if (!DynamicFormResult.status) return res.status(500).send(DynamicFormResult);

        return res.status(200).send(DynamicFormResult);
    }
    catch (err) {
        console.log(err)
        return res.status(500).send({ status: false, error: err });
    }
};

module.exports.deletePackage = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};


