const MasterData = require('../Model/MasterData');
const MasterDataServices = require('../Services/CreateServices')("MasterData");

module.exports.getAll = async (req, res) => {
    try {
        const user = req.local.user;
        let query = req.query;
        let agQuery = [];

        agQuery = [
            { $unwind: { path: "$data" } },
            {
                $match: {
                    "name": query.name || "",
                    ...(query.propertyType && { "data.propertyType": { $in: [query.propertyType] } }),
                    ...(query.typeOfReport && { "data.typeOfReport": { $in: [query.typeOfReport] } }),
                    ...(query.template_name && { "data.template_name": { $in: [query.template_name] } }),
                }
            },
            {
                $group: { _id: "$_id", data: { $addToSet: "$data" } },
            },
            {
                $project: { _id: 0, "data.typeOfReport": 0, "data.propertyType": 0, "data.createdAt": 0 },
            }
        ];
        console.log("🚀 ~ file: MasterData.controller.js:11 ~ module.exports.getAll= ~ agQuery:", agQuery);

        const createMasterDataRes = await MasterDataServices.aggregate(agQuery);
        if (!createMasterDataRes.status || createMasterDataRes.error) return res.status(500).send(createMasterDataRes);

        return res.status(200).send({ status: true, data: createMasterDataRes.data[0]?.data || [] });
    } catch (e) {
        console.log("🚀 ~ file: MasterData.controller.js:11 ~ module.exports.createMasterData= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}

module.exports.get = async (req, res) => {
    try {
        const id = req.params.id;

        const createMasterDataRes = await MasterDataServices.findById(id);
        if (!createMasterDataRes.status || createMasterDataRes.error) return res.status(500).send(createMasterDataRes);

        return res.status(200).send(createMasterDataRes);
    } catch (e) {
        console.log("🚀 ~ file: MasterData.controller.js:25 ~ module.exports.createMasterData= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}

module.exports.createMasterData = async (req, res) => {
    try {
        const user = req.local.user;
        let data = req.local.masterData || {};
        data.createdBy = user._id;
        data.organizationId = user.organization._id;

        const createMasterDataRes = await MasterDataServices.create(data);
        if (!createMasterDataRes.status || createMasterDataRes.error) return res.status(500).send(createMasterDataRes);

        return res.status(200).send({ status: true, data: createMasterDataRes.data });

    } catch (e) {
        console.log("🚀 ~ file: MasterData.controller.js:39 ~ module.exports.createMasterData= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}

module.exports.updateMasterData = async (req, res) => {
    try {
        const id = req.params.id;
        const data = req.local.masterData;
        data.updatedBy = user._id;

        const updateMasterDataRes = await MasterDataServices.updateById(id, data);
        if (!updateMasterDataRes.status || updateMasterDataRes.error) return res.status(500).send(updateMasterDataRes);

        return res.status(200).send(updateMasterDataRes);
    } catch (e) {
        console.log("🚀 ~ file: MasterData.controller.js:54 ~ module.exports.updateMasterData= ~ e:", e)
        return res.status(200).send({ status: false, error: e });
    }
}