const jwt = require('jsonwebtoken');

const config = require('../Constants/Config');
const UserServices = require('../Services/User.services');
const { logger } = require('../logger');

module.exports = async (req, res, next) => {
    try {
        let token = req.headers['authorization'] || req.headers['Authorization'];
        if (!token) return res.status(401).send({ status: false, auth: false, error: 'No token provided.' });

        token = token.split('Bearer ')[1];
        if (!token) return res.status(401).send({ status: false, auth: false, error: 'Invalid Token.' });

        jwt.verify(token, config.secret, async function (err, decoded) {
            if (err) {
                if (err.name === "TokenExpiredError") {
                    return res.status(403).send({ status: false, auth: false, error: 'Authenticate token Expired' });
                }
                else {
                    return res.status(403).send({ status: false, auth: false, error: 'Invalid token.' });
                }
            }

            const user = await UserServices.findById(decoded._id);
            if (!user.users || !user.status) return res.status(404).send({ status: false, auth: false, error: 'User not found!' })

            if (!user.users.status) {
                return res.status(403).send({ status: false, auth: false, error: 'Invalid User Access. Please Contact Administrator.' });
            }
            req.local = { ...decoded, user: user.users };
            next();
        });
    } catch (e) {
        logger.error(`🚀 ~ module.exports= ~ e:`, e.toString())
        return res.status(500).send({ status: false, error: 'Internal Server Error!' });
    }
}