const multer = require('multer');
const fs = require('fs');

if (!fs.existsSync('./public/document')) {
    fs.mkdirSync('./public/document', { recursive: true });
}

const fileStorage = multer.diskStorage({
    destination: (req, file, cb) => {
        // ================================== For  upload Empanelment Letter ========================================
        if (!fs.existsSync('./public/empanelment')) { fs.mkdirSync('./public/empanelment', { recursive: true }); }
        if (file.fieldname === 'empanelmentLetter') { cb(null, './public/empanelment'); }

        // ================================= For upload profile photo ==============================================
        if (!fs.existsSync('./public/profilePhoto')) { fs.mkdirSync('./public/profilePhoto', { recursive: true }); }
        if (file.fieldname === 'profilePhoto') { cb(null, './public/profilePhoto'); }

        // ================================= For upload organization/branch logo photo ==============================================
        if (!fs.existsSync('./public/org_logos')) { fs.mkdirSync('./public/org_logos', { recursive: true }); }
        if (file.fieldname === 'logo') { cb(null, './public/org_logos'); }

        //  =============================== for upload document in pdf ==============================================
        if (!fs.existsSync('./public/document/pdf')) { fs.mkdirSync('./public/document/pdf', { recursive: true }); }
        if (file.fieldname === 'pdf') { cb(null, './public/document/pdf'); }
        if (file.fieldname === 'documents') { cb(null, './public/document'); }

        // for upload document in Image, from here go for convert into pdf 
        if (!fs.existsSync('./public/document/images')) { fs.mkdirSync('./public/document/images', { recursive: true }); }
        if (file.fieldname === 'docs') { cb(null, './public/document/images'); }

        //================================= For upload Employees Documents ==========================================
        if (!fs.existsSync('./public/empDocs')) { fs.mkdirSync('./public/empDocs', { recursive: true }); }
        if (file.fieldname === 'employeeDocs') { cb(null, './public/empDocs'); }

        //================================= For upload Map Images ==========================================
        if (!fs.existsSync('./public/mapImages')) { fs.mkdirSync('./public/mapImages', { recursive: true }); }
        if (file.fieldname === 'locationMapImg' || file.fieldname === 'routeMapImg') { cb(null, './public/mapImages'); }

        // images i.e. Composition Images
        if (!fs.existsSync('./public/images')) { fs.mkdirSync('./public/images', { recursive: true }); }
        if (file.fieldname === 'images') { cb(null, 'public/images'); }

        // images/externalImage i.e. External Image of Building 
        if (!fs.existsSync('./public/images/externalImages')) { fs.mkdirSync('./public/images/externalImages', { recursive: true }); }
        if (file.fieldname === 'externalImages') { cb(null, 'public/images/externalImages'); }

        // ASSETS
        if (!fs.existsSync('./public/assets')) { fs.mkdirSync('./public/assets', { recursive: true }); }
        if (file.fieldname === 'assetImage') { cb(null, 'public/assets'); }

        // EXTERNAL Documents
        if (!fs.existsSync('./public/case/externalDocuments')) { fs.mkdirSync('./public/case/externalDocuments', { recursive: true }); }
        if (file.fieldname === "externalDocuments") { cb(null, 'public/case/externalDocuments'); }

        // PROCESSING DOCUMENTS
        if (!fs.existsSync('./public/case/processingDocuments')) { fs.mkdirSync('./public/case/processingDocuments', { recursive: true }); }
        if (file.fieldname === "processingDocuments") { cb(null, 'public/case/processingDocuments'); }

        // Audio Notes
        if (!fs.existsSync('./public/case/audioNotes')) { fs.mkdirSync('./public/case/audioNotes', { recursive: true }); }
        if (file.fieldname === "audioNotes") { cb(null, 'public/case/audioNotes'); }

        // Invoice 
        if (!fs.existsSync('./public/case/invoice')) { fs.mkdirSync('./public/case/invoice', { recursive: true }); }
        if (file.fieldname === "invoice") { cb(null, 'public/case/invoice'); }

        // Payment
        if (!fs.existsSync('./public/case/payment')) { fs.mkdirSync('./public/case/payment', { recursive: true }); }
        if (file.fieldname === "payment") { cb(null, 'public/case/payment'); }
       
    },
    filename: (req, file, cb) => {

        let fileName = file.originalname;
        // get the extension from filename using javascript
        var ext = fileName.substr(fileName.lastIndexOf('.') + 1);

        // get file name without extension
        fileName = fileName.split('.').slice(0, -1).join('.')

        let FileName = fileName.replace(/[^\w\s]/gi, '');
        let originalFileName = FileName + "." + ext;

        cb(null, Date.now() + '_' + originalFileName.split(' ').join('_'));
    }
});

const fileFilter = (req, file, cb) => {
    console.log("🚀 ~ file: uploadFile.js:89 ~ fileFilter ~ file.fieldname:", file)
    if (
        file.mimetype === 'image/png' ||
        file.mimetype === 'image/jpg' ||
        file.mimetype === 'image/x-icon' ||
        file.mimetype === 'image/jpeg' ||
        file.mimetype === 'application/pdf' ||
        (file.fieldname === "audioNotes" && ["audio/wav", "audio/vnd.wave", "audio/m4a"].includes(file.mimetype)) ||
        (["externalDocuments", "documents"].includes(file.fieldname) && ["application/vnd.openxmlformats-officedocument.wordprocessingml.document", "application/vnd.openxmlformatsofficedocument.wordprocessingml.document", "application/msword", "application/pdf", "application/json"].includes(file.mimetype)) ||
        (["processingDocuments", "import_excel"].includes(file.fieldname) && ["text/csv", "application/vnd.openxmlformats-officedocument.wordprocessingml.document", "application/vnd.openxmlformatsofficedocument.wordprocessingml.document", "application/vnd.ms-excel", "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "application/vnd.ms-powerpoint", "application/msword", "application/pdf", "text/plain", "application/json"].includes(file.mimetype))
    ) { cb(null, true); }
    else { cb(null, false); }
};

module.exports.createOrganization = multer({ storage: fileStorage, fileFilter: fileFilter }).fields([{ name: 'logo', maxCount: 1 }, { name: 'documents', maxCount: 5 }, { name: 'images', maxCount: 8 }]);

module.exports.uploadImage = multer({ storage: fileStorage, fileFilter: fileFilter }).single('image');

module.exports.uploadAsset = multer({ storage: fileStorage, fileFilter: fileFilter }).single('assetImage');

module.exports.uploadImages = multer({ storage: fileStorage, fileFilter: fileFilter }).fields([{ name: 'images', maxCount: 20 }, { name: 'employeeDocs', maxCount: 20 }, { name: 'routeMapImg', maxCount: 20 }, { name: 'empanelmentLetter', maxCount: 20 }, { name: "externalImages", maxCount: 20 }, { name: "waterMarkImage", maxCount: 20 }, { name: "stampImage", maxCount: 20 }]);

module.exports.userProfile = multer({ storage: fileStorage, fileFilter: fileFilter }).single('profilePhoto');
module.exports.logo = multer({ storage: fileStorage, fileFilter: fileFilter }).single('logo');

module.exports.docs = multer({ storage: fileStorage, fileFilter: fileFilter }).array('docs', 50);

module.exports.uploadDocuments = multer({ storage: fileStorage, fileFilter: fileFilter }).fields([{ name: 'pdf', maxCount: 20 }, { name: 'audioNotes', maxCount: 10 }, { name: 'employeeDocs', maxCount: 20 }, { name: 'empanelmentLetter', maxCount: 20 }, { name: "documents", maxCount: 20 }]);

module.exports.uploadEmpanelment = multer({ storage: fileStorage, fileFilter: fileFilter }).array('empanelmentLetter');

module.exports.uploadExternalDocumentImage = multer({ storage: fileStorage, fileFilter: fileFilter }).array('externalDocuments', 5);
module.exports.uploadExternalDocuments = multer({ storage: fileStorage, fileFilter: fileFilter }).array('externalDocuments', 10);

module.exports.uploadProcessingDocuments = multer({ storage: fileStorage, fileFilter: fileFilter }).array('processingDocuments', 10);

// for invoice
module.exports.uploadInvoice = multer({ storage: fileStorage, fileFilter: fileFilter }).single('invoice');

// for payment
module.exports.uploadPayment = multer({ storage: fileStorage, fileFilter: fileFilter }).single('payment');

module.exports.audioRemarks = multer({ storage: fileStorage, fileFilter: fileFilter, limits: { fieldSize: 25 * 1024 * 1024 } }).fields([{ name: 'audioNotes', maxCount: 10 }, { name: "images", maxCount: 20 }]);