const mongoose = require('mongoose');
const moment = require('moment');
const { userSchema } = require('./CommonSchema');

const Schema = mongoose.Schema;

const attendanceSchema = Schema({
    userId: {
        type: Schema.Types.ObjectId,
        ref: 'User',
    },
    orgId: {
        type: Schema.Types.ObjectId,
        ref: 'Organization',
    },
    date: { type: Date, default: new Date(), required: true },
    checkIn: {
        time: { type: Date },
        geo: {
            type: {
                type: String,
                enum: ['Point'],
                default: "Point",
                required: true
            },
            coordinates: {
                type: [Number],
                // index: { type: '2dsphere', spares: true },
                default: [0, 0]
            }
        },
        note: { type: String },
    },
    checkOut: {
        time: { type: Date },
        geo: {
            type: {
                type: String,
                enum: ['Point'],
                default: "Point",
                required: true
            },
            coordinates: {
                type: [Number],
                // index: { type: '2dsphere', spares: true },
                default: [0, 0]
            }
        },
        note: { type: String },
        autoClose: { type: Boolean, defaut: false }
    },
    workingHours: { type: Number },
    present: {
        type: Boolean,
        default: false,
    },
    attendanceType: { type: String, default: "APP" }, // APP, MANUAL_ENTRY
    approvedBy: userSchema,
    approvedAt: {
        type: Date
    },
    logStatus: {
        type: String,
        default: "COMPLETED"  // PENDING, APPROVED, REJECTED, "COMPLETED"
    },
    status: {
        type: Boolean
    },
}, { timestamps: true });

attendanceSchema.index({ "checkIn.geo.coordinates": "2dsphere", "checkOut.geo.coordinates": "2dsphere" });

attendanceSchema.pre('save', function (next) {
    if (this.checkIn.time && this.checkOut.time) {
        const duration = moment.duration(
            moment(this.checkOut.time).diff(moment(this.checkIn.time))
        );
        this.workingHours = duration.asHours();
    }
    next();
});

module.exports = mongoose.model('Attendance', attendanceSchema);