const mongoose = require('mongoose');

const Schema = mongoose.Schema;

const bugTicketSchema = Schema({
    organizationId: {
        type: Schema.Types.ObjectId,
        ref: 'Organization',
    },
    userId: {
        type: Schema.Types.ObjectId,
        ref: 'User',
    },
    title: { type: String },
    ticketNo: { type: Number, default: 1 },
    priority: { type: Number, enum: [0, 1, 2], default: 0 },
    description: { type: String },
    images: [{}],
    audioNotes: [{}],
    logs: [{
        log: {
            type: String,
        },
        createdBy: {
            _id: false,
            userId: {
                type: Schema.Types.ObjectId,
                ref: 'User',
            },
            firstName: { type: String },
            lastName: { type: String },
            role: { type: String },
        },
        createdAt: { type: Date }
    }],
    pickedAt: { type: Date },
    completedAt: { type: Date },
    type: {
        type: String,
        default: "BUG"          // BUG, FEATURE
    },
    createdBy: {
        type: Schema.Types.ObjectId,
        ref: 'User',
    },
    status: {
        type: String,
        default: "OPEN",
        enum: ["OPEN", "MY_TASK", "REVIEW", "TESTING", "COMPLETED", "CANCELLED"]
    }
}, { timestamps: true });

module.exports = mongoose.model('BugTicket', bugTicketSchema);