const mongoose = require('mongoose');
const Schema = mongoose.Schema;
const { userSchema } = require('./CommonSchema');

const paymentSchema = Schema({
    organization: {
        _id: {
            type: Schema.Types.ObjectId,
            ref: 'Organization',
        },
        name: String,
        shortName: String,
        email: String,
        address: String,
        city: String,
        phone: String,
    },
    invoiceIds: [{ type: Schema.Types.ObjectId, ref: 'nInvoice' }],
    caseIds: [{ type: Schema.Types.ObjectId, ref: 'Case' }],
    completedCaseIds: [{ type: Schema.Types.ObjectId, ref: 'Case' }],
    paymentSettlement: [{
        _id: {
            type: Schema.Types.ObjectId,
            ref: 'Invoice'
        },
        invoiceType: { type: String, default: "PROFORMA" },
        isCompleted: { type: Boolean, default: false },
        invoicedAmount: { type: Number, default: 0 },
        receivedAmount: { type: Number, default: 0 },
        pendingAmount: { type: Number, default: 0 },
        cases: [{
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'Case'
            },
            isCompleted: { type: Boolean, default: false },
            invoicedAmount: { type: Number, default: 0 },
            receivedAmount: { type: Number, default: 0 },
            pendingAmount: { type: Number, default: 0 },
        }],
    }],
    paymentType: {
        type: String,
        default: "cash",
        // enum: ["cash", "cheque", "online"]
    },
    number: { type: String, default: "0" },         // cheque number, reference number
    date: { type: Date, }, // for cheque
    note: { type: String },
    discount: { type: Number, default: 0 },
    amount: { type: Number, default: 0 },           // total amount
    nextFollowUpDate: { type: Date, default: Date.now },
    paymentFile: {
        _id: false,
        originalname: String,
        path: String,
        fileName: String,
        createdAt: String,
        size: String,
        documentType: {},
        uploadedBy: {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'User',
            },
            firstName: String,
            lastName: String,
            profilePhoto: String,
        },
        uploadedAt: { type: Date },
    },
    voucher: {
        guid: { type: String },
        date: { type: String },
        dispatchDocNo: { type: String },
        voucher_type: { type: String },
        _voucher_type: { type: String },
        voucher_number: { type: String },
        voucher_amount: { type: Number },
        narration: { type: String },
        party_name: { type: String },
        _party_name: { type: String },
        place_of_supply: { type: String },
        is_invoice: { type: Number },
        is_accounting_voucher: { type: Number },
        is_inventory_voucher: { type: Number },
        is_order_voucher: { type: Number },
        accounting: [{}],
        againstRef: [{}],
        newRef: [{}]
    },
    remarks: [
        {
            remark: { type: String, default: "" },
            createdAt: { type: Date, default: Date.now },
            createdBy: {
                _id: {
                    type: Schema.Types.ObjectId,
                    ref: 'User',
                },
                firstName: String,
                lastName: String,
                profilePhoto: String,
            },
        }
    ],
    collectedBy: userSchema, // who collected cash or cheque = collectedBy
    collectedAt: { type: Date },
    completedBy: userSchema,
    completedAt: { type: Date },
    paymentStatus: {
        type: String,
        default: "PENDING",
        enum: ['PENDING', 'PROCESSING', 'REVIEW', 'COMPLETED']
    },
    tds: {},
    tdsStatus: { type: String, default: "NO", enum: ["NO", "PENDING", "RECEIVED"] },
    createdBy: userSchema,
    isDeleted: { type: Boolean, default: false },
    status: {
        type: Boolean, default: true
    }
}, { timestamps: true });

module.exports = mongoose.model('Payment', paymentSchema);