
const mongoose = require('mongoose');
const { permissionSchema } = require('./CommonSchema');
const Schema = mongoose.Schema;

const userSchema = new Schema({
    firstName: {
        type: String,
    },
    fatherName: {
        type: String,
    },
    lastName: {
        type: String,
    },
    shortName: {
        type: String,
    },
    PANNo: {
        type: String
    },
    email: {
        type: String,
    },
    alternateEmail: {
        type: String
    },
    password: {
        type: String,
        select: false
    },
    designation: {
        type: String
    },
    mobileNo: {
        type: String,
    },
    workMobileNo: {
        type: String,
    },
    emergencyMobileNo: {
        type: String,
    },
    salutation: {
        type: String,
    },
    fax: {
        type: String,
    },
    goodPropertyType: [{
        _id: false,
        value: {
            type: String
        },
        label: {
            type: String
        }
    }],
    birthDate: {
        type: String,
    },
    role: {
        type: String,
    },
    type: {
        type: String,
        required: true,
        default: "EMPLOYEE"
    },
    permissions: {
        organization: permissionSchema,
        lead: permissionSchema,
        case: permissionSchema,
        user: permissionSchema,
        client: permissionSchema,
        zone: permissionSchema,
        branch: permissionSchema,
        roles: permissionSchema,
        account: permissionSchema,
        dropDown: permissionSchema,
        createContact: permissionSchema,
        empanelment: permissionSchema,
        template: permissionSchema,
        documentList: permissionSchema,
    },
    permission: {
        type: Object
    },
    reportTo: {
        type: String,
    },
    notificationToken: {
        type: String,
    },
    jwtToken: {
        type: String,
    },
    resetPasswordToken: {
        type: String,
    },
    remark: {
        type: String,
    },
    branch: {
        _id: {
            type: String
        },
        type: {
            type: String
        },
        sugar_client_id: {
            type: String
        },
        sugar_branch_id: {
            type: String
        },
        branchCode: {
            type: String
        },
        name: {
            type: String,
        },
        email: {
            type: String,
        },
        mobileNo: {
            type: String,
        },
        alternateMobileNo: {
            type: String
        },
        address: {
            type: String,
        },
        pickupPerson: {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'User',
            },
            firstName: {
                type: String,
            },
            lastName: {
                type: String,
            },
            shortName: {
                type: String,
            },
            email: {
                type: String,
            },
            alternateEmail: {
                type: String
            },
            designation: {
                type: String
            },
            mobileNo: {
                type: String,
            },
            workMobileNo: {
                type: String,
            },
            salutation: {
                type: String,
            },
            fax: {
                type: String,
            },
            role: {
                type: String
            },
            reportTo: {
                type: String
            },
            isdelete: {
                type: Boolean
            },
            status: {
                type: Boolean
            }
        },
        client: {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'Clients'
            },
            type: {
                type: String,
            },
            sugar_client_id: {
                type: String
            },
            name: {
                type: String,
            },
            displayName: {
                type: String,
            },
            initial: {
                type: String
            },
            clientKey: {
                type: String
            },
            remark: {
                type: String
            },
            isdelete: {
                type: Boolean
            },
            status: {
                type: Boolean
            }
        }
    },
    orgBranches: [
        {
            _id: {
                type: Schema.Types.ObjectId,
                ref: 'Organization'
            },
            name: { type: String },
            email: { type: String },
            shortName: { type: String },
            address: { type: String },
            type: { type: String },
            logo: { type: String },
            contact: {
                name: String,
                mobileNo: String,
                email: String,
            },
            workTypes: [],
        }
    ],
    organization: {
        _id: {
            type: String
            // type: Schema.Types.ObjectId,
            // ref: 'Organization'
        },
        name: {
            type: String
        },
        email: {
            type: String
        },
        shortName: {
            type: String
        },
        address: {
            type: String
        },
        type: {
            type: String
        },
        status: {
            type: Boolean,
            default: true
        },
        isdelete: {
            type: Boolean,
            default: false
        },
        city: {
            type: String
        },
        organization: {
            _id: {
                type: String
            },
            name: {
                type: String
            },
            email: {
                type: String
            },
            gstNo: {
                type: String
            }
        }
    },
    isdelete: {
        type: Boolean,
        default: false
    },
    status: {
        type: Boolean,
        default: true
    },
    resetPassword: {
        type: Boolean,
        default: false
    },
    registrationNo: [{
        title: {
            value: {
                type: String
            },
            label: {
                type: String
            }
        },
        number: {
            type: String
        }
    }],
    employeeCode: {
        type: String
    },
    profilePhoto: {
        type: String
    },
    employeeDocs: [{
        type: Object
    }],
    qualification: {
        type: String
    },
    workWithDocumentMaker: {
        type: String
    },
    incentiveAccount: {
        type: String
    },
    taskLimit: { type: String, default: 50 },
    bankDetails: {
        bankName: { type: String },
        branchName: { type: String },
        accountNo: { type: String },
        ifscCode: { type: String },
        benificaryName: { type: String },
    },
    address: {
        // street: { type: String },
        city: { type: String },
        state: {
            // value: { type: String },
            // label: { type: String }
            type: String
        },
        country: { type: String },
        postalCode: { type: String },
        description: { type: String }
    },
    assetManagement: [{
        _id: false,
        assetId: {
            type: String,
        },
        assetName: {
            type: String
        },
        status: {
            type: String
        },
        issueDate: {
            type: String
        },
        returnDate: {
            type: String
        }
    }]
}, { timestamps: true });

module.exports = mongoose.model('User', userSchema);