const Branch = require('../Model/Branch');

/**
 * @description This is used for get All branches
 * @param {*} query 
 * @returns Object { status: true, branches: Array }
 *  { status: false, error: String }
 */
module.exports.getAllBranches = async (query, limit, startIndex) => {
    return Branch.find(query).collation({ 'locale': 'en' }).sort({ name: 1 }).skip(startIndex).limit(limit)
        .then((branches) => {
            return { status: true, branches: branches };
        }).catch((e) => {
            return { status: false, error: e };
        });
};

/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, branches: Object }
 *  { status: false, error: String }
 */
 module.exports.getCount = async (query) => {
    return Branch.countDocuments(query).then((branches) => {
        return { status: true, branches: branches };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This function is used for get one branch from requested branch id.  
 * @param {string} id 
 * @returns Object { status : true, branches : object }
 * - { status: false, error: string }
 */
module.exports.getBranch = async (id) => {

    return Branch.findById(id).then((branch) => {
        return { status: true, branches: branch };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This function is used for create branch.
 * @param {Object} branch 
 * @returns Object { status : true, branches : object }
 * - { status: false, error: string }
 */
module.exports.createBranch = async (branch) => {

    return Branch.create(branch)
        .then((branch) => {
            return { status: true, branches: branch }
        })
        .catch((err) => {
            console.log(err)
            return { status: false, err }
        })
};

/**
 * @description This function is used for update branch data
 * @param {String} id 
 * @param {Object} branch_data 
 * @returns Object { status: true, branches: Object}
 * - { status: false, err }
 */
module.exports.editBranch = async (id, branch_data) => {
    return Branch.findOneAndUpdate({ _id: id }, branch_data, { new: true })
        .then((branch) => {
            return { status: true, branches: branch }
        })
        .catch((err) => {
            return { status: false, err }
        })
};

/**
 * 
 * @param {object} find_query 
 * @param {object} branch_data 
 * @returns 
 */
module.exports.editMultipleBranch = async (find_query, branch_data) => {
    return Branch.updateMany(find_query, branch_data, { new: true })
        .then((branch) => {
            return { status: true, branches: branch }
        })
        .catch((err) => {
            return { status: false, err }
        })
};

module.exports.deleteBranch = async (req, res) => {
    // return res.status(200).send({ message: "Hi There Its  Admin" });
};
