const CaseLog = require('../Model/CaseLog');
/**
 * @description This is used for get list of all caseLog
 * @param {*} query 
 * @returns Object { status: true, caseLogs: Array }
 * - { status: false, error: string }
 */
module.exports.getAllCaseLog = async (query = {}) => {

    // console.log(query)
    return CaseLog.find(query).then((CaseLog) => {
        return { status: true, caseLogs: CaseLog };
    }).catch((e) => {
        return { status: false, error: e };
    });

};

// module.exports.deleteCaseLog = async (query = {}) => {

//     // console.log(query)
//     return CaseLog.deleteMany(query).then((CaseLog) => {
//         return { status: true, caseLogs: CaseLog };
//     }).catch((e) => {
//         return { status: false, error: e };
//     });

// };
/**
 * @description This is used for get one caseLog from an Id 
 * @param {String} id 
 * @returns  Object { status: true, caseLogs: Object }
 * - { status: false, error: string }
 */
module.exports.getCaseLog = async (id) => {

    return CaseLog.findById(id).then((CaseLog) => {
        return { status: true, caseLogs: CaseLog };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for get one caseLog from query  
 * @param {*} query 
 * @returns  Object { status: true, caseLogs: Object }
 * - { status: false, error: string }
 */
module.exports.getOneCaseLog = async (query) => {

    return CaseLog.findOne(query).sort({ createdAt: -1 }).then((CaseLog) => {
        return { status: true, caseLogs: CaseLog };
    }).catch((e) => {
        return { status: false, error: e };
    });

};
/**
 * @description This is used for create caseLog 
 * @param {Object} CaseLogData 
 * @returns Object { status: true, caseLogs: Object }
 * - { status: false, error: string }
 */
module.exports.createCaseLog = async (CaseLogData) => {

    return CaseLog.create(CaseLogData)
        .then((result) => {
            // console.log("create CaseLog Service")
            return { status: true, caseLogs: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })
};
/**
 * @description This is used for update caseLog
 * @param {String} id 
 * @param {Object} CaseLogData 
 * @returns Object { status: true, caseLogs: Object }
 * - { status: false, error: string }
 */
module.exports.updateCaseLog = async (id, CaseLogData) => {

    return CaseLog.findOneAndUpdate({ _id: id }, CaseLogData, { new: true })
        .then((result) => {
            return { status: true, caseLogs: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
module.exports.aggregateCaseLog = async (CaseLogData) => {

    return CaseLog.aggregate([
        { $match: { ...CaseLogData } },
        { $sort: { caseStatus: 1, createdAt: 1 } },
        {
            $group:
            {
                _id: "$caseStatus",
                createdAt: { $last: "$createdAt" },
                user: { $first: '$user' }
            }

        }
    ]
    ).then((result) => {
        return { status: true, caseLogs: result };
    })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};