const Case = require('../Model/Case');

const defaultUserSelect = 'firstName lastName email profilePhoto';
const defaultPopulate = [
    { path: 'invoiceId' },
    { path: 'createdBy', select: defaultUserSelect },
    { path: 'outward.generatedBy', select: defaultUserSelect },
    { path: 'inquiryTo' },
    { path: 'leadRemarks.createdBy', select: defaultUserSelect },
    { path: 'audioRemarks.createdBy', select: defaultUserSelect }
]
/**
 * @description This is used for get list of all Leads
 * @param {*} query 
 * @returns Object { status: true, Leads: Array }
 * - { status: false, error: string }
 */
module.exports.getAllLeads = async (query = {}, startIndex, limit) => {

    return Case.find(query).populate(defaultPopulate).skip(startIndex).sort({ createdAt: -1 }).limit(limit)
        .then((Leads) => {
            return { status: true, Leads: Leads };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, Leads: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    // console.log(" Query => ", query)
    return Case.countDocuments(query)
        .then((Leads) => {
            return { status: true, Leads: Leads };
        }).catch((e) => {
            return { status: false, error: e };
        });

};
/**
 * @description This is used for get One Lead By Id
 * @param {String} id 
 * @returns Object { status: true, Leads: Object }
 * - { status: false, error: string }
 */
module.exports.getLead = async (id) => {

    return Case.findById(id).populate(defaultPopulate).then((Leads) => {
        return { status: true, Leads: Leads };
    }).catch((e) => {
        return { status: false, error: e };
    });
};
/**
 * @description This is used for create Lead
 * @param {Object} leadData 
 * @returns Object { status: true, Leads: Object }
 * - { status: false, error: string }
 */
module.exports.createLead = async (leadData) => {

    // console.log(leadData);
    return Case.create(leadData)
        .then((result) => {
            return { status: true, Leads: result };
        })
        .catch((err) => {
            console.log(err)
            return { status: false, error: err }
        })

};
/**
 * @description This is used for update Lead
 * @param {String} id 
 * @param {Object} leadData 
 * @returns Object { status: true, Leads: Object }
 * - { status: false, error: string }
 */
module.exports.updateLead = async (id, leadData) => {
    try {
        // console.log(leadData)
        // let caseHistory = [];
        // caseHistory = [
        //     {
        //         "id": Math.floor(100000 + Math.random() * 900000),
        //         "title": "Update Lead",
        //         "description": "Update Lead",
        //         "datetime": new Date(),
        //     }
        // ];

        const Lead = await Case.findOneAndUpdate({ _id: id }, leadData, { new: true });

        // const Lead = await Case.findOneAndUpdate({ _id: id }, { $push: { caseHistory: caseHistory }, ...leadData }, { new: true });

        // console.log("usr:"+user);
        if (!Lead) return { status: false, error: "Lead not found" };

        return { status: true, Leads: Lead };

    }
    catch (err) {
        console.log(err);
        return { status: false, error: err };
    }
};
