const ReportTemplate = require('../Model/ReportTemplate');

/**
 * @description This is used for get list of All ReportTemplates
 * @param {*} query 
 * @returns Object { status: true, reportTemplates: Array }
 * - { status: false, error: string }
 */
module.exports.getAllReportTemplates = async (query = {}, limit, startIndex) => {

    return ReportTemplate.find(query).sort({ 'client.name': -1 }).skip(startIndex).limit(limit)
        .then((ReportTemplates) => {
            return { status: true, reportTemplates: ReportTemplates };
        }).catch((e) => {
            return { status: false, error: e };
        });
};

/**
 * @description This is used for get count of documents 
 * @param {*} query 
 * @returns Object { status: true, reportTemplates: Object }
 *  { status: false, error: String }
 */
module.exports.getCount = async (query) => {
    return ReportTemplate.countDocuments(query).then((ReportTemplates) => {
        return { status: true, reportTemplates: ReportTemplates };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for get one ReportTemplate By Id
 * @param {string} id 
 * @returns  Object { status: true, reportTemplates: Object }
 * - { status: false, error: string }
 */
module.exports.getReportTemplate = async (id) => {

    return ReportTemplate.findById(id).then((ReportTemplate) => {
        return { status: true, reportTemplates: ReportTemplate };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for get one ReportTemplate
 * @param {*} query 
 * @returns Object { status: true, reportTemplates: Object }
 * - { status: false, error: string }
 */
module.exports.findReportTemplate = async (query = {}) => {

    return ReportTemplate.findOne(query).then((ReportTemplate) => {
        return { status: true, ReportTemplates: ReportTemplate };
    }).catch((e) => {
        return { status: false, error: e };
    });
};

/**
 * @description This is used for create ReportTemplate
 * @param {Object} ReportTemplateData 
 * @returns Object { status: true, reportTemplates: Object }
 * - { status: false, error: string } 
 */
module.exports.createReportTemplate = async (ReportTemplateData) => {

    return ReportTemplate.create(ReportTemplateData)
        .then((result) => {
            return { status: true, reportTemplates: result };
        })
        .catch((err) => {
            return { status: false, error: err }
        })
};

/**
 * @description This is used for update ReportTemplate
 * @param {String} id 
 * @param {Object} ReportTemplateData 
 * @returns Object { status: true, reportTemplates: Object }
 * - { status: false, error: string } 
 */
module.exports.updateReportTemplate = async (id, ReportTemplateData) => {

    return ReportTemplate.findOneAndUpdate({ _id: id }, ReportTemplateData)
        .then((result) => {
            return { status: true, reportTemplates: result };
        })
        .catch((err) => {
            return { status: false, error: err }
        })

};