const winston = require('winston');
const { combine, timestamp, json } = winston.format;
const ApiLog = require('./models/ApiLog'); // Import the Mongoose model

const logger = winston.createLogger({
    level: 'debug',
    format: combine(
        timestamp({
            format: 'YYYY-MM-DD hh:mm:ss.SSS A',
        }),
        json()
    ),
    defaultMeta: { service: 'user-service' },
    transports: [
        new winston.transports.File({ filename: 'logs/error.log', level: 'error' }),
        new winston.transports.File({ filename: 'logs/combined.log' }),
    ],
});

const cronLogger = winston.createLogger({
    level: 'debug',
    format: combine(
        timestamp({
            format: 'YYYY-MM-DD hh:mm:ss.SSS A',
        }),
        json()
    ),
    defaultMeta: { service: 'user-service' },
    transports: [
        new winston.transports.File({ filename: 'logs/cron-logger.log', level: 'error' }),
        new winston.transports.File({ filename: 'logs/cron-combined.log' }),
    ],
});

if (process.env.NODE_ENV !== 'production') {
    logger.add(new winston.transports.Console({
        format: winston.format.simple(),
    }));
}

const apiLogger = async (req, res, next) => {
    const start = Date.now();

    res.on('finish', async () => {
        try {
            const logEntry = {
                userId: req.local.user ? req.local.user.id : "0",
                userName: req.local.user ? req.local.user.firstName || req.user.lastName || "-" : "Anonymous",
                method: req.method,
                path: req.originalUrl,
                date: new Date(),
                body: req.body,
                query: req.query,
                ipAddress: req.ip,
                userAgent: req.headers['user-agent'],
                statusCode: res.statusCode,
                responseTime: Date.now() - start,
                headers: req.headers,
                requestSize: JSON.stringify(req.body || {}).length,
                responseSize: res.get('content-length') || 0,
                referrer: req.headers['referer'] || req.headers['referrer'] || "Direct",
            };

            // Save log to MongoDB
            await ApiLog.create(logEntry);

            // Also log to file (optional)
            // logger.info(logEntry);
        } catch (error) {
            logger.error("Error logging API request: ", error);
        }
    });

    next();
};

module.exports = { logger, cronLogger }