const express = require('express');
const router = express.Router();

const CONFIG = require('../Constants/Config');
const PaymentController = require('../Controllers/Payment.controller')
const CaseServices = require('../Services/Case.services');
const pagination = require('../Middleware/pagination');
const isAuth = require('../Middleware/isAuth');
const uploadFile = require('../Middleware/uploadFile');
const validation = require('../Middleware/validation');
const isPermission = require('../Middleware/isPermission');


// get case list of prepared invoice and pending payment DropDown
router.get('/pendingPaymentCases', isAuth, pagination, PaymentController.pendingPaymentCaseList);
router.get('/pendingInvoiceCases', isAuth, pagination, PaymentController.pendingInvoiceCaseList);

router.get('/outstanding-payments', isAuth, pagination, PaymentController.outstandingPaymentList);

// get dashboard count for payment
router.get('/payment-count', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, PaymentController.getDashboardPaymentCount);

// CRUD Payments
router.post('/payments', isAuth, uploadFile.uploadPayment, validation.addPayment, PaymentController.addPayment);
router.get('/payments', isAuth, pagination, PaymentController.getAllPayments);
router.get('/payments/:id', isAuth, PaymentController.getPayment);
router.put('/payments/:id/status/:status', isAuth, PaymentController.updatePaymentStatus);
router.delete('/payments/:id', isAuth, PaymentController.deletePayment);

// update invoice/case select, tally voucher and settlement
router.put('/payments/:id', isAuth, uploadFile.uploadPayment, validation.updatePayment, PaymentController.updatePayment);
router.put('/payments/:id/complete', isAuth, PaymentController.completePayment);

router.get('/tds-total', isAuth, PaymentController.tdsCount);
router.get('/tds-list', isAuth, pagination, PaymentController.tdsList);

router.put('/payments/:id/remarks', isAuth, validation.addPayment, PaymentController.addRemark);
router.put('/payments/:id/taxInvoice', isAuth, uploadFile.uploadInvoice, validation.uploadTaxInvoiceValidation, PaymentController.taxInvoiceUpload);

router.put('/payments/reset-case-invoice/:id', isAuth, PaymentController.resetCaseInvoicePayment);

router.get('/update-client-branch-invoice', PaymentController.updateClientBranchInOutstanding);

module.exports = router;