const express = require('express');
const router = express.Router();
const TaskController = require('../Controllers/Task.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const uploadFile = require('../Middleware/uploadFile');
const pagination = require('../Middleware/pagination');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');
const { workType, workTypeArr } = require('../Types/Task');

// For get Tasks count by status
router.get('/task-count', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, TaskController.getDashboardTaskCount);

// For get list of all Tasks
router.get('/tasks', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "list" }) }, pagination, TaskController.getAllTasks);

// For get only One Task
router.get('/tasks/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "view" }) }, TaskController.getTask);

//For create new Task
router.post('/tasks', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "create" }) }, validation.createTask, TaskController.createTaskRequest);

// For update Task
router.put('/tasks/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.createTask, TaskController.updateTask);

// pick task
router.put('/tasks/:id/pick', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, TaskController.pickTask);

router.put('/task/case/:caseId/pick/:workType', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, TaskController.pickByCaseWorkType);

//review Task
router.put('/tasks/:id/review', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, validation.reviewTaskValidation, TaskController.reviewTask);

// update task status
router.put('/tasks/:id/status/:status', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "case", value: "edit" }) }, TaskController.updateTaskStatus);

router.get('/tasks-types', (rea, res) => {
    return res.status(200).send({ status: true, taskWorkType: workTypeArr });
})

router.get('/client-visit/:id', isAuth, TaskController.ClientVisitTask);
router.post('/client-visit', isAuth, validation.clientVisitTask, TaskController.ClientVisitCreateTask);

router.put('/client-visit/:id', isAuth, uploadFile.uploadImages, validation.clientVisitUpdateTask, TaskController.ClientVisitUpdateTask);
router.get('/client-visit', isAuth, pagination, TaskController.ClientVisitTasks);

module.exports = router;