const express = require('express');
const router = express.Router();
const UserController = require('../Controllers/User.controller');
const CreditOfficerController = require('../Controllers/CreditOfficer.controller');
const isAuth = require('../Middleware/isAuth');
const isAdmin = require('../Middleware/isAdmin');
const isPermission = require('../Middleware/isPermission');
const validation = require('../Middleware/validation');
const pagination = require('../Middleware/pagination');
const uploadImage = require('../Middleware/uploadFile');

router.get('/users/taskWorkTypes', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "edit" }) }, UserController.taskWorkTypes);

// For get list of all users 
router.get('/users/', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, pagination, UserController.getAllUsers);

// For get list of all creditOfficer with its case Count
router.get('/creditOfficer', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, pagination, CreditOfficerController.getAllCreditOfficer);

// For get one User
router.get('/users/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "view" }) }, UserController.getUser);

// For create User
router.post('/users', isAuth, isAdmin, uploadImage.userProfile, validation.createUserValidation, UserController.createUser);

// For update User
router.put('/users/:id', isAuth, isAdmin, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "edit" }) }, uploadImage.userProfile, validation.createUserValidation, UserController.updateUser);

// For update User Profile
router.post('/users/profileUpdate', isAuth, uploadImage.userProfile, validation.updateUserProfileValidation, UserController.updateUserProfile);
router.put('/users-active/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "edit" }) }, UserController.modifyUserStatus)

router.delete('/users/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "edit" }) }, UserController.deleteUser);

// For generate Password 
router.get('/genPass/:pass', UserController.generatePassword);

// For change Password
router.post('/changePassword', isAuth, UserController.changePassword);

// For check user already exist  or not i.e. check from Email because Email should be unique
router.post('/users/checkEmailAlreadyExist', isAuth, UserController.checkUserAlreadyExist);

// For check registration Number already exist  or not
router.post('/users/checkRegNoAlreadyExist', isAuth, UserController.checkRegNoAlreadyExist);

// For delete creditOfficer 
router.delete('/deleteCreditOfficer/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "delete" }) }, CreditOfficerController.deleteCreditOfficer);

//For get CreditOfficer Branch History 
router.get('/CreditOfficerBranchHistory/:id', isAuth, (req, res, next) => { return isPermission(req, res, next, { name: "user", value: "list" }) }, CreditOfficerController.getCreditOfficerBranchHistory)

//For get assetHistory details of user
router.get('/users/assetManagement/:id', isAuth, UserController.assetManagement);

router.post('/users/notification_token/:token', isAuth, UserController.notificationToken)

// for get reset password link 
router.post('/resetPasswordLink', UserController.resetPasswordLink);

// for reset password 
router.post('/resetPassword', UserController.resetPassword);

// for reset organization's Admin Password
router.post('/resetAdminPassword', isAuth, UserController.resetAdminPassword);

router.get('/email-suggestions', isAuth, UserController.emailSuggestion);

module.exports = router;

